/****************************** Module Header *******************************
*
* Module Name: newmenu.e
*
* Copyright (c) Netlabs EPM Distribution Project 2002
*
* $Id: newmenu.e 4650 2021-08-26 19:18:07Z aschn $
*
* ===========================================================================
*
* This file is part of the Netlabs EPM Distribution package and is free
* software.  You can redistribute it and/or modify it under the terms of the
* GNU General Public License as published by the Free Software
* Foundation, in version 2 as it comes in the "COPYING" file of the
* Netlabs EPM Distribution.  This library is distributed in the hope that it
* will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
* of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
****************************************************************************/

; This is a replacement for STDMENU.E.

compile if not defined( SMALL)  -- If SMALL not defined, then being separately
   EA_comment 'This defines the menu.'

define
   INCLUDING_FILE = 'NEWMENU.E'

   include 'stdconst.e'
   include 'menuhelp.h'
const
   tryinclude 'mycnf.e'
const
 compile if not defined( NLS_LANGUAGE)
   NLS_LANGUAGE = 'ENGLISH'
 compile endif
   include NLS_LANGUAGE'.e'
compile endif

; ---------------------------------------------------------------------------
const
compile if not defined( CHECK_FOR_LEXAM)
   CHECK_FOR_LEXAM = 0
compile endif
compile if not defined( SUPPORT_USERS_GUIDE)
   SUPPORT_USERS_GUIDE = 1
compile endif
compile if not defined( SUPPORT_TECHREF)
   SUPPORT_TECHREF = 1
compile endif
; Also added to ENGLISH.E
compile if not defined( NEWLINE_KEY__MSG)
   NEWLINE_KEY__MSG = 'Newline'
compile endif

; ---------------------------------------------------------------------------
; Set menu ids (mid) for submenus.
; The menuitem ids (i) will be numbered automatically as followed:
; I.e. mid = 7, i = 701, 702, 703, ..., 799
; The values for mid together with i and ids for accelerator keys must be
; unique.
; We use mid = 0, (1, )..., 79 and i = 101, ..., 7999(, ..., 65535).
;
; The menu ids 1001, ..., 1599 (maybe 1001, ..., 1999) are used for
; accelerator keys. (Accelerator keys cause the message handler to call
; defc processcommand with the id as arg, as well as menuitems.)
;
; The menu ids 44, 45 and 8101 are hardcoded: nextfile, prevfile and
; 'configdlg SYS'. See defc processcommand. The menuid 50 is used by
; ETK_FID_POPUP in MOUSE.E, while mid 80 is free, because it's defined
; with another menuname then defaultmenu.
;
; Don't use mid = 1 if the menuitem attribute should be changed. This is
; usually done by 'processmenuinit', calling 'executemenuinits' with the
; submenuid or menuitemid as arg. Due to a bug in EPM, 'processmenuinit'
; doesn't get called for menuid = 1. Therefore we use mid = 0 instead,
; while keeping the menu item ids i = 101, ..., 199.
;
; All menu ids are saved as array vars, using the procedures SetAVar and
; GetAVar. E.g. using GetAVar( 'mid_file') will be replaced by 2. We can't
; use consts anymore, because a const can't be built of two strings, e.g.
; a stem and a iterated number. Fortunately there's no remarkable
; performance loss.
;
; Submenu ids for packages
;
; default
;    0  stdmenu: Command (new)
;    1  stdmenu: Command (ori)
;    2  stdmenu: File
;    2  newmenu: File
;    3  stdmenu: Search
;    3  newmenu: Search
;    4  stdmenu: Options
;    4  newmenu: Options
;   (5  stdmenu: Ring)
;    5  newmenu: View
;    6  stdmenu: Help
;    6  newmenu: Help
;    7  htmepm99\htmepm.ex: HTML (no src available)
;    7  gcppgp11\gcpgpfe.e: PGP (menu id = 700)
;    8  stdmenu: Edit
;    8  newmenu: Edit
;    9  texfe: TeX/VTeX
;    9  EPMTeX: TeX
;    9  epmsmp\epmprt.e: Printer
;    9  cstepm\epmprt.e: Printer
;   16  bookmark.e: Compile (Workframe)
;   19  epmsmp\errparse.e
;   21  eco: Compile
;   21  epmgcc: GCC
;   21  ebooke\bkeys.e: Ebookie
;   32  epmcomp: Compare!
;   33  epmcomp: Sync!
;   37  cstepm\custepm.e: Actions
;   38  cstepm\gmltags.e: GML
;   39  cstepm\latexmnu.e: LaTeX
;   40  cstepm\sgmltags.e: SGML
;   41  cstepm\htmltags.e: HTML
;   51  newmenu (unique): Mark
;   52  newmenu (unique): Format
;   53  newmenu (unique): Run
;   54  newmenu (unique): Project
;   62  khtepm14\kenhtepm.e: kHTML
;   80  eshtml01\eshtml.e: ES-HTML
;  900  htmlks10\htmlkeys.e: HTML
; 1969  pmcstex: cstex.e (item ids = 6941,...)
;     Further used mids:
;   44  nextfile button
;   45  prevfile button
;   80  popup
; 1001...1599...1999  accelerator keys  <-- old, unused
; 8101  configdlg SYS                   <-- old, unused
;10001...65534        accelerator keys  <-- new
;65535  Halt command

; ---------------------------------------------------------------------------
defexit
   -- Delete all submenu mids and names and the list of all used mids
   -- The menu item ids are not deleted, because there exists no list of
   -- defined ones yet. But that has no negative effect, because any new
   -- menu definition simply overrides and re-uses the old ids.
   'ResetMids'

; Because the menu is linked at definit, this definit is executed immediately
; after linking, before the rest of definit in INIT.E. Therefore the
; EPM_utility_array_id must already exist at this point.
definit
   -- Sometimes the rc for a module's definit overrides the link rc.
   -- Therefore a linkable module with code in definit that changes rc,
   -- should save it at the begin of definit and restore it at the end.
   save_rc = rc
   'InitMenu'
   rc = save_rc  -- don't change rc of the link statement by definit code

defc InitMenu
   call SetAVar( 'submenunames', 'file edit mark format search view run help')

   -- Define here some submenu ids for compatibility.
   -- If a submenu id is not defined, DefSubMenu calls GetUniqueMid to get
   -- a new unique one.
   call SetAVar( 'mid_file'   , 2)
   call SetAVar( 'mid_edit'   , 8)
;  call SetAVar( 'mid_mark'   , GetUniqueMid())  -- first available mid is 51 (50 is used by popup menu)
;  call SetAVar( 'mid_format' , GetUniqueMid())  -- second available mid is 52
   call SetAVar( 'mid_search' , 3)
   call SetAVar( 'mid_view'   , 5)
;  call SetAVar( 'mid_options', 4)
;  call SetAVar( 'mid_run'    , GetUniqueMid())  -- mid 0 would crash PM on refresh, use 53
;  call SetAVar( 'mid_project', GetUniqueMid())  -- submenu replaced by the current selected project's submenu, e.g. 'TeX'
   call SetAVar( 'mid_help'   , 6)  -- 6 should not be changed to not break other packages

   -- Reset the word list of used menu accelerators
   call SetAVar( 'usedmenuaccelerators', '')

   'InitMenuSettings'
   call CheckTool( 'uconv')
   call CheckTool( 'recode')
   call CheckTool( 'grep')
   call CheckTool( 'compare1')
   call CheckTool( 'compare2')
   'CheckWps'

   -- Refresh the Format submenu, if required, for Reflow menu items
   'HookAdd fileswitched RefreshFormatMenu'

; ---------------------------------------------------------------------------
; Better don't use NepmdQueryConfigValue at definit, because this can cause
; stop of the definit processing sometimes. Executing this from a command is
; processed delayed, after definit is completed.
defc InitMenuSettings
   universal nodismiss

   KeyPath = '\NEPMD\User\Menu\NoDismiss'
   on = (QueryConfigKey( KeyPath) = 1)
   --if on then
   --   nodismiss = 'NODISMISS'
   --else
   --   NoDisMiss = ''
   --endif
   nodismiss = 32 * on

; ---------------------------------------------------------------------------
; Called by defmain -> initconfig, STDCTRL.E (formerly by definit, MENUACCEL.E).
/*
Ŀ
What's it called  : loaddefaultmenu                                          
                                                                             
What does it do   : used by init.e to setup default EPM menu bar             
                    (Note: a menu id of 0 halts the interpreter when         
                     selected.)                                              
                    Really? The id, that doesn't work, is 1, not 0!          
                                                                             
Who and When      : Jerry C.     2/25/89                                     

*/

defc LoadDefaultMenu
   universal activemenu
   universal defaultmenu
   universal menuloaded                   -- for to check if menu is already built

   parse arg menuname .
   if menuname = '' then                  -- Initialization call
      menuname = 'default'
      defaultmenu = menuname              -- default menu name
      activemenu  = defaultmenu
   endif

   call Add_File_Menu()      -- id = 2
   call Add_Edit_Menu()      -- id = 8
   call Add_Mark_Menu()      -- id = ?
   call Add_Format_Menu()    -- id = ?
   call Add_Search_Menu()    -- id = 3
   call Add_View_Menu()      -- id = 5
;   call Add_Command_Menu()  -- replaced with add_run_menu
   call Add_Run_Menu()       -- id = 0 (menuitem ids = 1xxx)
;   call Add_Project_Menu()  -- id = 9 (= TeX, epmprt)

   -- Process hook: add a user-defined submenu
   if isadefc( 'HookExecute') then
      'HookExecute addmenu'
   endif

   call Add_Help_Menu()      -- id = 6 (keep this)
   -- Note: showmenu_activemenu() must be called separately.
   menuloaded = 1

; -------------------------------------------------------------------------------------- File -------------------------
defc AddFileMenu
   call Add_File_Menu()

defproc Add_File_Menu
   universal nodismiss
   universal wpsstarted

   DefSubMenu( 'file',                                                  -- File ------------
               FILE_BAR__MSG,
               FILE_BARP__MSG,
               HP_FILE)

   DefMenuItem( '',                                                           -- New
                'N~ew',
                'NewUntitledFile',
                'Create a new, empty file in this window',
                HP_FILE_EDIT)
   DefMenuItem( '',                                                           -- Open...
                '~Open...',
                'EditFileDlg',
                ADD_MENUP__MSG,
                HP_FILE_EDIT)
   DefMenuSep()                                                               --------------------

   DefMenuItem( '',                                                           -- New window
                'New ~window',
                'Open',
                OPEN_NEW_MENUP__MSG,
                HP_FILE_OPEN_NEW)
   DefMenuItem( '',                                                           -- Open bin...
                'Open ~bin...',
                'OpenBinDlg',
                'Select a binary file to edit in a new window',
                '')
   DefMenuSep()                                                               --------------------

   DefNextItemStartsSubMenu()
   DefMenuItem( 'history',                                                    -- History   >
                '~History',
                '',
                'Edit previously loaded files',
                '')
   DefMenuItem( '',                                                                 -- List edit history...
                'List ~edit history...',
                'History edit',
                'Open a list box with previous edit cmds',
                '')
   DefMenuItem( '',                                                                 -- List loaded files...
                'List ~loaded files...',
                'History load',
                'Open a list box with previous loaded files',
                '')
   DefMenuItem( '',                                                                 -- List saved files...
                'List ~saved files...',
                'History save',
                'Open a list box with previous saved files',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Save all rings as last
                'Save all ~rings as last',
                'SaveLastRing',
                'Save this ring and those of all other windows to Last config tree',
                '')
   DefMenuItem( '',                                                                 -- Load all saved last rings
                '~Load all saved last rings',
                'RestoreLastRing',
                'Restore all saved rings from Last config tree',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Save all rings as file list (.epm)
                'Save all rings as ~file list (.epm)...',
                'SaveFileList',
                'Save file lists of all EPM rings as .epm file',
                '')
   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 -- Load all rings from file list (.epm)
                'L~oad all rings from file list (.epm)...',
                'OpenFileListDlg',
                'Select a previously saved file list for to load files from',
                '')

if wpsstarted then
   DefMenuItem( 'openfolder',                                                 -- Open folder
                'Open ~folder',
                'OpenFolder OPEN=DEFAULT',
                'Open WPS folder where the current file is located in its default view',
                '')
endif
/*
   DefMenuItem( '',                                                           -- FTP   >
                'F~TP',
                '',
                'Download or upload file from/to FTP server',
                '',
                'DISABLED')
*/
   DefMenuSep()                                                               --------------------

   DefMenuItem( '',                                                           -- Open extracted string
                'Open e~xtracted string',
                'Alt_1',
                'Load file or URL under cursor',
                '')
   DefMenuItem( 'importfile',                                                 -- Import file...
                '~Import file...',
                'OpenDlg get',
                GET_MENUP__MSG,
                HP_FILE_GET)
   DefMenuItem( '',                                                           -- Reload
                '~Reload',
                'Revert',
                'Reload file from disk, ask if modified',
                '')
   DefMenuSep()                                                               --------------------

   DefNextItemStartsSubMenu()
   DefMenuItem( 'fileproperties',                                             -- File properties   >
                'File ~properties',
                '',
                'Properties for this buffer/file only')
                -- Added nodismiss to submenu items only. This is required for the submenu item 'Close menu' to work.
   DefMenuItem( 'mode',                                                             -- Mode...
                '~Mode: %1...',
                'Mode',
                'Select or show mode for the current file',
                '',
                nodismiss)
   DefMenuItem( '',                                                                 -- Show file and mode settings
                'Show ~file and mode settings',
                'ShowFileSettings',
                'Display a virtual file with file and mode settings for all files',
                '',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'tabs',                                                             -- Tabs...
                '~Tabs: %1...',
                'Tabs',
                'Select or show tabs for the current file',
                '',
                nodismiss)
   DefMenuItem( 'margins',                                                          -- Margins...
                'Mar~gins: %1...',
                'Ma',
                'Select or show margins for the current file',
                '',
                nodismiss)
   DefMenuItem( 'autospellcheck',                                                   -- Auto-spellcheck
                DYNASPELL_MENU__MSG,
                'Toggle_Dynaspell',
                DYNASPELL_MENUP__MSG,
                HP_OPTIONS_DYNASPELL)
                -- nodismiss doesn't work here because of RefreshMenu, called by SetKeyset2
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'readonlyattrib',                                                   -- Read-only file attribute
                'Read-~only file attribute',
                'Toggle_Readonly_Attrib',
                'Set or reset read-only file attribute',
                '',
                nodismiss)
   DefMenuItem( 'readonly',                                                         -- Read-only mode
                '~Read-only mode',
                'Toggle_Readonly',
                'Enable or disable read-only mode',
                '',
                nodismiss)
if nodismiss = 0 & not wpsstarted then
   DefNextItemEndsSubMenu()
endif
   DefMenuItem( 'locked',                                                           -- Locked
                '~Locked',
                'Toggle_Locked',
                'Enable or disable write access for other apps',
                '',
                nodismiss)
if wpsstarted then
   if nodismiss = 0 then
      DefNextItemEndsSubMenu()
   endif
   DefMenuItem( 'wpsproperties',                                                    -- WPS properties...
                '~WPS properties...',
                'OpenSettings',
                'Open WPS properties dialog for current file')
endif

if nodismiss > 0 then
   DefMenuSep()                                                                     --------------------

   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 -- Close menu
                '~Close menu',
                '',
                '',
                '')
endif
   DefMenuItem( '',                                                           -- Rename...
                'Re~name...',
                'Rename',
                RENAME_MENUP__MSG,
                HP_FILE_NAME)
   DefMenuItem( '',                                                           -- Copy filename
                'Cop~y filename',
                'CopyFilename2Clip',
                'Copy current filename to clipboard',
                '')
   DefMenuSep()                                                               --------------------

   DefMenuItem( 'save',                                                       -- Save
                SAVE_MENU__MSG,
                'Save',
                SAVE_MENUP__MSG,
                HP_FILE_SAVE)
   DefMenuItem( '',                                                           -- Save as...
                SAVEAS_MENU__MSG,
                'Saveas_Dlg',
                SAVEAS_MENUP__MSG,
                HP_FILE_SAVEAS)
   DefMenuItem( 'saveandquit',                                                -- Save and close file
                'Sa~ve and close file',
                'File',
                'Save this file, then close it',
                HP_FILE_FILE)
   DefMenuItem( '',                                                           -- Save loaded files
                'Save ~loaded files',
                'SaveAll',
                'Save all modified files in the edit ring',
                '')
   DefMenuSep()                                                               --------------------

   DefMenuItem( '',                                                           -- Print file...
                PRT_FILE_MENU__MSG,
                'PrintDlg',
                ENHPRT_FILE_MENUP__MSG,
                HP_FILE_ENHPRINT)
   DefMenuItem( 'printmark',                                                  -- Print mark...
                'Print ~mark...',
                'PrintDlg M',
                ENHPRT_MARK_MENUP__MSG,
                HP_EDIT_ENHPRINT)
   DefMenuSep()                                                               --------------------

   DefMenuItem( '',                                                           -- Close file
                '~Close file',
                'Quit',
                'Close this file',
                HP_FILE_QUIT)
   DefMenuSep()                                                               --------------------

   DefMenuItem( '',                                                           -- Restart
                'Res~tart',
                'Restart',
                'Restart all EPM windows',
                '')
   return

; -------------------------------------------------------------------------------------- Edit -------------------------
defc AddEditMenu
   call Add_Edit_Menu()

defproc Add_Edit_Menu
   universal nodismiss

   DefSubMenu( 'edit',                                                  -- Edit ------------
               '~Edit',
               'Menus related to edit operations',
               '')

   DefMenuItem( 'undoline',                                                   -- Undo line
                UNDO_MENU__MSG,
                'UndoLine',
                UNDO_MENUP__MSG,
                HP_EDIT_UNDO)
   DefMenuItem( 'undo',                                                       -- Undo...
                UNDO_REDO_MENU__MSG,
                'UndoDlg',
                UNDO_REDO_MENUP__MSG,
                HP_EDIT_UNDOREDO)
   DefMenuItem( 'undolast',                                                   -- Undo last
                'Undo las~t',
                'Undo1MenuCmd',
                'Go 1 step back in the change history',
                HP_EDIT_UNDOREDO,
                nodismiss)
   DefMenuItem( 'redonext',                                                   -- Redo next
                '~Redo next',
                'Redo1MenuCmd',
                'Go 1 step forth in the change history',
                HP_EDIT_UNDOREDO,
                nodismiss)
   DefMenuItem( 'undokeepscrollpos',                                          -- Keep scroll position
                'Keep scroll position',
                'ToggleUndoScrollPos',
                'When active, the visible part of a file doesn''t move during undo and redo',
                '',
                nodismiss)
   DefMenuSep()                                                               --------------------

   DefMenuItem( 'recovermarkdelete',                                          -- Recover mark delete
                RECOVER_MARK_MENU__MSG,
                'GetDMBuff',
                RECOVER_MARK_MENUP__MSG,
                HP_EDIT_RECOVER)
   DefMenuItem( 'discardchanges',                                             -- Discard changes
                '~Discard changes',
                'DiscardChanges',
                'Reset modified state',
                '')
   DefMenuItem( 'backuplistfiles',                                            -- List backups for current file
                'List backups for current ~file',
                'ListBackupDirCurrentFile',
                '',
                '')
   DefMenuSep()                                                               --------------------

   DefMenuItem( '',                                                           -- Duplicate line
                'Dupli~cate line',
                'DuplicateLine',
                'Duplicate current line (insert below)',
                '')
   DefNextItemStartsSubMenu()
   DefMenuItem( 'insert',                                                     -- Insert   >
                '~Insert',
                '',
                'Insert text',
                '')
   DefMenuItem( '',                                                                 -- New line after
                '~New line after',
                'NewLineAfter',
                'Insert empty line after current',
                '',
                nodismiss)
   DefMenuItem( '',                                                                 -- New line before
                'New ~line before',
                'NewLineBefore',
                'Insert empty line before current',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Char from above
                '~Char from above',
                'InsertCharAbove',
                'Copy char above to cursor position',
                '',
                nodismiss)
   DefMenuItem( '',                                                                 -- Char from below
                'Char from ~below',
                'InsertCharBelow',
                'Copy char below to cursor position',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'insertsum',                                                        -- Sum of numbers in block
                '~Sum of numbers in block',
                'Add',
                'Insert sum of block-marked values below the mark',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Filename
                '~Filename',
                'TypeFilename',
                'Insert current filename at cursor position',
                '')
   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 -- Date Time (ISO)
                '~Date Time (ISO)',
                'TypeDateTime',
                'Insert current date and time at cursor position',
                '')
   DefNextItemStartsSubMenu()
   DefMenuItem( 'move',                                                       -- Move   >
                'Mo~ve',
                '',
                'Move current text',
                '',
                nodismiss)
   DefMenuItem( '',                                                                 -- Line up
                'Line ~up',
                'MoveLineUp',
                'Move current line up',
                '')
   DefMenuItem( '',                                                                 -- Line down
                'Line ~down',
                'MoveLineDown',
                'Move current line down',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Char left
                'Char ~left',
                'MoveCharLeft',
                'Move current char left',
                '')
   DefMenuItem( '',                                                                 -- Char right
                'Char ~right',
                'MoveCharRight',
                'Move current char right',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'moveexpressionleft',                                               -- Expression/Marked expression left
                '|E~xpression left|Marked e~xpression left',
                'MoveExpressionLeft',
                'Move expression under cursor left',
                '')
   DefNextItemEndsSubMenu()
   DefMenuItem( 'moveexpressionright',                                              -- Expression/Marked expression right
                '|~Expression right|Marked ~expression right',
                'MoveExpressionRight',
                'Move expression under cursor right',
                '')
   DefMenuSep()                                                               --------------------

   DefMenuItem( '',                                                           -- Delete line
                'D~elete line',
                'DeleteLine',
                'Delete current line',
                '')
   DefMenuItem( '',                                                           -- Delete rest of line
                'Delete rest of li~ne',
                'DeleteUntilEndLine',
                'Delete from cursor until end of line',
                '')
   DefMenuItem( '',                                                           -- Delete up to next word
                'Delete up to ne~xt word',
                'DeleteUntilNextWord',
                'Delete from cursor until begin of next word',
                '')
   DefMenuSep()                                                               --------------------

   DefMenuItem( '',                                                           -- Show file and mode settings
                'Sho~w file and mode settings',
                'ShowFileSettings',
                'Display a virtual file with file and mode settings for all files',
                '',
                '')
   DefMenuItem( 'preferences',                                                -- Preferences...
                'Pre~ferences...',
                'PrefsPopupMenu',
                'Menus related to editor settings',
                '')
if nodismiss > 0 then
   DefMenuSep()                                                               --------------------

   DefMenuItem( '',                                                           -- Close menu
                'Close men~u',
                '',
                '',
                '')
endif

   return


; -------------------------------------------------------------------------------------- Mark -------------------------
defc AddMarkMenu
   call Add_Mark_Menu()

defproc Add_Mark_Menu
   universal nodismiss

   DefSubMenu( 'mark',                                                  -- Mark ------------
               '~Mark',
               'Menus related to basic mark operations',
               '')

   DefNextItemStartsSubMenu()
   DefMenuItem( 'createmark',                                                 -- Create mark   >
                'Create mar~k',
                '',
                'Mark text at cursor position in different ways',
                '')
   DefMenuItem( 'markchars',                                                        -- Mark chars
                'Mark ~chars',
                'MarkChar',
                'Create a char mark between two cursor positions',
                '')
   DefMenuItem( 'markblock',                                                        -- Mark block
                'Mark ~block',
                'MarkBlock',
                'Create a block mark between two cursor positions',
                '')
   DefMenuItem( 'marklines',                                                        -- Mark lines
                'Mark ~lines',
                'MarkLine',
                'Create a line mark between two cursor positions',
                '')
   DefMenuItem( 'marktoken',                                                        -- Mark identifier
                'Mark ~identifier',
                'MarkToken',
                'Mark identifier (C-style word) under cursor',
                '')
   DefMenuItem( 'markword',                                                         -- Mark word
                'Mark ~word',
                'MarkWord',
                'Mark word under cursor',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'marklinesetatcursor',                                              -- Mark line set at cursor
                'Mark line se~t at cursor',
                'MarkLineSetAtCursor',
                'Mark lines with same or more indent, starting at current line',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'marksentence',                                                     -- Mark sentence
                'Mark ~sentence',
                'MarkSentence',
                'Mark sentence around mouse pointer',
                '')
   DefMenuItem( 'extendsentence',                                                   -- Extend sentence
                '~Extend sentence',
                'ExtendSentence',
                'Extend character mark through end of next sentence',
                '')
   DefMenuItem( 'markparagraph',                                                    -- Mark paragraph
                'Mark ~paragraph',
                'MarkParagraph',
                'Mark paragraph around mouse pointer',
                '')
   DefNextItemEndsSubMenu()
   DefMenuItem( 'extendparagraph',                                                  -- Extend paragraph
                'E~xtend paragraph',
                'ExtendParagraph',
                'Extend character mark through end of next paragraph',
                '')
   DefMenuSep()                                                               --------------------

   DefMenuItem( 'copy',                                                       -- Copy
                '~Copy',
                'Copy2Clip',
                CLIP_COPY_MENUP__MSG,
                HP_EDIT_COPY)
   DefMenuItem( 'cut',                                                        -- Cut
                'Cu~t',
                'Cut',
                CUT_MENUP__MSG,
                HP_EDIT_CUT)

   -- Keep in sync with DefaultPast and AlternatePaste
   KeyPath = '\NEPMD\User\Mark\DefaultPaste'
   next = substr( upcase( QueryConfigKey( KeyPath)), 1, 1)
   if next = 'L' then
      style    = 'L'
      altstyle = 'C'
   elseif next = 'B' then
      style    = 'B'
      altstyle = 'C'
   else
      style    = 'C'
      altstyle = 'L'
   endif
   if style = 'C' then
      PasteCMenuAccelString = MenuAccelString( 'DefaultPaste')
   elseif altstyle = 'C' then
      PasteCMenuAccelString = MenuAccelString( 'AlternatePaste')
   else
      PasteCMenuAccelString = ''
   endif
   if style = 'L' then
      PasteLMenuAccelString = MenuAccelString( 'DefaultPaste')
   elseif altstyle = 'L' then
      PasteLMenuAccelString = MenuAccelString( 'AlternatePaste')
   else
      PasteLMenuAccelString = ''
   endif
   if style = 'B' then
      PasteBMenuAccelString = MenuAccelString( 'DefaultPaste')
   elseif altstyle = 'B' then
      PasteBMenuAccelString = MenuAccelString( 'AlternatePaste')
   else
      PasteBMenuAccelString = ''
   endif

   DefMenuItem( 'paste',                                                      -- Paste
                '~Paste'PasteCMenuAccelString,
                'Paste C',
                PASTE_C_MENUP__MSG,
                HP_EDIT_PASTEC)
   DefMenuItem( 'pastelines',                                                 -- Paste lines
                PASTE_L_MENU__MSG''PasteLMenuAccelString,
                'Paste',
                PASTE_L_MENUP__MSG,
                HP_EDIT_PASTE)
   DefMenuItem( 'pasteblock',                                                 -- Paste block
                PASTE_B_MENU__MSG''PasteBMenuAccelString,
                'Paste B',
                PASTE_B_MENUP__MSG,
                HP_EDIT_PASTEB)
   DefMenuSep()                                                               --------------------

   DefMenuItem( 'advancedmarking3',                                           -- Advanced marking
                '~Advanced marking',
                'toggle_cua_mark',  -- nodismiss doesn't work here, because RefreshMenu is executed for hint texts
                ADVANCEDMARK_MENUP__MSG,
                HP_OPTIONS_CUATOGGLE)
   DefMenuItem( 'copymark',                                                   -- Copy mark
                COPY_MARK_MENU__MSG,
                'CopyMark',
                COPY_MARK_MENUP__MSG,
                HP_EDIT_COPYMARK)
   DefMenuItem( 'movemark',                                                   -- Move mark
                MOVE_MARK_MENU__MSG,
                'MoveMark',
                MOVE_MARK_MENUP__MSG,
                HP_EDIT_MOVE)
   DefMenuItem( 'overlaymark',                                                -- Overlay mark
                OVERLAY_MARK_MENU__MSG,
                'OverLayMark',
                OVERLAY_MARK_MENUP__MSG,
                HP_EDIT_OVERLAY)
   DefMenuItem( 'adjustmark',                                                 -- Adjust mark
                ADJUST_MARK_MENU__MSG,
                'AdjustMark',
                ADJUST_MARK_MENUP__MSG,
                HP_EDIT_ADJUST)
   DefMenuSep()                                                               --------------------

   DefMenuItem( '',                                                           -- Select all
                SELECT_ALL_MENU__MSG,
                'Select_All',
                SELECT_ALL_MENUP__MSG,
                HP_EDIT_SELECTALL)
   DefMenuItem( 'unmark',                                                     -- Unmark
                UNMARK_MARK_MENU__MSG,
                'UnMark',
                UNMARK_MARK_MENUP__MSG,
                HP_EDIT_UNMARK)
   DefMenuItem( 'deletemark',                                                 -- Delete mark
                DELETE_MARK_MENU__MSG,
                'DeleteMark',
                DELETE_MARK_MENUP__MSG,
                HP_EDIT_DELETE)
   DefMenuItem( 'fillmark',                                                   -- Fill mark...
                '~Fill mark...',
                'FillMark',
                'Fill mark with a char',
                '')

   return


; -------------------------------------------------------------------------------------- Format -----------------------
defc AddFormatMenu
   call Add_Format_Menu()

defproc Add_Format_Menu
   universal nodismiss
compile if CHECK_FOR_LEXAM
   universal LEXAM_is_available
compile endif

   DefSubMenu( 'format',                                                -- Format ----------
               'Form~at',
               'Menus related to formatting operations',
               '')

   DefNextItemStartsSubMenu()
   DefMenuItem( 'lineendchars',                                               -- Line end chars   >
                '~Line end chars',
                '',
                '',
                '',
                nodismiss)
   DefMenuItem( 'changelineend',                                                    -- Change %1 to
                '~Change %1 to',
                'ChangeFileLineEnd',
                'Change line ends to selected option',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Options
                'Options',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'doslineend',                                                       -- DOS (CRLF)
                '~DOS (CRLF)',
                'ToggleLineEndOption DOS',
                'Select DOS line end',
                '',
                nodismiss)
   DefMenuItem( 'unixlineend',                                                      -- Unix (LF)
                '~Unix (LF)',
                'ToggleLineEndOption Unix',
                'Select Unix line end',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefNextItemEndsSubMenu()
   DefMenuItem( 'format_saveoptions_lineend',                                       -- Save: %1 will be saved as %2...
                '~Save: %1 will be saved as %2...',
                'PrefsPopupMenu',
                'Use Preferences to change default save settings',
                '')

   DefNextItemStartsSubMenu()
   DefMenuItem( 'emptylines',                                                 -- Empty lines   >
                '~Empty lines',
                '',
                '',
                '',
                nodismiss)
   DefMenuItem( 'removeemptylines',                                                 -- Remove empty lines
                '|R~emove empty lines|~Ensure empty lines',
                'RemoveEmptyLinesMenuCmd',
                'Remove or add empty lines according to selected options',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Options
                'Options',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'emptylinesmark',                                                   -- Marked lines
                '~Marked lines',
                'ToggleEmptyLinesOption mark',
                'Process marked lines or the entire file',
                '',
                nodismiss)
   DefMenuItem( 'emptylinesmulti',                                                  -- Multiple marked lines
                'M~ultiple empty lines',
                'ToggleEmptyLinesOption multi',
                'Compress multiple empty lines to one',
                '',
                nodismiss)
   DefMenuItem( 'emptylinesall',                                                    -- All empty lines
                '~All empty lines',
                'ToggleEmptyLinesOption all',
                'Remove all empty lines',
                '',
                nodismiss)
   DefMenuItem( 'emptylinesreadd',                                                  -- Ensure empty lines after '.' and ':'
                "Ensu~re empty lines after '.' and ':'",
                'ToggleEmptyLinesOption add',
                'Ensure that an empty line exists after a trailing dot or a colon or at the end',
                '',
                nodismiss)
   DefNextItemEndsSubMenu()
   DefMenuItem( 'emptylinessingle',                                                 -- Singlespace maxro
                '~Singlespace macro',
                'ToggleEmptyLinesOption single',
                'Remove duplicated line ends',
                '',
                nodismiss)
   DefNextItemStartsSubMenu()
   DefMenuItem( 'stripblanks',                                                -- Strip blanks   >
                '~Strip blanks',
                '',
                '',
                '',
                nodismiss)
   DefMenuItem( 'striptrailingblanks',                                              -- Strip trailing blanks
                'Strip ~trailing blanks',
                'StripBlanksFromLinesMenuCmd',
                'Strip trailing spaces and tab chars from lines according to selected options',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Options
                'Options',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'stripblanksmark',                                                  -- Mark
                '~Mark',
                'ToggleStripBlanksOption mark',
                'Process marked lines or the entire file',
                '',
                nodismiss)
   DefMenuItem( 'stripblanksleading',                                                -- Include leading blanks
                'Include ~leading blanks',
                'ToggleStripBlanksOption leading',
                '',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefNextItemEndsSubMenu()
   DefMenuItem( 'format_saveoptions_striptrailingspaces',                           -- Save: trailing spaces will | will not be stripped...
                'Save: trailing spaces %1 be stripped...',
                'PrefsPopupMenu',
                'Use Preferences to change default save settings',
                '')
   DefNextItemStartsSubMenu()
   DefMenuItem( 'tabsspaces',                                                 -- Tabs/spaces   >
                '~Tabs/spaces',
                '',
                '',
                '',
                nodismiss)
   DefMenuItem( 'expandtabstospaces',                                               -- Expand tabs to spaces...
                '~Expand tabs to spaces...',
                'TabExpandMenuCmd',
                'Process according to selected options',
                '')
   DefMenuItem( 'compressspacestotabs',                                             -- Compress spaces to tabs...
                '~Compress spaces to tabs...',
                'TabCompressMenuCmd',
                'Process according to selected options; use this instead of "Save /t"',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Options
                'Options',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'tabsspacestabwidth',                                               -- Tab width for conversion
                'Tab width for conversion: %1...',
                'SelectTabsSpacesTabWidth',
                'Select tab width, starts with current tabs value, file-specific',
                '',
                nodismiss)
   DefMenuItem( 'tabsspacesmark',                                                   -- Mark
                '~Mark',
                'ToggleTabsSpacesOption mark',
                'Process mark or the entire file',
                '',
                nodismiss)
   DefMenuItem( 'tabsspacesindent',                                                 -- Compress indent only
                'Compress ~indent only',
                'ToggleTabsSpacesOption indent',
                'When active, tab compression processes the line indents only',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefNextItemEndsSubMenu()
   DefMenuItem( 'format_saveoptions_spaces2tabs',                                   -- Save: spaces will | will not be compressed to tabs...
                'Save: spaces %1 be compressed to tabs...',
                'PrefsPopupMenu',
                'Use Preferences to change default save settings',
                '')
   DefMenuSep()                                                               --------------------

   DefNextItemStartsSubMenu()
   DefMenuItem( 'indent',                                                     -- Indent   >
                '~Indent',
                '',
                '',
                '',
                nodismiss)
   DefMenuItem( 'indentlines',                                                      -- Indent lines
                '~Indent lines',
                'IndentBlockMenuCmd',
                'Indent line set at cursor or marked lines by 1 level',
                '')
   DefMenuItem( 'unindentlines',                                                    -- Unindent lines
                '~Unindent lines',
                'IndentBlockMenuCmd U',
                'Unindent line set at cursor or marked lines by 1 level',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Options
                'Options',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'indentmark',                                                       -- Marked lines
                '~Marked lines',
                'ToggleIndentOption mark',
                'Process marked lines',
                '',
                nodismiss)
   DefMenuItem( 'indentlineset',                                                    -- Line set at cursor
                'Line ~set at cursor',
                'ToggleIndentOption lineset',
                'Process lines with same or more indent, starting at current line',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'shiftleft',                                                        -- Shift left
                'Shift mark ~left',
                'ShiftLeft',
                'Shift marked lines left by 1 character',
                '')
   DefNextItemEndsSubMenu()
   DefMenuItem( 'shiftright',                                                       -- Shift right
                'Shift mark ~right',
                'ShiftRight',
                'Shift marked lines right by 1 character',
                '')
   DefNextItemStartsSubMenu()
   DefMenuItem( 'comment',                                                    -- Comment  >
                '~Comment',
                '',
                '',
                '',
                nodismiss)
   DefMenuItem( 'commentlines',                                                     -- Comment lines
                '~Comment lines',
                'CommentMenuCmd',
                'Comment line set at cursor or marked lines',
                '')
   DefMenuItem( 'uncommentlines',                                                   -- Uncomment lines
                '~Uncomment lines',
                'UnCommentMenuCmd',
                'Remove comment chars for line set at cursor or marked lines',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Options
                'Options',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'commentmark',                                                      -- Marked lines
                '~Marked lines',
                'ToggleCommentOption mark',
                'Process marked lines',
                '',
                nodismiss)
   DefNextItemEndsSubMenu()
   DefMenuItem( 'commentlineset',                                                   -- Line set at cursor
                'Line ~set at cursor',
                'ToggleCommentOption lineset',
                'Process lines with same or more indent, starting at current line',
                '',
                nodismiss)
   DefNextItemStartsSubMenu()
   DefMenuItem( 'doublelines',                                                -- Double lines   >
                '~Double lines',
                '',
                '',
                '',
                nodismiss)
   DefMenuItem( 'removedoublelines',                                                 -- Remove double lines
                'Remove ~double lines',
                'RemoveDoubleLinesMenuCmd',
                'Remove all double lines, except empty lines',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Option
                'Option',
                '',
                '',
                '',
                'STATIC')
   DefNextItemEndsSubMenu()
   DefMenuItem( 'doublelinesmark',                                                  -- Marked lines
                '~Marked lines',
                'ToggleDoubleLinesOption mark',
                'Process marked lines or the entire file',
                '',
                nodismiss)
   DefNextItemStartsSubMenu()
   DefMenuItem( 'sort',                                                       -- Sort  >
                'S~ort',
                '',
                '',
                '',
                nodismiss)
   DefMenuItem( 'sortlines',                                                        -- Sort lines
                '~Sort lines',
                'SortMenuCmd',
                'Sort marked lines or entire file',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Options
                'Options',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'sortmark',                                                         -- Marked lines
                '~Marked lines',
                'ToggleSortOption mark',
                'Process mark or the entire file',
                '',
                '',
                nodismiss)
   DefMenuItem( 'sortdescending',                                                   -- Descending
                '~Descending',
                'ToggleSortOption descending',
                'Sort in reverse order',
                '',
                nodismiss)
   DefMenuItem( 'sortignorecase',                                                   -- Ignore case
                '~Ignore case',
                'ToggleSortOption ignorecase',
                'Sort ignoring case',
                '',
                nodismiss)
   DefNextItemEndsSubMenu()
   DefMenuItem( 'sortcollatingorder',                                               -- Collating order
                '~Collating order',
                'ToggleSortOption collatingorder',
                'Sort according to country and codepage settings',
                '',
                nodismiss)
   DefMenuSep()                                                               --------------------

   DefNextItemStartsSubMenu()
   DefMenuItem( 'case',                                                       -- Case   >
                'C~ase',
                '',
                'Change case of text',
                '',
                nodismiss)
   DefMenuItem( '',                                                                 -- Char toggle
                '~Char toggle',
                'CaseChar',
                'Toggle case of char under cursor, move right',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Word toggle
                'Word ~toggle',
                'CaseWord',
                'Toggle word through mixed, upper and lower cases',
                '')
   DefMenuItem( '',                                                                 -- Word uppercase
                'Word ~uppercase',
                'UppercaseWord',
                'Change word to uppercase',
                '')
   DefMenuItem( '',                                                                 -- Word lowercase
                'Word ~lowercase',
                'LowercaseWord',
                'Change word to uppercase',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'markuppercase',                                                    -- Mark uppercase
                'Mark u~ppercase',
                'UppercaseMark',
                'Change mark to uppercase',
                '')
   DefNextItemEndsSubMenu()
   DefMenuItem( 'marklowercase',                                                    -- Mark lowercase
                'Mark l~owercase',
                'LowercaseMark',
                'Change mark to lowercase',
                '')
   DefNextItemStartsSubMenu()
   DefMenuItem( 'encoding',                                                   -- Encoding   >
                '~Encoding',
                '',
                '',
                '',
                nodismiss)
   DefMenuItem( 'encodefrom',                                                       -- From %1 to SystemEncoding
                '~From %1 to %2',
                'EncodeFrom',
                'Encode current file to the system codepage with the selected tool',
                '',
                nodismiss)
   DefMenuItem( 'encodeto',                                                         -- From SystemEncoding to %2
                'From %1 ~to %2',
                'EncodeTo',
                'Encode current file from the system codepage with the selected tool',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Options
                'Options',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'useuconv',                                                         -- Use uconv.exe
                'Use ~uconv.exe',
                'SwitchEncodingTool uconv',
                'Uconv.exe knows more encodings. For a list: unconv --list',
                '',
                nodismiss)
   DefMenuItem( 'userecode',                                                        -- Use recode.exe
                'Use ~recode.exe',
                'SwitchEncodingTool recode',
                'Recode.exe doesn''t support Unicode. For a list: recode --list',
                '',
                nodismiss)
   DefMenuItem( 'switchencoding',                                                   -- Select other encoding: [%1]
                'Select other ~encoding: [%1]',
                'SwitchEncoding',
                'Switch to next defined encoding',
                '',
                nodismiss)
   DefMenuItem( 'switchstandardencoding',                                           -- Select standard encoding: [%1]
                'Select ~standard encoding: [%1]',
                'SwitchStandardEncoding',
                'Switch to next defined encoding plus system encoding',
                '',
                nodismiss)
   DefMenuItem( 'configencoding',                                                   -- Configure encoding list...
                '~Configure encoding list...',
                'SelectEncoding',
                'Change encoding list or select or edit an encoding',
                '')
   DefNextItemEndsSubMenu()
   DefMenuItem( 'editencodingcmd',                                                  -- Edit command before execution
                'Edit command ~before execution',
                'ToggleEditEncodingCmd',
                'Open in entry box to edit',
                '',
                nodismiss)
compile if CHECK_FOR_LEXAM
   if LEXAM_is_available then
compile endif
   DefNextItemStartsSubMenu()
   DefMenuItem( 'spellcheck',                                                 -- Spell check   >
                'Spell chec~k',
                '',
                '',
                '',
                nodismiss)
   DefMenuItem( 'proofall',                                                         -- Proof
                '~Proof',
                'ProofMenuCmd',
                PROOF_MENUP__MSG,
                HP_OPTIONS_PROOF)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Options
                'Options',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'spellcheckmark',                                                   -- Mark
                '~Mark',
                'ToggleSpellCheckOption mark',
                'Process mark or the entire file',
                '',
                nodismiss)
   DefMenuItem( 'spellcheckcursor',                                                 -- Start at cursor
                'S~tart at cursor',
                'ToggleSpellCheckOption cursor',
                'Start at cursor or at top of file/mark',
                '',
                nodismiss)
   DefMenuItem( 'switchdict',                                                       -- Select dictionary: [%1]
                'Select ~dictionary: [%1]',
                'SwitchDict',
                'Switch to next defined dictionary language',
                '',
                nodismiss)
   DefMenuItem( '',                                                                 -- Configure dictionaries...
                '~Configure dictionaries...',
                'SelectDictLang',
                'Select a set of Netscape 4.6.1 dictionaries',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Proof word
                PROOF_WORD_MENU__MSG,
                'ProofWord',
                PROOF_WORD_MENUP__MSG,
                HP_OPTIONS_PROOFW)
   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 -- Synonym
                SYNONYM_MENU__MSG,
                'Syn',
                SYNONYM_MENUP__MSG,
                HP_OPTIONS_SYN)
compile if CHECK_FOR_LEXAM
   endif
compile endif
   DefMenuSep()                                                               --------------------

   DefNextItemStartsSubMenu()
   DefMenuItem( 'linewraps',                                                  -- Line wraps   >
                'Line ~wraps',
                '',
                '',
                '')
   DefMenuItem( 'wraplines',                                                        -- Wrap lines
                '|~Wrap lines|Soft-~wrap lines|Soft-~wrapped',
                'WrapMenuCmd',
                'Wrap lines according to selected options',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Options
                'Options',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'softwrap2',                                                        -- Soft wrap
                '~Soft wrap',
                'ToggleLineWrapOption softwrap',
                'Toggle non-destructive wrap',
                '',
                nodismiss)
   DefMenuItem( 'softwraptowindow2',                                                -- Use window width
                'Use wind~ow width',
                'ToggleLineWrapOption towindow',
                'Toggle soft wrap at window width or at header width',
                '',
                nodismiss)
   DefMenuItem( 'headerwidth2',                                                     -- Header width: %1
                '~Header width: %1...',
                'DefaultHeaderWidth',
                'Configure default header width used for boxes, center, wrap and syntax expansion',
                '')
   DefMenuItem( 'linewrapsmark',                                                    -- Marked lines
                '~Marked lines',
                'ToggleLineWrapOption mark',
                '',
                '',
                nodismiss)
   DefMenuItem( 'linewrapspar',                                                     -- Per paragraph
                '~Per paragraph',
                'ToggleLineWrapOption par',
                '',
                '',
                nodismiss)
   DefMenuItem( 'linewrapskeepindent',                                              -- Keep indent
                '~Keep indent',
                'ToggleLineWrapOption keepindent',
                '',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Split line
                'Spli~t line at cursor',
                'SplitLines',
                'Split current line at cursor column, keep indent',
                '')
   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 -- Join line
                '~Join line',
                'JoinLines',
                'Join current line with next line, respect right margin',
                '')

   DefNextItemStartsSubMenu()
   DefMenuItem( 'reflow',                                                     -- Reflow   >
                '~Reflow',
                '',
                '',
                '')
   DefMenuItem( 'reflowmode',                                                       -- Reflow %1
                '~Reflow %1',
                'ReflowMenuCmd',
                'Reflow lines according to selected options',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Options
                'Options',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'reflowmark',                                                       -- Marked lines
                '~Marked lines',
                'ToggleReflowOption mark',
                '',
                '',
                nodismiss)
   DefMenuItem( 'reflowpar',                                                        -- Per paragraph
                '~Per paragraph',
                'ToggleReflowOption par',
                '',
                '',
                nodismiss)
   DefMenuItem( 'reflowselectmode',                                                 -- Select special mode...
                '~Select special mode: %1...',
                'ReflowSelectMode',
                '',
                '')
   DefMenuSep()                                                                     --------------------

   -- Reflow mode options
   SelectedMode = ReflowGetSelectedMode()
   if 0 then
/*
   elseif SelectedMode = 'CONFIGSYS' then
      -- CONFIGSYS (test for RefreshSubMenu, must add defc ReflowCONFIGSYS)
      DefNextItemEndsSubMenu()
      DefMenuItem( 'reflowmodeoptions',                                             -- %1 options
                   '%1 options',
                   '',
                   '',
                   '',
                   'STATIC')
*/
   else
      -- TEXT
      DefMenuItem( 'reflowtypes',                                                   -- %1 reflow types
                   '%1 reflow types',
                   '',
                   '',
                   '',
                   'STATIC')
      DefMenuItem( 'reflownormal',                                                  -- Normal
                   '~Normal',
                   'ToggleReflowOption normal',
                   'Normal text reflow',
                   '',
                   nodismiss)
      DefMenuItem( 'reflowblock',                                                   -- Block mark
                   '~Block mark',
                   'ToggleReflowOption block',
                   'Mark lines or block first, then mark new block size',
                   '',
                   nodismiss)
      DefMenuItem( 'reflowwordproc',                                                -- Word processor (unterminated)
                   '~Word processor (unterminated)',
                   'ToggleReflowOption wordproc',
                   'Rejoin lines to prepare for export to a word processor',
                   '',
                   nodismiss)
      DefMenuItem( 'reflowmail',                                                    -- Mail
                   '~Mail',
                   'ToggleReflowOption mail',
                   'Reformat current text as email',
                   '',
                   nodismiss)
      DefMenuSep()                                                                  --------------------

      DefMenuItem( 'reflowmodeoptions',                                             -- %1 options
                   '%1 options',
                   '',
                   '',
                   '',
                   'STATIC')
      DefMenuItem( 'reflowtwospaces',                                               -- Two spaces
                   '~Two spaces',
                   'ToggleReflowOption twospaces',
                   'Put 2 spaces after periods etc.',
                   '',
                   nodismiss)
      DefNextItemStartsSubMenu()
      DefMenuItem( 'reflowmargins',                                                 -- Reflow margins   >
                   'Reflow ~margins: %1',
                   '',
                   'Margins/right margin for wrap and reflow actions',
                   '')
      DefMenuItem( 'reflowmargins1',                                                      -- 1: %1 (header width)
                   '~1: %1 (header width)',
                   'ToggleReflowMargins 1',
                   'Select specified value(s) as reflow margins',
                   '',
                   nodismiss)
      DefMenuItem( 'reflowmargins2',                                                      -- 2: %1
                   '~2: %1',
                   'ToggleReflowMargins 2',
                   'Select specified value(s) as reflow margins',
                   '',
                   nodismiss)
      DefMenuItem( 'reflowmargins3',                                                      -- 3: %1 (margins)
                   '~3: %1 (margins)',
                   'ToggleReflowMargins 3',
                   'Select file''s margins as reflow margins',
                   '',
                   nodismiss)
      DefMenuSep()                                                                        --------------------

      DefNextItemEndsSubMenu()
      DefMenuItem( 'reflowmarginsconfig',                                                 -- Configure selected...
                   '~Configure selected...',
                   'ReflowMarginsMenuCmd',
                   'Configure selected reflow margins item...',
                   '')
      DefMenuItem( 'reflowkeepindented',                                            -- Keep indented lines
                   '~Keep indented lines',
                   'ToggleReflowOption keepindented',
                   'Keep indented lines indented',
                   '',
                   nodismiss)
/*
      DefMenuItem( 'reflowwrapindented',                                            -- Wrap indented lines
                   'Wrap ~indented lines',
                   'ToggleReflowOption wrapindented',
                   'Wrap indented lines',
                   '',
                   nodismiss)
*/
      DefMenuItem( 'reflowreflowlists',                                             -- Reflow lists
                   '~Reflow lists',
                   'ToggleReflowOption reflowlists',
                   'Activate special handling of lists',
                   '',
                   nodismiss)
/*
      DefMenuItem( 'reflowreindentlists',                                           -- Reindent lists
                   'Reindent ~lists',
                   'ToggleReflowOption reindentlists',
                   '',
                   '',
                   nodismiss)
      DefMenuItem( 'reflowlistindent',                                              -- List indent
                   'List i~ndent: %1...',
                   '',
                   'Specify indent in columns for lists',
                   '')
*/
      DefNextItemEndsSubMenu()
      DefMenuItem( 'reflowmailfixreplylevels',                                      -- Fix false reply levels
                   '~Fix false reply levels',
                   'ToggleReflowOption mailfixreplylevels',
                   'Ask to fix false reply levels or to add empty lines',
                   '',
                   nodismiss)
   endif

   DefNextItemStartsSubMenu()
   DefMenuItem( 'codingstyle',                                                -- Coding style   >
                'Codi~ng style',
                '',
                'Definition set for syntax expansion',
                '')
   DefMenuItem( 'selectcodingstyle2',                                               -- Select coding style (%1) for mode %2...
                'Select ~coding style (%1) for mode %2...',
                'SelectCodingStyle',
                'Select a previously defined coding style for current mode',
                '',
                '')
   DefMenuItem( '',                                                                 -- Export user and default mode settings
                'Export user and default ~mode settings',
                'ExportConfig ud_mode \NEPMD\User\Mode\ 1',
                'Export user and default mode settings to a text file for later import',
                '',
                '')
   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 -- Export user and default coding styles
                'Export user and default ~coding styles',
                'ExportConfig ud_coding \NEPMD\User\CodingStyles 0',
                'Export user and default coding styles to a text file for later import',
                '',
                '')
   DefNextItemStartsSubMenu()
   DefMenuItem( 'centerboxdraw',                                              -- Center, Box and Draw   >
                'Center, Bo~x and Draw',
                '',
                'Alignment and draw boxes and lines',
                '')
   DefMenuItem( 'centermark',                                                       -- Center mark
                '~Center mark',
                'CenterMark',
                'Center lines of text within mark',
                '')
   DefMenuItem( 'box',                                                              -- Box mark
                '~Box mark',
                'Box',
                'Draw a box with the previously selected style around mark',
                '')
   DefMenuItem( 'selectboxstyle',                                                   -- Select box style
                '~Select box style: %1...',
                'Box select',
                'Open a list to select a style for boxes',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'headerwidth',                                                      -- Header width: %1
                '~Header width: %1...',
                'DefaultHeaderWidth',
                'Configure default header width used for boxes, center, wrap and syntax expansion',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'draw',                                                             -- Draw lines
                '~Draw lines',
                'Draw',
                'Draw lines with the previously selected style',
                '')
   DefNextItemEndsSubMenu()
   DefMenuItem( 'selectdrawstyle',                                                  -- Select draw style
                'S~elect draw style: %1...',
                'Draw select',
                'Open a list to select a style for drawing lines',
                '')
   DefMenuSep()                                                               --------------------

   DefNextItemStartsSubMenu()
   DefMenuItem( '',                                                           -- Special   >
                'S~pecial',
                '',
                '',
                '',
                nodismiss)
   DefMenuItem( '',                                                                 -- Remove HTML
                'Remove ~HTML',
                'UnHtml',
                'Remove HTML tags from current file',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Validate HTML
                '~Validate HTML...',
                'ValidateHtml',
                'Check syntax of a HTML file',
                '')
   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 -- Validate CSS
                'V~alidate CSS...',
                'ValidateCss',
                'Check syntax of a CSS file',
                '')
   DefNextItemStartsSubMenu()
   DefMenuItem( '',                                                           -- Font styles   >
                'Font st~yles',
                '',
                'Font and color attributes',
                '',
                nodismiss)
   DefMenuItem( '',                                                                 -- Style dialog...
                'St~yle dialog...',
                'FontList',
                STYLE_MENUP__MSG,
                HP_OPTIONS_STYLE)
   DefMenuItem( '',                                                                 -- Apply style...
                '~Apply style...',
                'linkexec stylebut apply_style S',
                'Select font style to apply on mark or all',
                '')
   DefMenuItem( '',                                                                 -- Remove attributes around cursor
                '~Remove attributes around cursor',
                'linkexec stylebut remove_style S',
                'Remove color and font attributes',
                '')
   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 -- Remove all attributes
                'R~emove all attributes',
                'DelAllAttr',
                'Remove all attributes, even bookmarks, from current file',
                '')
if nodismiss > 0 then
   DefMenuSep()                                                               --------------------

   DefMenuItem( '',                                                           -- Close menu
                'Close men~u',
                '',
                '',
                '')
endif

   return


; -------------------------------------------------------------------------------------- Search -----------------------
defc AddSearchMenu
   call Add_Search_Menu()

defproc Add_Search_Menu
   universal nodismiss

   fGrepFound  = (GetAVar( 'tool.grep') <> '')
   ToolFullName = GetAVar( 'tool.compare1')
   if ToolFullName = '' then
      fCompare1Found = 0
      Compare1Basename = ''
   else
      fCompare1Found = 1
      lp1 = lastpos( '\', ToolFullName)
      ToolName = substr( ToolFullName, lp1 + 1)
      lp2 = lastpos( '.', ToolName)
      ToolBaseName = leftstr( ToolName, Max( 0, lp2 - 1))
      Compare1Basename = ToolBasename
   endif
   ToolFullName = GetAVar( 'tool.compare2')
   if ToolFullName = '' then
      fCompare2Found = 0
      Compare2Basename = ''
   else
      fCompare2Found = 1
      lp1 = lastpos( '\', ToolFullName)
      ToolName = substr( ToolFullName, lp1 + 1)
      lp2 = lastpos( '.', ToolName)
      ToolBaseName = leftstr( ToolName, Max( 0, lp2 - 1))
      Compare2Basename = ToolBasename
   endif

   DefSubMenu( 'search',                                                -- Search ----------
               SEARCH_BAR__MSG,
               SEARCH_BARP__MSG,
               HP_SEARCH)


   DefMenuItem( '',                                                           -- Search dialog (search)...
                '~Search dialog (search)...',
                'SearchDlg S',
                SEARCH_MENUP__MSG' with a blank Replace entry field',
                HP_SEARCH_SEARCH)
   DefMenuItem( '',                                                           -- Search dialog (replace)...
                '~Search dialog (replace)...',
                'SearchDlg',
                SEARCH_MENUP__MSG,
                HP_SEARCH_SEARCH)
   DefMenuSep()                                                               --------------------

   -- The standard keyset doesn't have keys to search backward. Therefore Ctrl+- was added.
   -- For a CUA keyset simply the shifted variants were used.
   -- Therefore the following Search and Change menu items depend on the keyset:
if GetAddKeyDefs() = '' then
   -- stdkeys
   DefMenuItem( 'findnext',                                                   -- Repeat find
                'Repeat ~find',
                'RepeatFind',
                FIND_NEXT_MENUP__MSG,
                HP_SEARCH_FIND)
   DefMenuItem( 'changenext',                                                 -- Repeat change
                'Repeat ~change',
                'RepeatChange',
                CHANGE_NEXT_MENUP__MSG,
                HP_SEARCH_CHANGE)
   DefMenuItem( 'globalfindnext',                                             -- Repeat find in loaded files
                '~Repeat find in loaded files',
                'RepeatFindAllFiles',
                'Repeat previous Locate command for all files in the ring',
                '')
   DefMenuItem( 'globalchangenext',                                           -- Repeat change in loaded files
                'Repeat chang~e in loaded files',
                'RepeatChangeAllFiles',
                'Repeat previous Change command for all files in the ring',
                '')
   DefMenuSep()                                                               --------------------

   DefMenuItem( 'searchbackward',                                             -- Backward
                'Backwar~d',
                'toggle_search_backward',
                'Toggle back/forward for next locate/change commands',
                '',
                nodismiss)
   DefMenuSep()                                                               --------------------

   DefMenuItem( '',                                                           -- Find all /<string>...
                'Find ~all /<string>...',
                'CommandLine all /',
                'List all occurrences in current file, then use Ctrl+Q to toggle',
                '')
   -- end stdkeys
else
   -- cuakeys
   DefMenuItem( 'findnext',                                                   -- Find next
                '~Find next',
                'FindNext',
                'Repeat find, foreward',
                HP_SEARCH_FIND)
   DefMenuItem( 'findprev',                                                   -- Find previous
                'Find ~previous',
                'FindPrev',
                'Repeat find, backward',
                HP_SEARCH_FIND)
   DefMenuItem( 'changenext',                                                 -- Change, then find next
                '~Change, then find next',
                'ChangeFindNext',
                'Change, then repeat find, foreward',
                HP_SEARCH_CHANGE)
   DefMenuItem( 'changeprev',                                                 -- Change, then find previous
                'C~hange, then find previous',
                'ChangeFindPrev',
                'Change, then repeat find, backward',
                HP_SEARCH_CHANGE)
   DefMenuItem( '',                                                           -- Find all /<string>...
                'Find ~all /<string>...',
                'CommandLine all /',
                'List all occurrences in current file, then use Ctrl+Q to toggle',
                '')
   DefMenuSep()                                                               --------------------

   DefMenuItem( 'globalfindnext',                                             -- Find next in loaded files
                'Find next in ~loaded files',
                'FindNextAllFiles',
                'Repeat find for all files in the ring, foreward',
                '')
   DefMenuItem( 'globalfindprev',                                             -- Find previous in loaded files
                'Find previous in loaded ~files',
                'FindPrevAllFiles',
                'Repeat find for all files in the ring, backward',
                '')
   DefMenuItem( 'globalchangenext',                                           -- Repeat change in loaded files
                'Repeat chang~e in loaded files',
                'RepeatChangeAllFiles',
                'Repeat previous Change command for all files in the ring',
                '')
   -- end cuakeys
endif
if fGrepFound then
   DefMenuSep()                                                               --------------------

   DefMenuItem( '',                                                           -- Grep...
                '~Grep...',
                'GrepDlg',
                'Scan external files using regular expressions',
                '')
endif
   DefMenuSep()                                                               --------------------

   DefMenuItem( '',                                                           -- Find identifier
                'Find ~identifier',
                'FindToken',
                'Find identifier (C-style word) under cursor',
                '')
   DefMenuItem( '',                                                           -- Find word
                'Find ~word',
                'FindWord',
                'Find word under cursor',
                '')
   DefMenuItem( 'findmark',                                                   -- Find mark
                'Find ~mark',
                'FindMark',
                'Find marked string else word under cursor',
                '')
/*
   DefMenuItem( '',                                                           -- ShowSearchArgs
                'Show search args',
                'ShowSearch',
                'Show last find/change args',
                '')
*/
   DefMenuItem( '',                                                           -- Find bracket
                'Find brac~ket',
                'Assist',
                'Find matching environment expression',
                '')
   DefMenuSep()                                                               --------------------

   DefNextItemStartsSubMenu()
   DefMenuItem( 'goto',                                                       -- Go to   >
                'G~o to',
                '',
                '',
                '')
   DefMenuItem( '',                                                                 -- Line...
                '~Line...',
                'GotoLineDlg',
                'Change line and optionally column',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'gotobeginmark',                                                    -- Begin of mark
                'Begin of ~mark',
                'BeginMark',
                'Position cursor on begin of marked area',
                '')
   DefMenuItem( 'gotoendmark',                                                      -- End of mark
                'End of mar~k',
                'EndMark',
                'Position cursor on end of marked area',
                '')
   DefMenuItem( 'gotoaftermark',                                                    -- After mark
                '~After mark',
                'AfterMark',
                'Position cursor after end of marked area',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Begin of word
                'Begin of ~word',
                'BeginWord',
                'Position cursor on begin of the nearest word',
                '')
   DefMenuItem( '',                                                                 -- End of word
                'End of wor~d',
                'EndWord',
                'Position cursor on end of the nearest word',
                '')
   DefMenuSep()                                                                     --------------------

   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 -- Show cursor
                'Show ~cursor'MenuAccelString( 'HighlightCursor'),
                'ShowCursor',
                'Scroll to cursor position and highlight cursor',
                '')
   DefNextItemStartsSubMenu()
   DefMenuItem( 'markstack',                                                  -- Mark stack   >
                '~Mark stack',
                '',
                'Save and restore a marked area',
                '')
   DefMenuItem( 'savemark',                                                         -- Save mark
                PUSH_MARK_MENU__MSG,
                'PushMark',
                PUSH_MARK_MENUP__MSG,
                HP_EDIT_PUSHMARK)
   DefMenuItem( 'restoremark',                                                      -- Restore mark
                POP_MARK_MENU__MSG,
                'PopMark',
                POP_MARK_MENUP__MSG,
                HP_EDIT_POPMARK)
   DefNextItemEndsSubMenu()
   DefMenuItem( 'swapmark',                                                         -- Swap mark
                SWAP_MARK_MENU__MSG,
                'SwapMark',
                SWAP_MARK_MENUP__MSG,
                HP_EDIT_SWAPMARK)
   DefNextItemStartsSubMenu()
   DefMenuItem( 'cursorstack',                                                -- Cursor stack   >
                'C~ursor stack',
                '',
                'Save and restore cursor position',
                '')
   DefMenuItem( 'savecursor',                                                       -- Save cursor
                PUSH_CURSOR_MENU__MSG,
                'PushPos',
                PUSH_CURSOR_MENUP__MSG,
                HP_EDIT_PUSHPOS)
   DefMenuItem( 'restorecursor',                                                    -- Restore cursor
                POP_CURSOR_MENU__MSG,
                'PopPos',
                POP_CURSOR_MENUP__MSG,
                HP_EDIT_POPPOS)
   DefNextItemEndsSubMenu()
   DefMenuItem( 'swapcursor',                                                       -- Swap cursor
                SWAP_CURSOR_MENU__MSG,
                'SwapPos',
                SWAP_CURSOR_MENUP__MSG,
                HP_EDIT_SWAPPOS)
   DefMenuSep()                                                               --------------------

   DefNextItemStartsSubMenu()
   DefMenuItem( 'bookmarks',                                                  -- Bookmarks   >
                BOOKMARKS_MENU__MSG,
                '',
                'Set and jump to bookmarks',
                HP_SEARCH_BOOKMARKS)
   DefMenuItem( 'bookmarks_set',                                                    -- Set...
                SET_MARK_MENU__MSG,
                'SetBookmark',
                SET_MARK_MENUP__MSG,
                HP_SEARCH_BOOKMARKS)
   DefMenuItem( '',                                                                 -- Set auto-named non-permanent
                'Set auto-named n~on-permanent',
                'SetAutoNamedBookmark',
                '',
                HP_SEARCH_BOOKMARKS)
   DefMenuItem( '',                                                                 -- Set auto-named permanent
                'Set auto-named p~ermanent',
                'SetAutoNamedBookmarkP',
                '',
                HP_SEARCH_BOOKMARKS)
   DefMenuItem( 'bookmarks_list',                                                   -- List...
                LIST_MARK_MENU__MSG,
                'ListMark',
                LIST_MARK_MENUP__MSG,
                HP_SEARCH_BOOKMARKS)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'bookmarks_next',                                                   -- Next
                NEXT_MARK_MENU__MSG,
                'NextBookMark',
                NEXT_MARK_MENUP__MSG,
                HP_SEARCH_BOOKMARKS)
   DefNextItemEndsSubMenu()
   DefMenuItem( 'bookmarks_previous',                                               -- Previous
                PREV_MARK_MENU__MSG,
                'NextBookMark P',
                PREV_MARK_MENUP__MSG,
                HP_SEARCH_BOOKMARKS)
   DefNextItemStartsSubMenu()
   DefMenuItem( 'finddef',                                                    -- Find definition   >
                'Find defi~nition',
                '',
                'Find def. in source files for identifier under cursor',
                '')
   DefMenuItem( 'finddefmode',                                                      -- Find definition for mode %1
                'Find ~definition for mode %1',
                'FindDef',
                'Find definition',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Search types
                'Search types',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'finddefincludes',                                                  -- Find in includes
                'Find in ~includes',
                'FindDefSearchTypeMenuCmd I',
                'Search indentifier under cursor in included files',
                '',
                nodismiss)
   DefMenuItem( 'finddeffilemasks',                                                 -- Find in filemasks
                'Find in ~filemasks',
                'FindDefSearchTypeMenuCmd F',
                'Search indentifier under cursor in configured filemasks with E and C code',
                '',
                nodismiss)
if fGrepFound then
   DefMenuItem( 'finddefgrep',                                                      -- Find in filemasks with grep.exe
                'Find in filemasks with ~grep.exe',
                'FindDefSearchTypeMenuCmd G',
                'Search indentifier under cursor in configured filemasks with grep.exe',
                '',
                nodismiss)
endif
   DefMenuItem( 'finddefwords',                                                     -- Find words in current file
                'Find ~words in current file',
                'FindDefSearchTypeMenuCmd W',
                'Search indentifier under cursor in current file only',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Options
                'Options',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'finddefexact',                                                     -- Exact search
                '~Exact search',
                'ToggleFindDefExact',
                'Exact search takes longer, but is required for some modes',
                '',
                nodismiss)
   DefMenuItem( 'finddefprototypes',                                                -- Function prototypes
                'Function ~prototypes',
                'ToggleFindDefPrototypes',
                'Include prototype definitions in the search result',
                '',
                nodismiss)
   DefMenuItem( 'finddefcomlit',                                                    -- Comments and literals
                '~Comments and literals',
                'ToggleFindDefComLit',
                'Search for definitions within comments and literals',
                '',
                nodismiss)
   DefMenuItem( 'finddefallmatches',                                                -- All matches
                '~All matches',
                'ToggleFindDefAllMatches',
                'Disable to stop searching at the first match',
                '',
                nodismiss)
   DefMenuItem( 'finddefrecursion',                                                 -- Recursion as specified
                '~Recursion as specified',
                'ToggleFindDefRecursion',
                'Activate recursion if configured',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Export user and default mode settings
                'E~xport user and default mode settings',
                'mc ;ExportConfig ud_mode \NEPMD\User\Mode\ 1; PostMe l /FindDef',
                'Export user and default mode settings to a text file for later import',
                '',
                nodismiss)
   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 -- Edit MYEPM.ENV
                'Edit ~MYEPM.ENV',
                'EditCreateMyepmEnv',
                'Edit user environment file. This file is loaded additionally to EPM.ENV.',
                '',
                nodismiss)
/*
   DefMenuItem( 'finddefaddsearchpath',                                             -- Add. search path for mode %1...
                'Add. ~search path for mode %1...',
                '',
                '',
                '')
   DefNextItemEndsSubMenu()
   DefMenuItem( 'finddefsearchrelative',                                             -- Relative search for mode %1...
                'Re~lative search for mode %1...',
                '',
                '',
                '')
*/
   DefNextItemStartsSubMenu()
   DefMenuItem( 'tags',                                                       -- Tags   >
                TAGS_MENU__MSG,
                '',
                'Find function in a tags file',
                HP_SEARCH_TAGS)
   DefMenuItem( '',                                                                 -- Tags dialog...
                TAGSDLG_MENU__MSG,
                'PopTagsDlg',
                TAGSDLG_MENUP__MSG,
                HP_SEARCH_TAGS)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'tagfindcurrent',                                                   -- Find current procedure
                FIND_TAG_MENU__MSG,
                'FindTag',
                FIND_TAG_MENUP__MSG,
                HP_SEARCH_TAGS)
   DefMenuItem( 'tagfindall',                                                       -- Find procedure...
                FIND_TAG2_MENU__MSG,
                'FindTag *',
                FIND_TAG2_MENUP__MSG,
                HP_SEARCH_TAGS)
   DefMenuSep()                                                                     --------------------

   DefNextItemEndsSubMenu()
   DefMenuItem( 'tagscan',                                                          -- Scan file for tags
                '~Scan file for tags' /*SCAN_TAGS_MENU__MSG*/,
                'TagScan',
                SCAN_TAGS_MENUP__MSG,
                HP_SEARCH_TAGS)

   DefMenuSep()                                                               --------------------

if fCompare1Found then
   MenuName = upcase( Compare1Basename)
   -- Use D as mnemonic key
   p = pos( 'D', MenuName)
   if not p then
      p = 1
   endif
   MenuName = insertstr( '~', MenuName, p - 1)
   DefMenuItem( '',                                                           -- KDiff3 current file
                MenuName 'current file...',
                'CompareCurrentFile compare1',
                'Compare current file with another',
                '')
endif
if fCompare2Found then
   MenuName = upcase( Compare2Basename)
   p = 1
   MenuName = insertstr( '~', MenuName, p - 1)
   DefMenuItem( '',                                                           -- GFC current file
                MenuName 'current file...',
                'CompareCurrentFile compare2',
                'Compare current file with another',
                '')
endif
if nodismiss > 0 then
   DefMenuSep()                                                               --------------------

   DefMenuItem( '',                                                           -- Close menu
                'Close men~u',
                '',
                '',
                '')
endif


   return

; -------------------------------------------------------------------------------------- View -------------------------
defc AddViewMenu
   call Add_View_Menu()

defproc Add_View_Menu
   universal nodismiss

   DefSubMenu( 'view',                                                  -- View ------------
               '~View',
               'Menus related to views, cursor pos and windows',
               '')
   DefMenuItem( 'softwrap',                                                   -- Soft wrap
                'Soft ~wrap',
                'ToggleWrap',
                'Toggle non-destructive wrap',
                '')
   DefMenuItem( 'softwraptowindow',                                           -- Use window width
                'Use wind~ow width',
                'ToggleSoftWrapToWindow',
                'Toggle soft wrap at window width or at header width',
                '')
   -- Check for a cmd of a linked file won't work here, because the menu is already built by 'initconfig'.
if isadefc( 'fold') | (NepmdSearchPath( 'fold.ex', 'EPMEXPATH') > '') then
   DefMenuSep()                                                               --------------------

   DefMenuItem( '',                                                           -- Fold
                '~Fold',
                'fold',
                '',
                '')
   DefMenuItem( '',                                                           -- Unfold
                '~Unfold',
                'fold off',
                '',
                '')
endif  -- isadefc( 'fold')
   DefMenuSep()                                                               --------------------

   DefMenuItem( 'listring',                                                   -- List ring...
                LIST_FILES_MENU__MSG,
                'Ring_More',
                LIST_FILES_MENUP__MSG,
                HP_OPTIONS_LIST)
   DefMenuItem( 'nextfile',                                                   -- Next file
                'Ne~xt file',
                'NextFile',
                'Activate the next file in the edit ring',
                '')
   DefMenuItem( 'prevfile',                                                   -- Previous file
                '~Previous file',
                'PrevFile',
                'Activate the previous file in the edit ring',
                '')
   DefMenuSep()                                                               --------------------

   DefMenuItem( '',                                                           -- Messages...
                MESSAGES_MENU__MSG,
                'MessageBox',
                MESSAGES_MENUP__MSG,
                HP_OPTIONS_MESSAGES)
   DefMenuSep()                                                               --------------------

   DefMenuItem( '',                                                           -- Add new view
                'Add new ~view',
                'e /v =',
                'Create another synchronized view of the current file',
                '')
   DefMenuItem( 'nextview',                                                   -- Switch to next view
                'Switch to ne~xt view',
                'NextView',
                'Activate the next view of the current file',
                '')
   DefMenuSep()                                                               --------------------

   DefMenuItem( '',                                                           -- Move to new window
                'Move to ~new window',
                'NewWindow',
                'Move current file to a new window',
                '')
   DefMenuItem( '',                                                           -- Open in bin mode
                'Open in ~bin mode',
                "Open 'BinEdit ='",
                'Open current file in a new window in binedit mode',
                '')
   DefMenuSep()                                                               --------------------

   DefMenuItem( 'switchnextwindow',                                           -- Switch to next window
                '~Switch to next window',
                'Next_Win',
                'Activate the next EPM window',
                '')

   return

; -------------------------------------------------------------------------------------- Run --------------------------
; processmenuinit is not executed for mid = 1, but 0 works. Changed menu id
; from 1 to 0 to make processmenuinit work. That hopefully doesn't interfere
; with any external package.
; In SELECT.E the code for the command menu is removed and now added here,
; in menuinit_run.
; The menu ids and the item ids must be unique. There 100 is added if mid = 0
; in order to not overwrite other mids.
defc AddRunMenu
   call Add_Run_Menu()

defproc Add_Run_Menu
   universal nodismiss

   DefSubMenu( 'run',                                                   -- Run -------------
               '~Run',
               'Menus to execute commands',
               '')

   NextMenuAccelString = ''
   if NextMenuAccelString = '' then
      NextMenuAccelString = MenuAccelString( 'CommandLine')
   else
      NextMenuAccelString = NextMenuAccelString' | 'strip( MenuAccelString( 'CommandLine'), 'L', \9)
   endif
   if NextMenuAccelString = '' then
      NextMenuAccelString = MenuAccelString( 'ProcessEscape CommandLine')
   else
      NextMenuAccelString = NextMenuAccelString' | 'strip( MenuAccelString( 'ProcessEscape CommandLine'), 'L', \9)
   endif
   DefMenuItem( '',                                                           -- Command dialog...
                COMMANDLINE_MENU__MSG''NextMenuAccelString,
                'CommandLine',
                COMMANDLINE_MENUP__MSG,
                HP_COMMAND_CMD)

   -- The menu item id for Halt must be 65535, which executes 'processbreak'
   -- internally.
   -- Executing 'processbreak' via command call won't work while another
   -- command is being processed.
   -- The message id 65535 is also sent on closing the action bar menu.
   -- Therefore better don't use a hint text here.
   DefMenuItem( 65535,                                                        -- Halt command
                HALT_COMMAND_MENU__MSG''MenuAccelString( 'ProcessBreak'),
                '',
                '',
                HP_COMMAND_HALT)
   DefMenuSep()                                                               --------------------

   DefMenuItem( '',                                                           -- Create command shell
                CREATE_SHELL_MENU__MSG,
                'Shell new',
                'Create a command shell buffer',
                HP_COMMAND_SHELL)
   DefMenuItem( '',                                                           -- Switch command shells
                'Sw~itch command shells',
                'Shell',
                'Loop through shell or starting non-shell buffers',
                HP_COMMAND_SHELL)
   DefMenuItem( 'writetoshell',                                               -- Write to shell...
                WRITE_SHELL_MENU__MSG,
                'shell_write',
                WRITE_SHELL_MENUP__MSG,
                HP_COMMAND_WRITE)
   DefMenuItem( 'sendbreaktoshell',                                           -- Send break to shell
                SHELL_BREAK_MENU__MSG,
                'shell_break',
                SHELL_BREAK_MENUP__MSG,
                HP_COMMAND_BREAK)
   DefNextItemStartsSubMenu()
   DefMenuItem( 'configureshell',                                             -- Configure command shells  >
                'Con~figure command shells',
                '',
                '',
                '')
   DefMenuItem( '',                                                                 -- Options
                'Options',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( '',                                                                 -- Set init command...
                'Set ~init command...',
                'ShellInitCmdDlg',
                'OS/2 command to be executed on start of a shell',
                '')
   DefMenuItem( 'filenamecompletion',                                               -- Activate filename completion
                'Activate ~filename completion',
                'toggle_filename_completion',
                'Use Tab and Sh+Tab to insert matching filenames',
                '',
                nodismiss)
   DefMenuItem( 'alias',                                                            -- Activate aliases
                'Activate ~aliases',
                'toggle_alias',
                'Put an asterisk before a command to temp. disable it',
                '',
                nodismiss)
   DefNextItemEndsSubMenu()
   DefMenuItem( 'editalias',                                                        -- Edit ALIAS.CFG
                'Edit ALIAS.~CFG',
                'EditCreateUserFile bin\alias.cfg',
                'Edit alias file for alias configuration',
                '')
   DefMenuSep()                                                               --------------------

   DefMenuItem( '',                                                           -- OS/2 window
                '~OS/2 window',
                'start /win',
                'Open an OS/2 window with NEPMD''s environment',
                '')
   DefMenuSep()                                                               --------------------

   DefNextItemStartsSubMenu()
   DefMenuItem( 'treecommands',                                               -- ~Tree commands   >
                '~Tree commands',
                '',
                'List a tree and specify a cmd to get executed on every file',
                '')
   DefMenuItem( '',                                                           -- ~Tree <filemask>...
                '~Tree <filemask>...',
                'commandline tree ',
                'List matched files recoursively',
                32111)
   DefMenuItem( '',                                                           -- Tree_~dir <filemask>...
                'Tree~Dir <filemask>...',
                'commandline TreeDir ',
                'List matched files',
                '')
   DefMenuItem( 'treesort',                                                   -- Tree~sort <columnname>...
                'Tree~sort <columnname>...',
                'commandline treesort ',
                'Sort tree listing (use MouseButton 2 alternatively)',
                '')
   DefNextItemEndsSubMenu()
   DefMenuItem( 'treeit',                                                     -- Tree~it <cmd>...
                'Tree~it <cmd>...',
                'commandline treeit ',
                'Execute a command on all listed files',
                '')
   DefMenuSep()                                                               --------------------

   DefMenuItem( '',                                                           -- Execute current line
                'E~xecute current line',
                'DoLines',
                'Execute line under cursor',
                '')
   DefMenuItem( '',                                                           -- Command line current line
                'Command line current ~line',
                'CommandDlgLine',
                'Open line under cursor in command dialog',
                '')
   DefMenuItem( '',                                                           -- Run current file
                '~Run current file',
                'rx Run',
                'Execute current file according to what is defined in RUN.ERX',
                '')
   DefMenuItem( '',                                                           -- Edit RUN.ERX
                '~Edit RUN.ERX',
                'EditCreateUserFile bin\run.erx',
                'Edit REXX file for Run configuration',
                '')
   DefMenuSep()                                                               --------------------

   DefNextItemStartsSubMenu()
   DefMenuItem( 'recordkeys',                                                 -- Key recorder   >
                '~Key recorder',
                '',
                'Record and playback keys',
                '')
   DefMenuItem( 'startrecording',                                                   -- Start recording
                'Start/end ~recording',
                'RecordKeys',
                '',
                '')
   DefMenuItem( 'playback',                                                         -- Playback
                '~Playback',
                'PlaybackKeys',
                '',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'selectrecordkeysmacro',                                            -- Select record keys macro: %1
                '~Select record keys macro: %1...',
                'RecKeysSelectFile',
                '',
                '')
   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 -- Edit record keys macro
                '~Edit record keys macro...',
                'RecKeysSelectEditFile',
                '',
                '')

/*
   DefMenuItem( '',                                                           -- Run main project file
                'Run main project file',
                'rx runmain',
                'Execute main project file according to what is def''d in RUN.ERX',
                '')
   DefMenuItem( '',                                                           -- Compile main project file
                'Compile main project file',
                '',
                '',
                '')
   DefMenuItem( '',                                                           -- Select project
                'Select project',
                '',
                '',
                '')
*/
   DefMenuSep()                                                               --------------------

   UserDir = Get_Env( 'NEPMD_USERDIR')
   UserDirName = substr( UserDir, lastpos( '\', UserDir) + 1)

   DefNextItemStartsSubMenu()
   DefMenuItem( 'emacros',                                                    -- E Macros   >
                'E ~Macros',
                '',
                'Compile EPM macro files',
                '')

   DefMenuItem( '',                                                                 -- Recompile new user macros
                '~Recompile new user macros',
                'RecompileNew',
                'Recompile all new user macros and maybe restart EPM',
                '')
   DefMenuItem( 'editaddlst',                                                       -- Edit MACROADD.LST
                'Edit MACROA~DD.LST',
                'EditMacroLstFile',
                'Edit or create list for additional .e files for RecompileNew',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Compile current .E file
                '~Compile current .E file',
                'etpm =',
                'Compile current macro file',
                '')
   DefMenuItem( '',                                                                 -- Relink current .E file
                'Re~link current .E file',
                'relink',
                'Compile current macro file, unlink and link it',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'editprofile',                                                      -- Edit PROFILE.ERX
                'Edit ~PROFILE.ERX',
                'EditCreateProfile',
                'Edit or create REXX configuration file',
                '')
   DefMenuItem( 'activateprofile',                                                  -- Activate PROFILE.ERX
                '~Activate PROFILE.ERX',
                'toggle_profile',
                'Activate REXX configuration file',
                '',
                nodismiss)
   DefMenuItem( 'editmystuff',                                                      -- Edit MYSTUFF.E
                'Edit MY~STUFF.E',
                'e %NEPMD_USERDIR%\macros\mystuff.e',
                'Edit or create E macro additions',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Open NETLABS\MACROS\*.E
                'Open ~NETLABS\MACROS\*.E',
                'o %NEPMD_ROOTDIR%\netlabs\macros\*.e',
                '',
                '')
   DefMenuItem( '',                                                                 -- Open MYEPM\MACROS\*.E
                'Open 'upcase( UserDirName)'\MACROS\*.~E',
                'o %NEPMD_USERDIR%\macros\*.e',
                '',
                '')
   DefMenuSep()                                                                     --------------------

   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 -- Open NEPMD.INI
                'Open NEPMD.~INI',
                'OpenObject %NEPMD_USERDIR%\bin\nepmd.ini',
                '',
                '')

   return

; -------------------------------------------------------------------------------------- Help -------------------------
defc AddHelpMenu
   call Add_Help_Menu()

defproc Add_Help_Menu
   universal nodismiss

   DefSubMenu( 'help',                                                  -- Help ------------
               HELP_BAR__MSG,
               HELP_BARP__MSG,
               HP_HELP)

   DefMenuItem( '',                                                           -- Help index
                HELP_INDEX_MENU__MSG,
                'helpmenu 10'/*64044*/,
                HELP_INDEX_MENUP__MSG,
                HP_HELP_INDEX)
   DefMenuItem( '',                                                           -- General help
                EXT_HELP_MENU__MSG,
                'helpmenu 4000',
                EXT_HELP_MENUP__MSG,
                HP_HELP_EXTENDED)
   DefMenuItem( '',                                                           -- Current keyset definitions
                'Current ~keyset definitions',
                'ShowKeyset',
                'Lists current key and mouse definitions',
                '')
   DefMenuItem( '',                                                           -- Keys help
                'Ke~ys help',
                'helpmenu 1000',
                KEYS_HELP_MENUP__MSG,
                HP_HELP_KEYS)
   DefMenuItem( '',                                                           -- Commands help
                COMMANDS_HELP_MENU__MSG,
                'helpmenu 2000',
                COMMANDS_HELP_MENUP__MSG,
                HP_HELP_COMMANDS)
   DefMenuItem( '',                                                           -- Using help
                HELP_HELP_MENU__MSG,
                'helpmenu 0'/*64027*/,
                HELP_HELP_MENUP__MSG,
                HP_HELP_HELP)
   DefMenuSep()                                                               --------------------

   DefMenuItem( '',                                                           -- Quick reference
                HELP_BROWSER_MENU__MSG,
                'help',
                HELP_BROWSER_MENUP__MSG,
                HP_HELP_BROWSE)
   DefMenuSep()                                                               --------------------

compile if SUPPORT_USERS_GUIDE
if NepmdSearchPath( 'epmusers.inf', 'BOOKSHELF') then
   DefNextItemStartsSubMenu()
   DefMenuItem( 'viewusersguide',                                             -- View User's Guide   >
                USERS_GUIDE_MENU__MSG,
                '',
                USERS_GUIDE_MENUP__MSG,
                HP_HELP_USERS_GUIDE,
                ''/*SYSCOMMAND*/)
   DefMenuItem( 'usersguide',                                                       -- View User's Guide
                VIEW_USERS_MENU__MSG,
                'start view epmusers',
                VIEW_USERS_MENUP__MSG,
                HP_HELP_USERS_GUIDE)
   DefMenuItem( '',                                                                 -- Current word
                VIEW_IN_USERS_MENU__MSG,
                'viewword epmusers',
                VIEW_IN_USERS_MENUP__MSG,
                HP_HELP_USERS_GUIDE)
   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 -- Summary
                VIEW_USERS_SUMMARY_MENU__MSG,
                'start view epmusers Summary',
                VIEW_USERS_SUMMARY_MENUP__MSG,
                HP_HELP_USERS_GUIDE)
endif
compile endif
compile if SUPPORT_TECHREF
if NepmdSearchPath( 'epmtech.inf', 'BOOKSHELF') then
   DefNextItemStartsSubMenu()
   DefMenuItem( 'viewtechnicalreference',                                     -- View Technical Reference   >
                TECHREF_MENU__MSG,
                '',
                TECHREF_MENUP__MSG,
                HP_HELP_TECHREF,
                ''/*SYSCOMMAND*/)
   DefMenuItem( 'technicalreference',                                               -- View Technical Reference
                VIEW_TECHREF_MENU__MSG,
                'start view epmtech',
                VIEW_TECHREF_MENUP__MSG,
                HP_HELP_TECHREF)
   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 -- Current word
                VIEW_IN_TECHREF_MENU__MSG,
                'viewword epmtech',
                VIEW_IN_TECHREF_MENUP__MSG,
                HP_HELP_TECHREF)
endif
compile endif
   DefNextItemStartsSubMenu()
   DefMenuItem( 'viewnepmdusersguide',                                        -- View NEPMD User's Guide   >
                'View ~NEPMD User''s Guide',
                '',
                '',
                '',
                ''/*SYSCOMMAND*/)
   DefMenuItem( 'nepmdusersguide',                                                  -- View NEPMD User's Guide
                '~View NEPMD User''s Guide',
                'start view neusr%NEPMD_LANGUAGE% netlabs',  -- start is used here to resolve environment var,
                '',
                '')
   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 -- Current word
                '~Current word',
                'viewword neusr%NEPMD_LANGUAGE%',
                '',
                '')
if NepmdSearchPath( 'neprg'Get_Env( 'NEPMD_LANGUAGE')'.inf', 'BOOKSHELF') then
   DefNextItemStartsSubMenu()
   DefMenuItem( 'viewnepmdprogrammingguide',                                  -- View NEPMD Programming Guide   >
                'View N~EPMD Programming Guide',
                '',
                '',
                '',
                ''/*SYSCOMMAND*/)
   DefMenuItem( 'nepmdprogrammingguide',                                            -- View NEPMD Programming Guide
                '~View NEPMD Programming Guide',
                'start view neprg%NEPMD_LANGUAGE% netlabs',  -- start is used here to resolve environment var,
                '',
                '')
   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 -- Current word
                '~Current word',
                'viewword neprg%NEPMD_LANGUAGE%',
                '',
                '')
endif
   DefMenuSep()                                                               --------------------

   DefNextItemStartsSubMenu()
   DefMenuItem( 'keywordhelp',                                                -- Keyword help
                'Key~word help',
                '',
                'View documentation for a keyword (defined via .ndx files)',
                '')
   DefMenuItem( 'keywordhelpcurrentword',                                     -- Current word
                '~Current word',
                'KwHelp',
                'View documentation for keyword under cursor',
                '')
   DefMenuItem( '',                                                           -- Word...
                '~Word...',
                'KwHelp ?',
                'Prompt for a keyword, then view documentation for it',
                '')
   DefMenuSep()                                                               --------------------

   DefMenuItem( '',                                                           -- Options
                'Options',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'usenewview',                                                 -- Use NewView if found
                'Use ~NewView if found',
                'toggle_use_newview',
                'NiewView.exe is searched in PATH and used if found',
                '',
                nodismiss)
   DefNextItemEndsSubMenu()
   DefMenuItem( 'usenewviewxsearch',                                          -- Use NewView's extended search
                'Use NewView''s ~extended search',
                'toggle_newview_xsearch',
                'Search in text instead of just a topic search',
                '',
                nodismiss)
   DefMenuSep()                                                               --------------------

/*
   DefMenuItem( '',                                                           --
                HELP_PROD_MENU__MSG,
                'IBMmsg',                                                     -- Product information,
                HELP_PROD_MENUP__MSG,
                HP_HELP_PROD)

*/
   DefMenuItem( '',                                                           -- NEPMD runtime information
                'NEPMD ~runtime information',
                'nepmdinfo',
                'Lists used DLLs and NEPMD''s dynamic configuration',
                '')
   return

; -------------------------------------------------------------------------------------- Preferences ------------------
const
compile if not defined( PRESET_BITSTR_USER_CUA)
   PRESET_BITSTR_USER_CUA = 011000000
compile endif
compile if not defined( PRESET_BITSTR_PROG_CUA)
   PRESET_BITSTR_PROG_CUA = 111111111
compile endif
compile if not defined( PRESET_BITSTR_PROG_STD)
   PRESET_BITSTR_PROG_STD = 100111111
compile endif

; ---------------------------------------------------------------------------
defc PrefsPopupMenu
   universal activemenu
   universal nodismiss  -- 0 | 32

   menuname = 'prefspopup'  -- 'preferences' is already used by the edit menu item
   activemenu = menuname
   call SetAVar( 'mid_'menuname, 90)
   call SetAVar( 'popupmids', '')

   deletemenu menuname, 0, 0, 0

   DefSubMenu( menuname, '')

   DefNextItemStartsSubMenu()
   DefMenuItem( 'basicconfig',                                                -- Basic configuration  >
                'Basic con~figuration',
                '',
                'Set basic configuration',
                '',
                nodismiss)
   DefMenuItem( '',                                                                 -- Presets
                'Presets',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'presetusercua',                                                    -- User, CUA
                '~User, CUA',
                'ApplyPresetBitStr' PRESET_BITSTR_USER_CUA,
                'Reset to standard user settings with CUA keyset',
                '',
                nodismiss)
   DefMenuItem( 'presetprogcua',                                                    -- Programmer, CUA (default)
                '~Programmer, CUA (default)',
                'ApplyPresetBitStr' PRESET_BITSTR_PROG_CUA,
                'Reset to programmer settings with CUA keyset',
                '',
                nodismiss)
   DefMenuItem( 'presetprogstd',                                                    -- Programmer, standard EPM
                'Programmer, standard ~EPM',
                'ApplyPresetBitStr' PRESET_BITSTR_PROG_STD,
                'Reset to standard EPM settings with line mode',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Main items
                'Main items',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'advancedmarking2',                                                 -- Advanced marking
                '~Advanced marking',
                'TogglePresetBit 1',
                ADVANCEDMARK_MENUP__MSG,
                HP_OPTIONS_CUATOGGLE,
                nodismiss)
   DefMenuItem( 'cuakeyset',                                                        -- CUA keyset
                '~CUA keyset',
                'TogglePresetBit 2',
                'Toggle between CUA keys and standard EPM keys',
                '',
                nodismiss)
   DefMenuItem( 'defaultstreammode2',                                               -- Stream mode
                '~Stream mode',
                'TogglePresetBit 3',
                'Toggle between stream and line editing mode',
                HP_OPTIONS_STREAM,
                nodismiss)
   DefMenuItem( 'cursoreverywhere2',                                                -- Allow cursor everywhere
                'Allow cu~rsor everywhere',
                'TogglePresetBit 4',
                'Cursor can be positioned after line end',
                '',
                nodismiss)
   DefMenuItem( 'homeendtoggle',                                                    -- Home and End keys toggle
                '~Home and End keys toggle',
                'TogglePresetBit 5',
                'Toggle between text and line boundaries',
                '',
                nodismiss)
   DefMenuItem( 'blockaltletterkeys2',                                              -- Block Alt+letter keys from jumping to menu bar
                '~Block Alt+letter keys from jumping to menu bar',
                'TogglePresetBit 6',
                'Enable for advanced mark operations (Ctrl+Alt+Shift+letter works for menu)',
                HP_OPTIONS_CUAACCEL,
                nodismiss)
   DefMenuItem( 'defaultsyntaxexpansion2',                                          -- Syntax expansion
                'S~yntax expansion',
                'TogglePresetBit 7',
                'Let '' and '' do syntax expansion',  -- Hint is updated on menu init
                '',
                nodismiss)
   DefMenuItem( 'matchbrackets',                                                    -- Match brackets
                '~Match brackets',
                'TogglePresetBit 8',
                'Highlight brackets and insert bracket pairs on typing',
                '',
                nodismiss)
   DefNextItemEndsSubMenu()
   DefMenuItem( 'modespecificsettings',                                             -- Mode-specific tabs and margins
                'M~ode-specific tabs and margins',
                'TogglePresetBit 9',
                'Allow to override default tabs and margins by mode settings',
                '',
                nodismiss)
   DefMenuSep()                                                               --------------------

   add_prefs_loadfiles()
   add_prefs_editfiles()
   add_prefs_saveandrestore()
   add_prefs_appearance()

   DefMenuSep()                                                               --------------------

   DefNextItemStartsSubMenu()
   DefMenuItem( '',                                                           -- User settings  >
                '~User settings',
                '',
                'Export and import settings',
                '',
                nodismiss)
   DefMenuItem( '',                                                                 -- Export user settings
                'Export ~user settings',
                'ExportConfig u \NEPMD\User 0',
                'Export user settings to a text file for later import',
                '',
                nodismiss)
   DefMenuItem( '',                                                                 -- Export history lists
                'Export ~history lists',
                'ExportHistory',
                'Export history lists to text files for later import',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Export user and default mode settings
                'Export user and default ~mode settings',
                'ExportConfig ud_mode \NEPMD\User\Mode\ 1',
                'Export user and default mode settings to a text file for later import',
                '',
                nodismiss)
   DefMenuItem( '',                                                                 -- Export user and default coding styles
                'Export user and default ~coding styles',
                'ExportConfig ud_coding \NEPMD\User\CodingStyles 1',
                'Export user and default coding styles to a text file for later import',
                '',
                nodismiss)
   DefMenuItem( '',                                                                 -- Export user and default settings
                'Export user and ~default settings',
                'ExportConfig ud \NEPMD\User 1',
                'Export user and default settings to a text file for later import',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Import user settings...
                '~Import user settings...',
                'ImportUserConfig',
                'Select a text file with user settings to import',
                '',
                nodismiss)
   DefMenuItem( '',                                                                 -- Import history list...
                'Import history ~list...',
                'ImportHistory',
                'Select a text file with history entries to import',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Delete user settings...
                '~Delete user settings...',
                'DeleteUserSettings',
                'Delete all user settings',
                '',
                nodismiss)
   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 -- Delete history lists...
                'D~elete history lists...',
                'DeleteHistoryLists',
                'Delete all existing Edit, Load, Save and Last lists',
                '',
                nodismiss)

if nodismiss <> 0 then
   DefMenuSep()                                                               --------------------

   DefMenuItem( '',                                                           -- Close menu
                '~Close menu',
                'mc ;SetFocusToEditClient;HighlightCursor',
                '',
                '')
endif  -- nodismiss <> 0

   showmenu menuname, 1  -- 0 = standard menu (default for missing parameter), 1 = popup menu, 2 = undefined, 3 = UC menu (toolbar)

   -- This highlights the cursor below the popup menu.
   'PostMe HighlightCursor'

; ---------------------------------------------------------------------------
defproc add_prefs_loadfiles
   universal nodismiss
   universal wpsstarted

   DefMenuItem( '',                                                           -- Load files
                'Load files',
                '',
                '',
                '',
                'STATIC')
   DefNextItemStartsSubMenu()
   DefMenuItem( 'editoptions',                                                -- Edit options  >
                '~Edit options: %1',
                '',
                'View/change default edit options',
                '',
                nodismiss)
   DefMenuItem( 'editoptions_b',                                                    --
                '/~b'\9'search both: ring and disk*',
                'EditOptionsMenuCmd /b',
                '',
                '',
                nodismiss)
   DefMenuItem( 'editoptions_c',                                                    --
                '/~c'\9'create a new file',
                'EditOptionsMenuCmd /c',
                '',
                '',
                nodismiss)
   DefMenuItem( 'editoptions_d',                                                    --
                '/~d'\9'create new if on disk',
                'EditOptionsMenuCmd /d',
                '',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'editoptions_nt',                                                   --
                '/~nt'\9'expand tabs (tabs = 8)',
                'EditOptionsMenuCmd /nt',
                '',
                '',
                nodismiss)
   DefMenuItem( 'editoptions_t',                                                    --
                '/~t'\9'don''t expand tabs*',
                'EditOptionsMenuCmd /t',
                '',
                '',
                nodismiss)
; In EPM 6 there's no difference between /u and /l anymore.
; EPM breaks lines at CRCRLF, CRLF, CR and LF, not dependent on /u or /l.
; EPM adds CRLF when Enter is pressed. That can't be changed with an option.
; /u and /l are senseless now.
; Per default all line ends are kept as on file loading. Even 'unterminated'
; is possible for the last line. (But it's not visible, if the last line is
; terminated or not. EPM won't add a blank line, if the last line is terminated.)
; Line ends can be forced to CRLF or CR on save. That applies also to the last line.
;    DefMenuSep()                                                                     --------------------
;
;    DefMenuItem( 'editoptions_u',                                                    --
;                 '/~u'\9'Unix line end (LF)',
;                 'EditOptionsMenuCmd /u',
;                 '',
;                 '')
;    DefMenuItem( 'editoptions_l',                                                    --
;                 '/~l'\9'DOS line end (CRLF)*',
;                 'EditOptionsMenuCmd /l',
;                 '',
;                 '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 --
                ''\9'~Reset to initial default (*)',
                'EditOptionsMenuCmd RESET',
                '',
                '',
                nodismiss)
   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 --
                ''\9'Sa~ve as default',
                'EditOptionsMenuCmd SAVE',
                '',
                '',
                NoDisMiss)
 if wpsstarted then
   DefNextItemStartsSubMenu()
   DefMenuItem( 'prg',                                                        -- Program objects  >
                'Program ob~jects',
                '',
                '',
                '',
                '')
   DefMenuItem( 'prgnewwindow',                                                     -- Open in same window
                'Open in ~same window (/r)',
                'toggle_new_same_window',
                'Open file objects in topmost EPM window',
                '',
                nodismiss)
   DefMenuItem( 'prgomitopenbox',                                                   -- File open selection: ~Omit open box (/o)
                'File open selection: ~Omit open box (/o)',
                'toggle_omit_openbox',
                'Show file dialog directly instead of history list first',
                '',
                nodismiss)
   DefMenuItem( '',                                                                 -- Set startup dir...
                'Set startup ~dir...',
                'StartupDirDlg',
                'Select startup dir for several EPM objects',
                '',
                '')
   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 -- Set associations...
                'Set ~associations...',
                'SelectAssoc',
                'Configure WPS associations for EPM objects',
                '',
                '')
 endif  -- wpsstarted
   DefNextItemStartsSubMenu()
   DefMenuItem( 'directories',                                                -- Directories  >
                '~Directories',
                '',
                '',
                '',
                '')
   DefNextItemStartsSubMenu()
   DefMenuItem( 'workdir',                                                          -- Set work dir  >
                'Set ~work dir',
                '',
                '',
                '',
                nodismiss)
   DefMenuItem( 'workdirprogram',                                                         -- By program object
                'By program ~object',
                'ChangeWorkDirMenuCmd 0',
                'This is EPM''s default',
                '',
                nodismiss)
   DefMenuItem( 'workdirprev',                                                            -- Use previous work dir
                'Use ~previous work dir',
                'ChangeWorkDirMenuCmd 1',
                'Keep work dir across EPM sessions',
                '',
                nodismiss)
   DefMenuItem( 'workdirfile',                                                            -- To dir of selected file
                'To dir of ~selected file',
                'ChangeWorkDirMenuCmd 2',
                'Change to dir of current file',
                '',
                nodismiss)
   DefMenuSep()                                                                           --------------------

   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                       -- To...
                '~To...',
                'CDDlg',
                'Show/change current work dir now',
                '')
   DefNextItemStartsSubMenu()
   DefMenuItem( 'opendlgdir',                                                       -- Start Edit/Add file dialog at  >
                '~Start Edit/Add file dialog at',
                '',
                '',
                '',
                nodismiss)
   DefMenuItem( 'opendlgdirprev',                                                         -- Previous dir
                '~Previous dir',
                'OpenDlgDirMenuCmd 0',
                'Start at dir from last Open dialog',
                '',
                nodismiss)
   DefMenuItem( 'opendlgdirwork',                                                         -- Work dir
                '~Work dir',
                'OpenDlgDirMenuCmd 1',
                'Start at work dir',
                '',
                nodismiss)
   DefNextItemEndsSubMenu()
   DefMenuItem( 'opendlgdirfile',                                                         -- Dir of current file
                '~Dir of current file',
                'OpenDlgDirMenuCmd 2',
                'Start at dir of current file',
                '',
                nodismiss)
   DefNextItemStartsSubMenu()
   DefMenuItem( 'saveasdlgdir',                                                     -- Start Save as dialog for virtual files at  >
                'Start Save-~as dialog for virtual files at',
                '',
                '',
                '',
                nodismiss)
   DefMenuItem( 'saveasdlgdirprev',                                                       -- Previous dir
                '~Previous dir',
                'SaveasDlgDirMenuCmd 0',
                'Start at dir from last saved file',
                '',
                nodismiss)
   DefNextItemEndsSubMenu()
   DefMenuItem( 'saveasdlgdirwork',                                                       -- Work dir
                '~Work dir',
                'SaveasDlgDirMenuCmd 1',
                'Start at work dir',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Edit MYEPM.ENV
                'Edit ~MYEPM.ENV',
                'EditCreateMyepmEnv',
                'Edit user environment file. This file is loaded additionally to EPM.ENV.',
                '',
                '')
   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 -- Edit EPM.ENV
                'Edit ~EPM.ENV',
                'EditCreateUserFile bin\epm.env',
                'Edit environment file',
                '',
                '')
   DefMenuSep()                                                               --------------------

; ---------------------------------------------------------------------------
defproc add_prefs_editfiles
   universal nodismiss
   universal wpsstarted

   DefMenuItem( '',                                                           -- Edit files
                'Edit files',
                '',
                '',
                '',
                'STATIC')
   DefNextItemStartsSubMenu()
   DefMenuItem( 'markingsettings',                                            -- Marking  >
                'Markin~g',
                '',
                '',
                '',
                nodismiss)
   DefMenuItem( 'advancedmarking',                                                  -- Advanced marking
                '~Advanced marking',
                'toggle_cua_mark',
                ADVANCEDMARK_MENUP__MSG,
                HP_OPTIONS_CUATOGGLE,
                nodismiss)
   DefMenuItem( 'defaultpaste',                                                     -- Default paste: [%1]
                'Default ~paste: [%1]',
                'toggle_default_paste',
                'Style for Sh+Ins/Alt+MB1, add Ctrl/Sh for alt. paste',
                '',
                nodismiss)
   DefMenuItem( 'shiftmarkextends',                                                 -- Sh-mark always extends mark
                '~Sh-mark always extends mark',
                'toggle_shift_mark_extends',
                'Extend mark always or just at boundaries',
                '',
                nodismiss)
   DefMenuItem( 'lineinsert',                                                       -- Insert marked lines: [%1]
                'Insert marked ~lines: [%1]',
                'toggle_lineinsert',
                'Line mark: insert below (standard EPM) or above current line',
                '',
                nodismiss)
   DefMenuItem( 'defaultmousemark',                                                 -- Default mouse mark: [%1]
                'Default m~ouse mark: [%1]',
                'toggle_defaultmousemark',
                'Mark definition for MB1, use Ctrl+MB1 or MB3 for alt. mark',
                '',
                nodismiss)
   DefMenuItem( 'dragalwaysmarks',                                                  -- Drag always marks
                '~Drag always marks',
                'toggle_drag_always_marks',
                'Every drag starts a new mark instead of a msg.',
                '',
                nodismiss)
   DefMenuItem( 'unmarkaftermove',                                                  -- Unmark after move
                '~Unmark after move',
                'toggle_unmark_after_move',
                'Unmark after doing a move mark',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'searchinmark',                                                     -- Search in multi-line mark if file is marked
                'Search in ~multi-line mark if file is marked',
                'toggle_SearchMarkIfMarked',
                'If file is marked, activate search in mark and from top',
                '',
                nodismiss)
   DefNextItemEndsSubMenu()
   DefMenuItem( 'searchmarkedwords',                                                -- Use marked words as search string
                'Use marked ~words as search string',
                'toggle_SearchMarkedWords',
                'If words are marked, overtake them as search string for the search dialog',
                '',
                nodismiss)

   DefNextItemStartsSubMenu()
   DefMenuItem( 'keysets',                                                    -- Keysets  >
                '~Keysets',
                '',
                'Configure key bindings',
                '',
                '')
   DefMenuItem( 'keydefs',                                                          -- Keyset additions: %1...
                'Keyset ~additions: %1...',
                'SelectKeyDefs',
                'Configure key def additions to the standard keyset',
                '',
                '')
   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 -- Edit STDKEYS.E
                '~Edit STDKEYS.E',
                'EditCreateUserMacro stdkeys.e',
                'Edit entire set of key definitions',
                '',
                '')

   DefNextItemStartsSubMenu()
   DefMenuItem( 'streamnewline',                                              -- Stream and Newline  >
                'Stream and Ne~wline',
                '',
                'Switch stream/line mode and define newline and enter',
                '',
                nodismiss)
   DefMenuItem( 'defaultstreammode',                                                -- Stream mode
                '~Stream mode',
                'toggle_default_stream',
                'Toggle between stream and line editing mode',
                HP_OPTIONS_STREAM,
                nodismiss)
   DefMenuItem( 'newlineenter',                                                     -- Configure [%1] keys:
                'Configure [%1] keys:',
                'toggle_newlineenter',
                '',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'newline',                                                          -- Newline in stream mode
                '',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'newlinesplit',                                                     -- Line and split: [%1]
                'Line and  split: [%1]',
                'toggle_newlinesplit newline',
                'Hold 'CTRL_KEY__MSG' to configure cmd',
                '',
                nodismiss)
   DefMenuItem( 'newlinecol',                                                       -- Cursor column: [%1]
                'Cursor column: [%1]',
                'toggle_newlinecol newline',
                '',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 's_newline',                                                        -- Sh+Newline in stream mode
                '',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 's_newlinesplit',                                                   -- Line and split: [%1]
                'Line and  split: [%1]',
                'toggle_newlinesplit s_newline',
                'Hold 'CTRL_KEY__MSG' to configure cmd',
                '',
                nodismiss)
   DefMenuItem( 's_newlinecol',                                                     -- Cursor column: [%1]
                'Cursor column: [%1]',
                'toggle_newlinecol s_newline',
                '',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'a_newline',                                                        -- Alt+Newline in stream mode
                '',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'a_newlinesplit',                                                   -- Line and split: [%1]
                'Line and  split: [%1]',
                'toggle_newlinesplit a_newline',
                'Hold 'CTRL_KEY__MSG' to configure cmd',
                '',
                nodismiss)
   DefMenuItem( 'a_newlinecol',                                                     -- Cursor column: [%1]
                'Cursor column: [%1]',
                'toggle_newlinecol a_newline',
                '',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'c_newline',                                                        -- Ctrl+Newline in stream mode
                '',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'c_newlinesplit',                                                   -- Line and split: [%1]
                'Line and  split: [%1]',
                'toggle_newlinesplit c_newline',
                'Hold 'CTRL_KEY__MSG' to configure cmd',
                '',
                nodismiss)
   DefMenuItem( 'c_newlinecol',                                                     -- Cursor column: [%1]
                'Cursor column: [%1]',
                'toggle_newlinecol c_newline',
                '',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 -- Reset all Newline and PadEnter key definitions
                'Reset all 'NEWLINE_KEY__MSG' and 'ENTER_KEY__MSG' key definitions',
                'ResetNewlineKeyDefs',
                '',
                '',
                nodismiss)

   DefNextItemStartsSubMenu()
   DefMenuItem( 'tabsandmargins',                                             -- Tabs and margins  >
                'Tab~s and margins',
                '',
                'Define behavior of special keys',
                '',
                nodismiss)
   DefMenuItem( '',                                                                 -- Tab and Backtab keys
                'Tab and Backtab keys',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'tab',                                                              -- Tab [%1]
                'Tab [%1]',
                'ChangeTab tab',
                'Change behavior of Tab',
                '',
                nodismiss)
   DefMenuItem( 'c_tab',                                                            -- Ctrl+Tab [%1]
                CTRL_KEY__MSG'+Tab [%1]',
                'ChangeTab c_tab',
                'Change behavior of 'CTRL_KEY__MSG'+Tab',
                '',
                nodismiss)
   DefMenuItem( 'c_a_tab',                                                          -- Ctrl+Alt+Tab [%1]
                CTRL_KEY__MSG'+'ALT_KEY__MSG'+Tab [%1]',
                'ChangeTab c_a_tab',
                'Change behavior of 'CTRL_KEY__MSG'+'ALT_KEY__MSG'+Tab',
                '',
                nodismiss)
   DefMenuItem( 's_backtab',                                                        -- Backtab [%1]
                'Backtab [%1]',
                'ChangeTab s_backtab',
                'Change behavior of Backtab',
                '',
                nodismiss)
   DefMenuItem( 'c_s_backtab',                                                      -- Ctrl+Backtab [%1]
                CTRL_KEY__MSG'+Backtab [%1]',
                'ChangeTab c_s_backtab',
                'Change behavior of 'CTRL_KEY__MSG'+Backtab',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Tab behavior
                'Tab behavior',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'defaultmatchtab',                                                  -- Matchtab: tab stops at word boundaries of lines above
                '~Matchtab: tab stops at word boundaries of lines above',
                'toggle_default_matchtab',
                'Tab and Backtab use word boundaries of prev. lines',
                '',
                nodismiss)
   DefMenuItem( 'showtabs',                                                         -- Show tab chars
                'Sho~w tab chars',
                'toggle_tabglyph',
                'Show a circle for every tab char',
                '',
                nodismiss)
   DefMenuItem( 'defaulttabs',                                                      -- Default tabs...
                'Default ~tabs: %1...%2',
                'DefaultTabs',
                'Change default tabs',
                '')
   DefMenuItem( 'modespecifictabs',                                                 -- Mode-specific tabs and indent
                'Mode-specific tabs and indent',
                'ToggleModeSpecific tabs',
                'Override default value by mode and coding style config',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Margins
                'Margins',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'joinafterwrap',                                                    -- Join after wrap on typing
                '~Join after wrap on typing',
                'Toggle_Join_After_Wrap',
                'Join wrapped part with next line',
                '',
                nodismiss)
   DefMenuItem( 'defaultmargins',                                                   -- Default margins...
                'Default ~margins: %1...%2',
                'DefaultMargins',
                'Change default margins',
                '')
   DefNextItemEndsSubMenu()
   DefMenuItem( 'modespecificmargins',                                              -- Mode-specific margins
                'Mode-specific margins',
                'ToggleModeSpecific margins',
                'Override default value by mode and coding style config',
                '',
                nodismiss)
   DefNextItemStartsSubMenu()
   DefMenuItem( 'specialkeys',                                                -- Special keys and mouse >
                'Special ke~ys and mouse',
                '',
                'Define behavior of special keys',
                '',
                nodismiss)
   DefMenuItem( '',                                                                 -- Cursor
                'Cursor',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'cursoreverywhere',                                                 -- Allow cursor everywhere
                '~Allow cursor everywhere',
                'toggle_cursor_everywhere',
                'Cursor can be positioned after line end',
                '',
                nodismiss)
   DefMenuItem( 'keepcursoronscreen',                                               -- Keep cursor on screen
                '~Keep cursor on screen',
                'toggle_keep_cursor_on_screen',
                'Keep cursor visible on scroll bar scrolling',
                '',
                nodismiss)
   DefMenuItem( 'vposafterlocate',                                                  -- V-position after locate: [%1]
                '~V-position after locate: [%1]',
                'SwitchVPosAfterLocate',
                'Select scroll type after a locate action',
                '',
                nodismiss)
   DefMenuItem( 'scrollafterlocate',                                                -- Scroll after locate: %1...
                'S~croll after locate: %1...',
                'SetScrollAfterLocate',
                'View found string at a special v-position',
                '')
   DefMenuItem( 'highlightcursor',                                                  -- Highlight cursor
                'H~ighlight cursor',
                'toggle_highlight_cursor',
                'Highlight cursor after several commands to make it better visible',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Mouse
                'Mouse',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'defaultmousemark2',                                                -- Default mouse mark: [char]
                'Default ~mouse mark: [%1]',
                'toggle_defaultmousemark',
                'Mark definition for MB1, use Ctrl+MB1 or MB3 for alt. mark',
                '',
                nodismiss)
   DefMenuItem( 'clickonlygivesfocus',                                              -- Click only gives focus
                '~Click only gives focus',
                'toggle_ClickOnlyGivesFocus',
                'When enabled, the cursor is placed at the second click only',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Home and End
                'Home and End',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'hometoggles',                                                      -- Home key toggles begin text and begin line
                '~Home key toggles begin text and begin line',
                'toggle_home_toggles',
                '',
                '',
                nodismiss)
   DefMenuItem( 'endtoggles',                                                       -- End key toggles end text and column after text
                '~End key toggles end text and column after text',
                'toggle_end_toggles',
                '',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Syntax expansion
                'Syntax expansion',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'syntaxexpansionkeys',                                              -- Keys for syntax expansion %1...
                'Keys for s~yntax expansion: %1...',
                'SelectExpandKeys',
                'Select e.g. "space newline" or "c_space c_newline"',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Alt
                'Alt',
                '',
                '',
                '',
                'STATIC')
   DefMenuItem( 'blockaltletterkeys',                                               -- Block Alt+letter keys from jumping to menu bar
                '~Block Alt+letter keys from jumping to menu bar',
                'accel_toggle',
                'Enable for advanced mark operations (Ctrl+Alt+Shift+letter works for menu)',
                HP_OPTIONS_CUAACCEL,
                nodismiss)
   DefMenuItem( 'blockleftaltkey',                                                  -- Block left Alt key from jumping to menu bar
                'Block ~left Alt key from jumping to menu bar',
                'toggle_block_left_alt_key',
                'When enabled, use F10',
                '',
                nodismiss)
   DefNextItemEndsSubMenu()
   DefMenuItem( 'blockrightaltkey',                                                 -- Block right Alt key from jumping to menu bar
                'Block ~right Alt key from jumping to menu bar',
                'toggle_block_right_alt_key',
                'When enabled, use F10',
                '',
                nodismiss)

   DefNextItemStartsSubMenu()
   DefMenuItem( 'windowsdialogs',                                             -- Windows and dialogs  >
                'Wind~ows and dialogs',
                '',
                '',
                '',
                nodismiss)
   DefMenuItem( 'clickonlygivesfocus2',                                             -- Click only gives focus
                '~Click only gives focus',
                'toggle_ClickOnlyGivesFocus',
                'When enabled, the cursor is placed at the second click only',
                '',
                nodismiss)
   DefMenuItem( 'focustoeditwindow',                                                -- Search sets focus to edit window
                'Search sets focus to ~edit window',
                'toggle_FocusToEditWindow',
                'When enabled, a search dialog action makes the edit window active',
                '',
                nodismiss)
   DefMenuItem( 'searchinmark2',                                                    -- Search in multi-line mark if file is marked
                'Search in ~multi-line mark if file is marked',
                'toggle_SearchMarkIfMarked',
                'If file is marked, activate search in mark and from top',
                '',
                nodismiss)
   DefMenuItem( 'searchmarkedwords2',                                               -- Use marked words as search string
                'Use marked ~words as search string',
                'toggle_SearchMarkedWords',
                'If words are marked, overtake them as search string for the search dialog',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefNextItemEndsSubMenu()
   DefMenuItem( 'startwithmaxcols',                                                 -- Start with maximal %1 columns
                '|No window width ~limitation...|Start with maxima~l %1 columns...',
                'LimitEditWindowWidth',
                'Limit window width on startup, 0 for no limitation',
                '')

   DefNextItemStartsSubMenu()
   DefMenuItem( 'modeexpansion',                                              -- Mode and expansion  >
                '~Mode and expansion',
                'Configure mode-dependent settings',
                '',
                '',
                '')
   DefMenuItem( 'matchfindonmove',                                                  -- MatchFindOnMove: match-find bracket on cursor move
                'MatchFindOn~Move: match-find bracket on cursor move',
                'toggle_MatchFindOnMove',
                'Highlight brackets on any typing',
                '',
                nodismiss)
   DefMenuItem( 'matchfindopening',                                                 -- MatchFindOpening: match-find opening bracket on typing closing one
                'MatchFind~Opening: match-find opening bracket on typing closing one',
                'toggle_MatchFindOpening',
                'Highlight opening bracket on typing the closing one',
                '',
                nodismiss)
   DefMenuItem( 'matchinsertpair',                                                  -- MatchInsertPair: insert bracket pair on typing opening one
                'MatchInsert~Pair: insert bracket pair on typing opening one',
                'toggle_MatchInsertPair',
                'Add closing bracket when typing opening one',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'defaultkeywordhighlighting',                                       -- Keyword highlighting
                'Keyword ~highlighting',
                'toggle_default_highlight',
                'Switch keyword highlighting on (Ctrl to reload keyword file)',
                '',
                nodismiss)
   DefMenuItem( 'openhilitedirs',                                                   -- Open hilite directories for mode CURMODE
                'Open hilite ~directories for mode %1',
                'OpenHiliteDirs',
                'Open dir(s) with .hil/.ini files',
                '',
                nodismiss)
   DefMenuItem( 'hilitecachedirdialog',                                             -- Hilite cache directory: %1...
                'Hilite ~cache directory: %1...',
                'HiliteCacheDir',
                '',
                '',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'defaultsyntaxexpansion',                                           -- Syntax expansion
                '~Syntax expansion',
                'toggle_default_expand',
                'Let '' and '' do syntax expansion',  -- Hint is updated on menu init
                '',
                nodismiss)
   DefMenuItem( 'selectcodingstyle',                                                -- Select coding style (%1) for mode %2...
                'Select ~coding style (%1) for mode %2...',
                'SelectCodingStyle',
                'Select a previously defined coding style for current mode',
                '',
                '')
   DefMenuItem( '',                                                                 -- Export user and default coding styles
                'Export user and default ~coding styles',
                'ExportConfig ud_coding \NEPMD\User\CodingStyles',
                'Export user and default coding styles to a text file for later import',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 -- Export user and default mode settings
                'Export user and default ~mode settings',
                'ExportConfig ud_mode \NEPMD\User\Mode\',
                'Export user and default mode settings to a text file for later import',
                '',
                nodismiss)

   DefNextItemStartsSubMenu()
   DefMenuItem( 'searchoptions',                                              -- Search options  >
                'Sear~ch options: %1',
                '',
                'View/change default search options',
                '',
                nodismiss)
   DefMenuItem( 'searchoptions_+',                                                  --
                '~+'\9'down: top to bottom*',
                'SearchOptionsMenuCmd +',
                '',
                '',
                nodismiss)
   DefMenuItem( 'searchoptions_-',                                                  --
                '~-'\9'up: bottom to top',
                'SearchOptionsMenuCmd -',
                '',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'searchoptions_f',                                                  --
                '~f'\9'foreward: left to right*',
                'SearchOptionsMenuCmd f',
                '',
                '',
                nodismiss)
   DefMenuItem( 'searchoptions_r',                                                  --
                '~r'\9'reverse: right to left',
                'SearchOptionsMenuCmd r',
                '',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'searchoptions_u',                                                  --
                ''\9'start at c~ursor*',
                'SearchOptionsMenuCmd u',
                '',
                '',
                nodismiss)
   DefMenuItem( 'searchoptions_t',                                                  --
                '~t'\9'start at top of file',
                'SearchOptionsMenuCmd t',
                '',
                '',
                nodismiss)
   DefMenuItem( 'searchoptions_b',                                                  --
                '~b'\9'start at bottom of file',
                'SearchOptionsMenuCmd b',
                '',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'searchoptions_a',                                                  --
                '~a'\9'all: in the whole file*',
                'SearchOptionsMenuCmd a',
                '',
                '',
                nodismiss)
   DefMenuItem( 'searchoptions_m',                                                  -- Search in mark only is not useful as default option
                '~m'\9'mark: in mark only',
                'SearchOptionsMenuCmd m',
                '',
                '',
                nodismiss 'DISABLED')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'searchoptions_e',                                                  --
                '~e'\9'case-sensitive',
                'SearchOptionsMenuCmd e',
                '',
                '',
                nodismiss)
   DefMenuItem( 'searchoptions_c',                                                  --
                '~c'\9'ignore case',
                'SearchOptionsMenuCmd c',
                '',
                '',
                nodismiss)
   DefMenuItem( 'searchoptions_o',                                                  --
                '~o'\9'automatic case*',
                'SearchOptionsMenuCmd o',
                '',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'searchoptions_i',                                                  --
                ''\9'~including*',
                'SearchOptionsMenuCmd i',
                '',
                '',
                nodismiss)
   DefMenuItem( 'searchoptions_^',                                                  -- options ~ and ^ are equivalent
                '~^'\9'excluding (negated)',
                'SearchOptionsMenuCmd ^',
                '',
                '',
                nodismiss)
                                                                                                   --------------------------
   DefMenuItem( 'searchoptions_h',                                                  --
                ''\9'c~hars*',
                'SearchOptionsMenuCmd h',
                '',
                '',
                'BREAKSEPARATOR' nodismiss)
   DefMenuItem( 'searchoptions_g',                                                  --
                '~g'\9'grep',
                'SearchOptionsMenuCmd g',
                '',
                '',
                nodismiss)
   DefMenuItem( 'searchoptions_x',                                                  --
                '~x'\9'egrep',
                'SearchOptionsMenuCmd x',
                '',
                '',
                nodismiss)
   DefMenuItem( 'searchoptions_w',                                                  --
                '~w'\9'words',
                'SearchOptionsMenuCmd w',
                '',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'searchoptions_p',                                                  --
                ''\9'change: re~place sets case*',
                'SearchOptionsMenuCmd p',
                '',
                '',
                nodismiss)
   DefMenuItem( 'searchoptions_k',                                                  --
                '~k'\9'change: keep case of search',
                'SearchOptionsMenuCmd k',
                '',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'searchoptions_1',                                                  --
                ''\9'change ~1 only*',
                'SearchOptionsMenuCmd 1',
                '',
                '',
                nodismiss)
   DefMenuItem( 'searchoptions_*',                                                  --
                '~*'\9'change all',
                'SearchOptionsMenuCmd *',
                '',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'searchoptions_q',                                                  --
                ''\9'change: ~quiet*',
                'SearchOptionsMenuCmd q',
                '',
                '',
                nodismiss)
   DefMenuItem( 'searchoptions_n',                                                  --
                '~n'\9'change: msg how many changes',
                'SearchOptionsMenuCmd n',
                '',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 --
                ''\9'Reset to initial ~default (*)',
                'SearchOptionsMenuCmd RESET',
                '',
                '',
                nodismiss)
   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 --
                ''\9'Sa~ve as default',
                'SearchOptionsMenuCmd SAVE',
                '',
                '',
                nodismiss)

   DefMenuSep()                                                               --------------------

; ---------------------------------------------------------------------------
defproc add_prefs_saveandrestore
   universal nodismiss
   universal wpsstarted

   DefMenuItem( '',                                                           -- Save and restore
                'Save and restore',
                '',
                '',
                '',
                'STATIC')
   DefNextItemStartsSubMenu()
   DefMenuItem( 'saveoptions',                                                -- Save options  >
                'Sa~ve options: %1',
                '',
                'View/change default save options',
                '',
                nodismiss)
   DefMenuItem( 'saveoptions_ns',                                                   --
                '/ns'\9'~don''t strip spaces',
                'SaveOptionsMenuCmd /ns',
                '',
                '',
                nodismiss)
   DefMenuItem( 'saveoptions_s',                                                    --
                '/~s'\9'strip trailing spaces*',
                'SaveOptionsMenuCmd /s',
                '',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'saveoptions_nt',                                                   --
                '/nt'\9'don''t compress s~paces*',
                'SaveOptionsMenuCmd /nt',
                '',
                '',
                nodismiss)
   DefMenuItem( 'saveoptions_t',                                                    --
                '/~t'\9'compress spaces to tabs (tabs = 8, buggy!)',
                'SaveOptionsMenuCmd /t',
                '',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'saveoptions_ne',                                                   --
                '/~ne'\9'no file end char*',
                'SaveOptionsMenuCmd /ne',
                '',
                '',
                nodismiss)
   DefMenuItem( 'saveoptions_e',                                                    --
                '/~e'\9'append a file end char',
                'SaveOptionsMenuCmd /e',
                '',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'saveoptions_a',                                                    --
                ''\9'~auto-line-end',
                'SaveOptionsMenuCmd /a',
                '',
                '',
                nodismiss)
   DefMenuItem( 'saveoptions_o',                                                    --
                '/~o'\9'force DOS line end (CRLF)*',
                'SaveOptionsMenuCmd /o',
                '',
                '',
                nodismiss)
   DefMenuItem( 'saveoptions_l',                                                    --
                '/~l'\9'force Unix line end (LF)',
                'SaveOptionsMenuCmd /l',
                '',
                '',
                nodismiss)
                              -- /u is the same as /l /ne
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 --
                ''\9'Reset to initial ~default (*)',
                'SaveOptionsMenuCmd RESET',
                '',
                '',
                nodismiss)
   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 --
                ''\9'Sa~ve as default',
                'SaveOptionsMenuCmd SAVE',
                '',
                '',
                nodismiss)

   DefNextItemStartsSubMenu()
   DefMenuItem( 'readonlyandlock',                                            -- Read-only and lock   >
                '~Read-only and lock',
                '',
                '',
                '',
                nodismiss)
   DefMenuItem( 'respectreadonly',                                                  -- Respect read-only
                '~Respect read-only',
                'toggle_respect_readonly',
                'Read-only file attribute disables edit mode',
                '',
                nodismiss)
   DefNextItemEndsSubMenu()
   DefMenuItem( 'lockonmodify',                                                     -- Lock on modify
                '~Lock on modify',
                'toggle_lock_on_modify',
                'Deny write access for other applications',
                '',
                nodismiss)

   DefNextItemStartsSubMenu()
   DefMenuItem( 'autorestore',                                                -- Auto-restore  >
                '~Auto-restore',
                '',
                '',
                '',
                nodismiss)
   DefMenuItem( 'restorecursorpos',                                                 -- Restore cursor position
                '~Restore cursor position',
                'toggle_restore_pos',
                'Restore of cursor pos. from file''s last save',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'trackhistorylists',                                                -- Track additional history lists
                '~Track additional history lists',
                'Toggle_History',
                'Enable edit, load and save history',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'autosavelastring',                                                 -- Auto-save last rings
                'Auto-~save last rings',
                'Toggle_Save_Ring',
                'Save last ring on load and quit, save all last rings on opening a new window',
                '',
                nodismiss)
   DefNextItemEndsSubMenu()
   DefMenuItem( 'autoloadlastring',                                                 -- Auto-load last rings
                'Auto-~load last rings',
                'Toggle_Restore_Ring',
                'Restore all previously saved last rings if the first EPM is started without args',
                '',
                nodismiss)
   DefNextItemStartsSubMenu()
   DefMenuItem( 'backup',                                                     -- Backup  >
                '~Backup',
                '',
                '',
                '',
                '')
   DefMenuItem( 'autosaveenabled',                                                  -- Autosave
                '~Autosave',
                'toggle_autosave',
                '',
                '',
                nodismiss)
   DefMenuItem( 'autosavenumdialog',                                                -- After %1 changes...
                'After %1 ~changes...',
                'AutosaveNum',
                '',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'backupenabled',                                                    -- Backup
                '~Backup',
                'toggle_backup',
                '',
                '',
                nodismiss)
   DefMenuItem( 'backupnumdialog',                                                  -- Keep %1 backups...
                '~Keep %1 backups...',
                'BackupNum',
                '',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( 'backupdirdialog',                                                  -- Directory: %1...
                '~Directory: %1...',
                'BackupDir',
                '',
                '',
                '')
   DefNextItemEndsSubMenu()
   DefMenuItem( 'backuplistdir',                                                    -- List directory
                '~List directory',
                'ListBackupDir',
                '',
                '',
                '')

   DefMenuSep()                                                               --------------------

; ---------------------------------------------------------------------------
defproc add_prefs_appearance
   universal nodismiss
   universal wpsstarted

   DefMenuItem( '',                                                           -- Appearance
                'Appearance',
                '',
                '',
                '',
                'STATIC')
   DefNextItemStartsSubMenu()
   DefMenuItem( 'menu',                                                       -- Menu   >
                'Me~nu',
                '',
                '',
                '',
                '')
   DefMenuItem( '',                                                                 -- Select menu
                'Select ~menu...',
                'ChangeMenu',
                'Open a listbox and change or refresh the menu',
                '',
                '')
   DefNextItemEndsSubMenu()
   DefMenuItem( 'nodismiss',                                                        -- Nodismiss menus
                '~Nodismiss menus',
                'toggle_nodismiss',
                'Keep menu open after selecting menu items',
                '',
                nodismiss)
   DefNextItemStartsSubMenu()
   DefMenuItem( 'infobars',                                                   -- Info bars   >
                '~Info bars',
                '',
                '',
                '',
                '')
   DefMenuItem( 'showlongname',                                                     -- Show .LONGNAME
                'Show .~LONGNAME',
                'toggle_longname',
                'Show .LONGNAME EA as filename in titlebar',
                '',
                nodismiss)
   DefMenuItem( 'messageline',                                                      -- Message line
                MSG_LINE_MENU__MSG,
                'ToggleFrame 2',
                MSG_LINE_MENUP__MSG,
                HP_FRAME_MESSAGE,
                nodismiss)
   DefMenuItem( 'statusbar',                                                        -- Status bar
                'Status b~ar',
                'ToggleFrame 1',
                STATUS_LINE_MENUP__MSG,
                HP_FRAME_STATUS,
                nodismiss)
   DefMenuItem( 'infoattop',                                                        -- Info at top
                INFOATTOP_MENU__MSG,
                'ToggleFrame 32',
                INFOATTOP_MENUP__MSG,
                HP_FRAME_EXTRAPOS,
                nodismiss)
   DefMenuItem( 'showmenuhints',                                                    -- Show menu hints
                'Show menu ~hints',
                'toggleprompt',
                PROMPTING_MENUP__MSG,
                HP_FRAME_PROMPT,
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Configure title bar...
                'Configure ~title bar...',
                'ConfigInfoLine TITLE',
                'Change layout of titletext',
                '',
                '')
   DefMenuItem( '',                                                                 -- Configure status bar...
                'Configure status ~bar...',
                'ConfigInfoLine STATUS',
                'Change layout of status bar',
                '',
                '')
   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 -- Configure separator...
                'Configure ~separator...',
                'ConfigInfoLine SEP',
                'Change layout of separator for title and status bar',
                '',
                '')
   DefNextItemStartsSubMenu()
   DefMenuItem( 'toolbar',                                                    -- Toolbar   >
                '~Toolbar',
                '',
                '',
                '',
                '')
   DefMenuItem( 'toolbarenabled',                                                   -- Enabled
                '~Enabled',
                'toggle_toolbar',
                TOGGLETOOLBAR_MENUP__MSG,
                HP_TOOLBAR_TOGGLE,
                nodismiss)
   DefMenuItem( '',                                                                 -- Select...
                '~Select...',
                'LoadToolbar',
                'Open a listbox and load, reload or delete a toolbar',
                '',
                '')
   DefMenuSep()                                                                     --------------------

   DefNextItemStartsSubMenu()
   DefMenuItem( 'toolbarstyle',                                                     -- Style   >
                'St~yle',
                '',
                'Configure toolbar style',
                '',
                nodismiss)
   DefMenuItem( 'toolbartext',                                                            -- Text
                '~Text',
                'toggle_toolbar_text',
                'Show button text',
                '',
                nodismiss)
   DefMenuSep()                                                                           --------------------

   DefMenuItem( 'toolbarautosize',                                                        -- Automatic size
                '~Automatic size',
                'toggle_toolbar_autosize',
                'Adjust button sizes to the .bmp sizes',
                '',
                nodismiss)
   DefMenuItem( 'toolbarsize',                                                            -- Size: 26x26...
                '~Size: %1...',
                'ToolbarSize',
                'Default = 26x26, add 4x4 to the .bmp size',
                '',
                nodismiss)
   DefNextItemEndsSubMenu()
   DefMenuItem( 'toolbarscaling',                                                         -- Scaling: [and]
                'S~caling: [%1]',
                'toggle_toolbar_scaling',
                'In most cases "and" looks best',
                '',
                nodismiss)
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Save as...
                'Save ~as...',
                'SaveToolbar',
                '',
                '',
                '')
   DefMenuSep()                                                                     --------------------

   DefMenuItem( '',                                                                 -- Import...
                '~Import...',
                'ImportToolbar',
                '',
                '',
                '')
   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 -- Export...
                'E~xport...',
                'ExportToolbar',
                '',
                '',
                '')
   DefNextItemStartsSubMenu()
   DefMenuItem( '',                                                           -- Cursor and pointer   >
                'Cursor and ~pointer',
                '',
                '',
                '',
                nodismiss)
   DefNextItemStartsSubMenu()
   DefMenuItem( 'cursorstyle',                                                      -- Cursor style   >
                '~Cursor style',
                '',
                '',
                '',
                nodismiss)
   DefMenuItem( 'cursorbar',                                                              -- Bar
                '~Bar',
                'CursorStyle' 1,
                'Vertical bar in insert and block in overwrite mode',
                '',
                nodismiss)
   DefMenuItem( 'cursorbarwidth',                                                         -- Bar width
                'Bar ~width: %1...',
                'CursorWidth',
                'Enter bar width',
                '')
   DefMenuItem( 'cursorunderline',                                                        -- Underline
                '~Underline',
                'CursorStyle' 2,
                'Underline in insert and half block in overwrite mode',
                '',
                nodismiss)
   DefMenuItem( 'cursorspecial',                                                          -- Special
                '~Special',
                'CursorStyle' 3,
                'Special cursor dimensions',
                '',
                nodismiss)
   DefNextItemEndsSubMenu()
   DefMenuItem( 'cursorspecialdimen',                                                     -- Dimensions: %1...
                '~Dimensions: %1...',
                'CursorStyle ?',
                'Set special cursor dimensions',
                '')
   DefNextItemEndsSubMenu()
   DefNextItemStartsSubMenu()
   DefMenuItem( 'pointerstyle',                                                     -- Pointer style   >
                '~Pointer style',
                '',
                '',
                '',
                nodismiss)
   DefMenuItem( 'pointerarrow',                                                           -- Arrow
                '~Arrow',
                'PointerStyle' 1,
                'Arrow pointer',
                '',
                nodismiss)
   DefNextItemEndsSubMenu()
   DefMenuItem( 'pointeribeam',                                                           -- I-beam
                '~I-beam',
                'PointerStyle' 2,
                'I-beam pointer',
                '',
                nodismiss)
   DefNextItemStartsSubMenu()
   DefMenuItem( 'backgroundbitmap',                                           -- Background bitmap   >
                'Bac~kground bitmap',
                '',
                '',
                '',
                '')
   DefMenuItem( 'backgroundbitmapenabled',                                          -- Enabled
                '~Enabled',
                'toggle_bitmap',
                TOGGLEBITMAP_MENUP__MSG,
                HP_FRAME_BITMAP,
                nodismiss)
   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 -- Select...
                '~Select...',
                'SetBackgroundBitmap SELECT',
                'Select a background bitmap',
                '',
                '')
   DefNextItemStartsSubMenu()
   DefMenuItem( '',                                                           -- Colors and fonts   >
                'Co~lors and fonts',
                '',
                '',
                '',
                '')
   DefMenuItem( '',                                                                 -- Change controls colors...
                'Change controls ~colors...',
                'OpenObject <WP_LORESCLRPAL>',
                'To change a foreground color, hold the Ctrl key',
                '')
   DefMenuItem( '',                                                                 -- Change controls fonts...
                'Change controls ~fonts...',
                'OpenObject <WP_FNTPAL>',
                '',
                '')
   DefMenuItem( '',                                                                 -- Old colors and fonts dialog...
                'Old colors and fonts ~dialog...',
                'PostMe ConfigDlg' 1023 - 2 - 16,  -- 2 = color page bit, 16 = fonts page bit, 1023 = sum for all pages
                'Use either the old configuration dialog or drag''n''drop',
                '',
                '')
   DefMenuSep()                                                                     --------------------

   DefNextItemEndsSubMenu()
   DefMenuItem( '',                                                                 -- Change highlighting color palette...
                'Change highlighting color ~palette...',
                'SelectColorPal',
                'Modify EPM''s 16-color palette (e.g. used for highlighting)',
                '',
                '')

; ---------------------------------------------------------------------------
; Syntax for adding menu itens:
;    'CascadeMenu' <submenu_mid> [<default_menuitem_mid>]
; Better use the optional executed-as-default submenu id here, to have that
; submenu item checked automatically.
; Otherwise the 1st submenu item is executed, but MIA_CHECKED is missing.
; This is executed by MENU.E:"defproc showmenu_activemenu".
; It has to be executed posted.
; It doesn't work reliable for ETK popup menus: The rectangle around the
; arrow of the submenu is only painted after activating that item, which is
; usually after a mouse click on it. The default item is executed then,
; bbefore it gets clear that the submenu has a default item.
; It works for the main (default) menu.
; The CascadeMenu def was extended to accept both menu item ids and menu
; item names.
defc AddCascadeMenus, Add_Cascade_Menus
   if isadefc( 'CascadeMenu') then
      CascadeCmd = 'CascadeMenu'
   else
      CascadeCmd = 'Cascade_Menu'
   endif
compile if SUPPORT_USERS_GUIDE
   CascadeCmd' viewusersguide usersguide'                        -- Help -> View User's Guide
compile endif
compile if SUPPORT_TECHREF
   CascadeCmd' viewtechnicalreference technicalreference'        -- Help -> View Technical Reference
compile endif
   CascadeCmd' viewnepmdusersguide nepmdusersguide'              -- Help -> NEPMD User Guide
   CascadeCmd' viewnepmdprogrammingguide nepmdprogrammingguide'  -- Help -> NEPMD Programming Guide
   CascadeCmd' keywordhelp keywordhelpcurrentword'               -- Help -> Keyword help
   -- CUSTEPM package
compile if defined( CUSTEPM_DEFAULT_SCREEN)
   CascadeCmd' 3700 (CUSTEPM_DEFAULT_SCREEN + 3700)              -- Host screen -> Screen?
compile elseif defined( HAVE_CUSTEPM)
   CascadeCmd' 3700 3701  -- ensure that the MIA_CHECKED is painted; 1st item is default automatically
compile endif
   -- Execute hook for external packages
   'HookExecute cascademenu'

; ---------------------------------------------------------------------------------------
; The menuinit_<mid_name> is called by defc ProcessMenuInit, when the menu id <mid_name>
; is selected. The defc must exist and must be added to the 'definedsubmenus' array var,
; see the SetAVar( 'definedsubmenus', <list of names>) definition at the top.

; ------------------------------------ File ---------------------------------
defc menuinit_file
   SetMenuAttribute( 'importfile',  MIA_DISABLED, .readonly = 0)
   SetMenuAttribute( 'save',        MIA_DISABLED, .readonly = 0)
   SetMenuAttribute( 'saveandquit', MIA_DISABLED, .readonly = 0)

   on = (marktype() <> '')
   SetMenuAttribute( 'printmark',   MIA_DISABLED, on)

defc menuinit_fileproperties
   universal stream_mode
   universal expand_on
   universal matchtab_on
compile if CHECK_FOR_LEXAM
   universal LEXAM_is_available
compile endif

compile if CHECK_FOR_LEXAM
   if LEXAM_is_available then
compile endif
   universal activeaccel
   on = (activeaccel = 'spell')
   SetMenuAttribute( 'autospellcheck', MIA_CHECKED, not on)
compile if CHECK_FOR_LEXAM
   endif
compile endif

   SetMenuAttribute( 'readonly',       MIA_CHECKED, not .readonly)

   SetMenuAttribute( 'readonlyattrib', MIA_CHECKED, not GetReadonly())

   SetMenuAttribute( 'locked',         MIA_CHECKED, not .lockhandle)

   new = Exist( .filename)
   SetMenuAttribute( 'readonly',       MIA_DISABLED, new)
   SetMenuAttribute( 'readonlyattrib', MIA_DISABLED, new)
   SetMenuAttribute( 'locked',         MIA_DISABLED, new)
   SetMenuAttribute( 'wpsproperties',  MIA_DISABLED, new)

   new = GetMode()
   SetMenuVarText( 'mode', new)
   new = .tabs
   SetMenuVarText( 'tabs', new)
   new = .margins
   SetMenuVarText( 'margins', new)

; ------------------------------------ Edit ---------------------------------
defc menuinit_edit
   universal DMbuf_handle

   SetMenuAttribute( 'undoline',          MIA_DISABLED, isadirtyline())

   'DisableUndoMenuItems'

   on = (GetAVar( 'undokeepscrollpos') = 1)
   SetMenuAttribute( 'undokeepscrollpos', MIA_CHECKED, not on)

   SetMenuAttribute( 'recovermarkdelete', MIA_DISABLED, DMbuf_handle)
   SetMenuAttribute( 'discardchanges',    MIA_DISABLED, .modify > 0)

defc Undo1MenuCmd
   'Undo1'
   'DisableUndoMenuItems'

defc Redo1MenuCmd
   'Redo1'
   'DisableUndoMenuItems'

defc DisableUndoMenuItems
   parse value QueryUndoState() with OldestState NewestState
   parse value GetFileAVar( 'undostaterange') with LastPresentState LastOldestState LastNewestState
   if LastNewestState = NewestState & LastOldestState = OldestState then
      PresentState = LastPresentState
   else
      PresentState = NewestState
   endif
   if OldestState = NewestState then
      fUndoPossible = (.modify <> 0)
      fRedoPossible = 0
   else
      fUndoPossible = (OldestState < PresentState)
      fRedoPossible = (PresentState < NewestState)
   endif
   SetMenuAttribute( 'undolast', MIA_DISABLED, fUndoPossible)
   SetMenuAttribute( 'redonext', MIA_DISABLED, fRedoPossible)
   SetMenuAttribute( 'undo',     MIA_DISABLED, fUndoPossible | fRedoPossible)

defc ToggleUndoScrollPos
   on = (GetAVar( 'undokeepscrollpos') = 1)
   on = not on
   SetAVar( 'undokeepscrollpos', on)
   SetMenuAttribute( 'undokeepscrollpos', MIA_CHECKED, not on)

defc menuinit_insert
   SetMenuAttribute( 'insertsum', MIA_DISABLED, marktype() = 'BLOCK')

defc menuinit_move
   fMark = MarkIsExpression()
   SetMenuCondText( 'moveexpressionleft',  not fMark, fMark)
   SetMenuCondText( 'moveexpressionright', not fMark, fMark)

; ------------------------------------ Mark ---------------------------------
defc menuinit_mark
   universal DMbuf_handle
   universal CUA_marking_switch

   paste = ClipCheck( format) & (format = 1024) & not (browse() | .readonly)
   SetMenuAttribute( 'paste',       MIA_DISABLED, paste)
   SetMenuAttribute( 'pastelines',  MIA_DISABLED, paste)
   SetMenuAttribute( 'pasteblock',  MIA_DISABLED, paste)
   on = (marktype() <> '')
   buf_flag = 0
   if not on then                                     -- Only check buffer if no mark
      bufhndl = buffer( OPENBUF, EPMSHAREDBUFFER)
      if bufhndl then                                -- If the buffer exists, check the
         buf_flag = itoa( peek( bufhndl, 2, 2), 10)  -- amount of used space in buffer
         call buffer( FREEBUF, bufhndl)              -- then free it.
      endif
   endif
   SetMenuAttribute( 'copy', MIA_DISABLED, on)
   SetMenuAttribute( 'cut',  MIA_DISABLED, on)
   if CUA_marking_switch then
      cua_on = 1
   else
      cua_on = 0
   endif
   SetMenuAttribute( 'advancedmarking3', MIA_CHECKED, cua_on)
   SetMenuAttribute( 'copymark',    MIA_DISABLED, (on & not cua_on) | buf_flag)  -- Can copy if mark or buffer has data
   SetMenuAttribute( 'movemark',    MIA_DISABLED, (on & not cua_on))
   SetMenuAttribute( 'overlaymark', MIA_DISABLED, (on & not cua_on) | buf_flag)  -- Ditto for Overlay mark
   SetMenuAttribute( 'adjustmark',  MIA_DISABLED, (on & not cua_on))
   SetMenuAttribute( 'unmark',      MIA_DISABLED, on)
   SetMenuAttribute( 'deletemark',  MIA_DISABLED, on)
   SetMenuAttribute( 'fillmark',    MIA_DISABLED, on)

defc menuinit_createmark
   universal CUA_marking_switch

   on = (FileIsMarked() & marktype() = 'CHAR' & not CUA_marking_switch)
   SetMenuAttribute( 'extendsentence',  MIA_DISABLED, on)
   SetMenuAttribute( 'extendparagraph', MIA_DISABLED, on)
   SetMenuAttribute( 'markchars',       MIA_DISABLED, not CUA_marking_switch)
   SetMenuAttribute( 'markblock',       MIA_DISABLED, not CUA_marking_switch)
   SetMenuAttribute( 'marklines',       MIA_DISABLED, not CUA_marking_switch)

; ------------------------------------ Format -------------------------------
defc menuinit_format
   SetMenuAttribute( 'sort', MIA_DISABLED, .last > 1)

defc menuinit_lineendchars
   universal default_save_options

   CurType = QueryFileLineEnd()
   SetMenuVarText( 'changelineend', CurType)
   SetMenuAttribute( 'doslineend',  MIA_DISABLED, not (lowcase( CurType) = 'unterm'))
   SetMenuAttribute( 'unixlineend', MIA_DISABLED, not (lowcase( CurType) = 'unterm'))

   Options = '/a /ns /ne /nt 'default_save_options  -- internal default + default_save_options
   Options = ProcessAvoidSaveOptions( Options)
   if pos( '/o ', Options' ') then
      SaveType = 'DOS'
   elseif pos( '/l ', Options' ') then
      SaveType = 'Unix'
   else
      SaveType = 'auto'
   endif
   if CurType = 'unterm' then
      SaveType = 'unterm'
   endif
   SetMenuVarText( 'format_saveoptions_lineend', CurType, SaveType)

   SelectedLineEnd = GetAVar( 'lineendoption')
   if SelectedLineEnd = '' then
      -- Init to Save option
      if lowcase( SaveType) = 'unix' then
         SelectedLineEnd = 'Unix'
      else
         SelectedLineEnd = 'DOS'
      endif
      SetAVar( 'lineendoption', SelectedLineEnd)
   endif
   SetMenuAttribute( 'doslineend',  MIA_CHECKED, not (lowcase( SelectedLineEnd) = 'dos'))
   SetMenuAttribute( 'unixlineend', MIA_CHECKED, not (lowcase( SelectedLineEnd) = 'unix'))

defc ToggleLineEndOption
   SelectedLineEnd = strip( arg( 1))
   SetAVar( 'lineendoption', SelectedLineEnd)
   'menuinit_lineendchars'

const
  DEFAULT_EMPTYLINES_OPTIONS = '0 1 0 0 0'

defc menuinit_emptylines
   EmptyLinesOptions = GetAVar( 'emptylinesoptions')
   --dprintf( 'menuinit_emptylines: EmptyLinesOptions = 'EmptyLinesOptions)
   -- Init
   if words( EmptyLinesOptions) <> 5 then
      EmptyLinesOptions = DEFAULT_EMPTYLINES_OPTIONS
   endif
   parse value EmptyLinesOptions with fMark fMulti fAll fAdd fSingle .

   -- Avoid all types unchecked
   if fMulti + fAll + fAdd + fSingle < 1 then
      EmptyLinesOptions = DEFAULT_EMPTYLINES_OPTIONS
      parse value EmptyLinesOptions with . fMulti fAll fAdd fSingle .
   endif

   if fSingle then
      fMark = 0
      fMarkEnabled = 0
   elseif not FileIsMarked() then
      fMark = 0
      fMarkEnabled = 0
   else
      fMarkEnabled = 1
   endif
   SetAVar( 'emptylinesoptions', fMark fMulti fAll fAdd fSingle)

   fMatch = StrMatch( GetAVar( 'emptylinesoptions'), '? 0 0 1 0')
   SetMenuCondText( 'removeemptylines', not fMatch, fMatch)

   SetMenuAttribute( 'emptylinesmark',   MIA_DISABLED, fMarkEnabled)

   SetMenuAttribute( 'emptylinesmark',   MIA_CHECKED, not fMark)
   SetMenuAttribute( 'emptylinesmulti',  MIA_CHECKED, not fMulti)
   SetMenuAttribute( 'emptylinesall',    MIA_CHECKED, not fAll)
   SetMenuAttribute( 'emptylinesreadd',  MIA_CHECKED, not fAdd)
   SetMenuAttribute( 'emptylinessingle', MIA_CHECKED, not fSingle)

defc ToggleEmptyLinesOption
   EmptyLinesOptions = GetAVar( 'emptylinesoptions')
   -- Init
   if words( EmptyLinesOptions) <> 5 then
      EmptyLinesOptions = DEFAULT_EMPTYLINES_OPTIONS
   endif

   parse value EmptyLinesOptions with fMark fMulti fAll fAdd fSingle .
   ThisOption = lowcase( strip( arg( 1)))

   if ThisOption = 'mark' then
      -- Toggle
      fMark = not fMark
   else
      -- Radio
      fMulti  = (ThisOption = 'multi')
      fAll    = (ThisOption = 'all')
      fAdd    = (ThisOption = 'add')
      fSingle = (ThisOption = 'single')
   endif

   SetAVar( 'emptylinesoptions', fMark fMulti fAll fAdd fSingle)
   'menuinit_emptylines'

defc RemoveEmptyLinesMenuCmd
   EmptyLinesOptions = GetAVar( 'emptylinesoptions')
   -- Init
   if words( EmptyLinesOptions) <> 5 then
      EmptyLinesOptions = DEFAULT_EMPTYLINES_OPTIONS
   endif
   parse value EmptyLinesOptions with fMark fMulti fAll fAdd fSingle .

   if fMark & FileIsMarked() then
      fProcessMark = 1
   else
      fProcessMark = 0
      pSave_Mark( SavedMark)  -- unmarks
   endif

   if fMulti then
      'RemoveMultipleEmptyLines'
   elseif fAll then
      'RemoveAllEmptyLines'
   elseif fAdd then
      'ReaddEmptyLines'
   elseif fSingle then
      'SingleSpace'
   endif

   if not fProcessMark then
      pRestore_Mark( SavedMark)
   endif

const
  DEFAULT_STRIPBLANKS_OPTIONS = '0 0'

defc menuinit_stripblanks
   universal default_save_options

   StripBlanksOptions = GetAVar( 'stripblanksoptions')
   -- Init
   if words( StripBlanksOptions) <> 2 then
      StripBlanksOptions = DEFAULT_STRIPBLANKS_OPTIONS
   endif
   parse value StripBlanksOptions with fMark fLeading .

   SetAVar( 'stripblanksoptions', fMark fLeading)

   if not FileIsMarked() then
      fMark = 0
      fMarkEnabled = 0
   else
      fMarkEnabled = 1
   endif
   SetMenuAttribute( 'stripblanksmark',    MIA_DISABLED, fMarkEnabled)

   SetMenuAttribute( 'stripblanksmark',    MIA_CHECKED, not fMark)
   SetMenuAttribute( 'stripblanksleading', MIA_CHECKED, not fLeading)

   opt = '/a /ns /ne /nt 'default_save_options  -- internal default + default_save_options
   if lastpos( '/s ', opt' ') > lastpos( '/ns ', opt' ') then
      type = 'will'
   else
      type = 'will not'
   endif
   SetMenuVarText( 'format_saveoptions_striptrailingspaces', type)

defc ToggleStripBlanksOption
   StripBlanksOptions = GetAVar( 'stripblanksoptions')
   -- Init
   if words( StripBlanksOptions) <> 2 then
      StripBlanksOptions = DEFAULT_STRIPBLANKS_OPTIONS
   endif

   parse value StripBlanksOptions with fMark fLeading .
   ThisOption = lowcase( strip( arg( 1)))

   if ThisOption = 'mark' then
      -- Toggle
      fMark = not fMark
   elseif ThisOption = 'leading' then
      -- Toggle
      fLeading = not fLeading
   endif

   SetAVar( 'stripblanksoptions', fMark fLeading)
   'menuinit_stripblanks'

defc StripBlanksFromLinesMenuCmd
   StripBlanksOptions = GetAVar( 'stripblanksoptions')
   -- Init
   if words( StripBlanksOptions) <> 2 then
      StripBlanksOptions = DEFAULT_STRIPBLANKS_OPTIONS
   endif
   parse value StripBlanksOptions with fMark fLeading .

   if fMark & FileIsMarked() then
      fProcessMark = 1
   else
      fProcessMark = 0
      pSave_Mark( SavedMark)  -- unmarks
   endif
   Opt = ''
   if fLeading then
      Opt = 'L'
   endif

   strip( 'StripBlanksFromLines' Opt)

   if not fProcessMark then
      pRestore_Mark( SavedMark)
   endif

defc menuinit_tabsspaces
   universal default_save_options

   KeyPath = '\NEPMD\User\TabsSpaces\Compress\IndentOnly'
   fIndent = QueryConfigKey( KeyPath)
   TabsSpacesTabWidth = GetFileAVar( 'tabsspacestabwidth')
   TabsSpacesOptions  = GetAVar( 'tabsspacesoptions')
   -- Init
   if TabsSpacesTabWidth = '' then
      TabsSpacesTabWidth = word( .tabs, 1)
   endif
   parse value TabsSpacesOptions with fMark .
   if fMark = '' then
      fMark = 0
   endif

   SetFileAVar( 'tabsspacestabwidth', TabsSpacesTabWidth)
   SetAVar( 'tabsspacesoptions', fMark fIndent)

   SetMenuVarText( 'tabsspacestabwidth', TabsSpacesTabWidth)

   if not FileIsMarked() then
      fMark = 0
      fMarkEnabled = 0
   else
      fMarkEnabled = 1
   endif
   SetMenuAttribute( 'tabsspacesmark',   MIA_DISABLED, fMarkEnabled)

   SetMenuAttribute( 'tabsspacesmark',   MIA_CHECKED, not fMark)
   SetMenuAttribute( 'tabsspacesindent', MIA_CHECKED, not fIndent)

   opt = '/a /ns /ne /nt 'default_save_options  -- internal default + default_save_options
   if lastpos( '/nt ', opt' ') > lastpos( '/t ', opt' ') then
      type = 'will not'
   else
      type = 'will'
   endif
   SetMenuVarText( 'format_saveoptions_spaces2tabs', type)

defc SelectTabsSpacesTabWidth
   TabsSpacesTabWidth = GetFileAVar( 'tabsspacestabwidth')
   if TabsSpacesTabWidth = '' then
      TabsSpacesTabWidth = word( .tabs, 1)
   endif

   Title = 'Select tab width for conversion'
   Text  = 'Enter a number for expand or compress:'
   Text  = Text''copies( ' ', Max( 100 - length( Text), 0))
   Buttons = '/~Set/~Reset/'CANCEL__MSG
   Entry = TabsSpacesTabWidth
   parse value EntryBox( Title,
                         Buttons,
                         Entry,
                         0,
                         240,
                         atoi( 1) || atoi( 0) || atol( 0) ||
                         Text) with button 2 NewTabWidth \0
   NewTabWidth = strip( NewTabWidth)
   if button = \1 then      -- Set
      TabsSpacesTabWidth = NewTabWidth
      SetFileAVar( 'tabsspacestabwidth', TabsSpacesTabWidth)
   elseif button = \2 then  -- Reset
      TabsSpacesTabWidth = word( .tabs, 1)
      SetFileAVar( 'tabsspacestabwidth', '')
   else                     -- Cancel
      return
   endif

   SetMenuVarText( 'tabsspacestabwidth', TabsSpacesTabWidth)

defc ToggleTabsSpacesOption
   KeyPath = '\NEPMD\User\TabsSpaces\Compress\IndentOnly'
   fIndent = QueryConfigKey( KeyPath)
   TabsSpacesOptions = GetAVar( 'tabsspacesoptions')
   -- Init
   if words( TabsSpacesOptions) <> 2 then
      TabsSpacesOptions = 0 fIndent
   endif

   parse value TabsSpacesOptions with fMark fIndent .
   ThisOption = lowcase( strip( arg( 1)))

   if ThisOption = 'mark' then
      -- Toggle
      fMark = not fMark
   elseif ThisOption = 'indent' then
      -- Toggle
      fIndent = not fIndent
      WriteConfigKey( KeyPath, fIndent)
   endif

   SetAVar( 'tabsspacesoptions', fMark fIndent)
   'menuinit_tabsspaces'

defc TabExpandMenuCmd
   TabsSpacesTabWidth = GetFileAVar( 'tabsspacestabwidth')
   TabsSpacesOptions  = GetAVar( 'tabsspacesoptions')
   -- Init
   if TabsSpacesTabWidth = '' then
      TabsSpacesTabWidth = word( .tabs, 1)
   endif
   parse value TabsSpacesOptions with fMark .
   if fMark = '' then
      fMark = 0
   endif

   if fMark & FileIsMarked() then
      fProcessMark = 1
   else
      fProcessMark = 0
      pSave_Mark( SavedMark)  -- unmarks
   endif

   'TabExpand 'TabsSpacesTabWidth

   if not fProcessMark then
      pRestore_Mark( SavedMark)
   endif

defc TabCompressMenuCmd
   TabsSpacesTabWidth = GetFileAVar( 'tabsspacestabwidth')
   TabsSpacesOptions  = GetAVar( 'tabsspacesoptions')
   -- Init
   if TabsSpacesTabWidth = '' then
      TabsSpacesTabWidth = word( .tabs, 1)
   endif
   parse value TabsSpacesOptions with fMark .
   if fMark = '' then
      fMark = 0
   endif

   if fMark & FileIsMarked() then
      fProcessMark = 1
   else
      fProcessMark = 0
      pSave_Mark( SavedMark)  -- unmarks
   endif

   'TabCompress 'TabsSpacesTabWidth

   if not fProcessMark then
      pRestore_Mark( SavedMark)
   endif

defc menuinit_indent
   IndentOptions = GetAVar( 'indentoptions')
   -- Init
   parse value IndentOptions with fMark .
   if fMark = '' then
      fMark = 0
   endif

   SetAVar( 'indentoptions', fMark)

   if not FileIsMarked() then
      fMark = 0
      fMarkEnabled = 0
   else
      fMarkEnabled = 1
   endif
   SetMenuAttribute( 'indentmark',    MIA_DISABLED, fMarkEnabled)

   SetMenuAttribute( 'indentmark',    MIA_CHECKED, not fMark)
   SetMenuAttribute( 'indentlineset', MIA_CHECKED, fMark)

   on = FileIsMarked()
   SetMenuAttribute( 'shiftleft',  MIA_DISABLED, on)
   SetMenuAttribute( 'shiftright', MIA_DISABLED, on)
   SetMenuAttribute( 'centermark', MIA_DISABLED, on)

defc ToggleIndentOption
   IndentOptions = GetAVar( 'indentoptions')
   -- Init
   if words( IndentOptions) <> 1 then
      IndentOptions = 0
   endif

   parse value IndentOptions with fMark .
   ThisOption = lowcase( strip( arg( 1)))

   if ThisOption = 'mark' then
      -- Radio
      fMark = 1
   elseif ThisOption = 'lineset' then
      -- Radio
      fMark = 0
   endif

   SetAVar( 'indentoptions', fMark)
   'menuinit_indent'

defc IndentBlockMenuCmd
   IndentOptions = GetAVar( 'indentoptions')
   -- Init
   parse value IndentOptions with fMark .
   if fMark = '' then
      fMark = 0
   endif

   if fMark & FileIsMarked() then
      fProcessMark = 1
   else
      fProcessMark = 0
      pSave_Mark( SavedMark)  -- unmarks
   endif

   if arg( 1) then
      'IndentBlock 'arg( 1)
   else
      'IndentBlock'
   endif

   if not fProcessMark then
      pRestore_Mark( SavedMark)
   endif

defc menuinit_comment
   CommentOptions = GetAVar( 'commentoptions')
   -- Init
   parse value CommentOptions with fMark .
   if fMark = '' then
      fMark = 0
   endif

   SetAVar( 'commentoptions', fMark)

   if not FileIsMarked() then
      fMark = 0
      fMarkEnabled = 0
   else
      fMarkEnabled = 1
   endif
   SetMenuAttribute( 'commentmark',    MIA_DISABLED, fMarkEnabled)

   SetMenuAttribute( 'commentmark',    MIA_CHECKED, not fMark)
   SetMenuAttribute( 'commentlineset', MIA_CHECKED, fMark)

defc ToggleCommentOption
   CommentOptions = GetAVar( 'commentoptions')
   -- Init
   if words( CommentOptions) <> 1 then
      CommentOptions = 0
   endif

   parse value CommentOptions with fMark .
   ThisOption = lowcase( strip( arg( 1)))

   if ThisOption = 'mark' then
      -- Radio
      fMark = 1
   elseif ThisOption = 'lineset' then
      -- Radio
      fMark = 0
   endif

   SetAVar( 'commentoptions', fMark)
   'menuinit_comment'

defc CommentMenuCmd
   CommentOptions = GetAVar( 'commentoptions')
   -- Init
   parse value CommentOptions with fMark .
   if fMark = '' then
      fMark = 0
   endif

   if fMark & FileIsMarked() then
      fProcessMark = 1
   else
      fProcessMark = 0
      pSave_Mark( SavedMark)  -- unmarks
   endif

  'Comment'

   if not fProcessMark then
      pRestore_Mark( SavedMark)
   endif

defc UncommentMenuCmd
   CommentOptions = GetAVar( 'commentoptions')
   -- Init
   parse value CommentOptions with fMark .
   if fMark = '' then
      fMark = 0
   endif

   if fMark & FileIsMarked() then
      fProcessMark = 1
   else
      fProcessMark = 0
      pSave_Mark( SavedMark)  -- unmarks
   endif

  'Uncomment'

   if not fProcessMark then
      pRestore_Mark( SavedMark)
   endif

defc menuinit_doublelines
   DoubleLinesOptions = GetAVar( 'doublelinesoptions')
   -- Init
   if words( DoubleLinesOptions) <> 1 then
      DoubleLinesOptions = 0
   endif
   parse value DoubleLinesOptions with fMark .

   SetAVar( 'doublelinesoptions', fMark)

   if not FileIsMarked() then
      fMark = 0
      fMarkEnabled = 0
   else
      fMarkEnabled = 1
   endif
   SetMenuAttribute( 'doublelinesmark',   MIA_DISABLED, fMarkEnabled)

   SetMenuAttribute( 'doublelinesmark',   MIA_CHECKED, not fMark)

defc ToggleDoubleLinesOption
   DoubleLinesOptions = GetAVar( 'doublelinesoptions')
   -- Init
   if words( DoubleLinesOptions) <> 1 then
      DoubleLinesOptions = 0
   endif

   parse value DoubleLinesOptions with fMark .
   ThisOption = lowcase( strip( arg( 1)))

   if ThisOption = 'mark' then
      -- Toggle
      fMark = not fMark
   endif

   SetAVar( 'doublelinesoptions', fMark)
   'menuinit_doublelines'

defc RemoveDoubleLinesMenuCmd
   DoubleLinesOptions = GetAVar( 'doublelinesoptions')
   -- Init
   if words( DoubleLinesOptions) <> 1 then
      DoubleLinesOptions = 0
   endif
   parse value DoubleLinesOptions with fMark .

   if fMark & FileIsMarked() then
      fProcessMark = 1
   else
      fProcessMark = 0
      pSave_Mark( SavedMark)  -- unmarks
   endif

   'RemoveDoubleLines'

   if not fProcessMark then
      pRestore_Mark( SavedMark)
   endif

defc menuinit_sort
   KeyPath = '\NEPMD\User\Sort'
   fDescending     = QueryConfigKey( KeyPath'\Descending')
   fIgnoreCase     = QueryConfigKey( KeyPath'\IgnoreCase')
   fCollatingOrder = QueryConfigKey( KeyPath'\CollatingOrder')
   SortOptions = GetAVar( 'sortoptions')
   -- Init
   if words( SortOptions) <> 4 then
      SortOptions = 0 fDescending fIgnoreCase fCollatingOrder
   endif
   parse value SortOptions with fMark .

   if not FileIsMarked() then
      fMark = 0
      fMarkEnabled = 0
   else
      fMarkEnabled = 1
   endif
   SetMenuAttribute( 'sortmark',           MIA_DISABLED, fMarkEnabled)

   SetMenuAttribute( 'sortmark',           MIA_CHECKED, not fMark)
   SetMenuAttribute( 'sortdescending',     MIA_CHECKED, not fDescending)
   SetMenuAttribute( 'sortignorecase',     MIA_CHECKED, not fIgnoreCase)
   SetMenuAttribute( 'sortcollatingorder', MIA_CHECKED, not fCollatingOrder)

defc ToggleSortOption
   KeyPath = '\NEPMD\User\Sort'
   fDescending     = QueryConfigKey( KeyPath'\Descending')
   fIgnoreCase     = QueryConfigKey( KeyPath'\IgnoreCase')
   fCollatingOrder = QueryConfigKey( KeyPath'\CollatingOrder')
   SortOptions = GetAVar( 'sortoptions')
   -- Init
   if words( SortOptions) <> 4 then
      SortOptions = 0 fDescending fIgnoreCase fCollatingOrder
   endif
   parse value SortOptions with fMark .
   ThisOption = lowcase( strip( arg( 1)))

   if ThisOption = 'mark' then
      -- Toggle
      fMark = not fMark
   elseif ThisOption = 'descending' then
      -- Toggle
      fDescending = not fDescending
      WriteConfigKey( KeyPath'\Descending', fDescending)
   elseif ThisOption = 'ignorecase' then
      -- Toggle
      fIgnoreCase = not fIgnoreCase
      WriteConfigKey( KeyPath'\IgnoreCase', fIgnoreCase)
   elseif ThisOption = 'collatingorder' then
      -- Toggle
      fCollatingOrder = not fCollatingOrder
      WriteConfigKey( KeyPath'\CollatingOrder', fCollatingOrder)
   endif

   SetAVar( 'sortoptions', fMark fDescending fIgnoreCase fCollatingOrder)
   'menuinit_sort'

defc SortMenuCmd
   SortOptions = GetAVar( 'sortoptions')
   -- Init
   parse value SortOptions with fMark .
   if fMark = '' then
      fMark = 0
   endif

   if fMark & FileIsMarked() then
      fProcessMark = 1
   else
      fProcessMark = 0
      pSave_Mark( SavedMark)  -- unmarks
   endif

   SortOptions = ''
   KeyPath = '\NEPMD\User\Sort\Descending'
   on = (QueryConfigKey( KeyPath) = 1)
   if on then
      SortOptions = SortOptions'D'
   endif
   KeyPath = '\NEPMD\User\Sort\IgnoreCase'
   on = (QueryConfigKey( KeyPath) = 1)
   if on then
      SortOptions = SortOptions'I'
   endif
   KeyPath = '\NEPMD\User\Sort\CollatingOrder'
   on = (QueryConfigKey( KeyPath) = 1)
   if on then
      SortOptions = SortOptions'C'
   endif
   'Sort 'SortOptions

   if not fProcessMark then
      pRestore_Mark( SavedMark)
   endif

defc menuinit_case
   on = FileIsMarked()
   SetMenuAttribute( 'marklowercase', MIA_DISABLED, on)
   SetMenuAttribute( 'markuppercase', MIA_DISABLED, on)

defc menuinit_encoding
   universal codepage

   KeyPath = '\NEPMD\User\Encoding'

   call CheckTool( 'uconv')
   call CheckTool( 'recode')
   fUconvFound  = (GetAVar( 'tool.uconv') <> '')
   fRecodeFound = (GetAVar( 'tool.recode') <> '')

   on = fUconvFound
   SetMenuAttribute( 'useuconv', MIA_DISABLED, on)
   on = fRecodeFound
   SetMenuAttribute( 'userecode', MIA_DISABLED, on)
   on = (fUconvFound | fRecodeFound)
   SetMenuAttribute( 'switchencoding', MIA_DISABLED, on)

   on = (lowcase( GetEncodingTool()) <> 'recode')
   SetMenuAttribute( 'useuconv', MIA_CHECKED, not on)
   on = not on
   SetMenuAttribute( 'userecode', MIA_CHECKED, not on)

   SetMenuVarText( 'switchencoding', GetEncodingName())
   SetMenuVarText( 'switchstandardencoding', GetStandardEncodingName())
   None = '-none-'
   new = GetEncodingCode()
   if new = '' then
      new = None
   endif
   old = GetStandardEncodingCode()
   if old = '' then
      old = None
   endif

   SetMenuVarText( 'encodefrom', old, new)
   SetMenuVarText( 'encodeto',   new, old)

   on = (new <> None) & (old <> None) & (fUconvFound | fRecodeFound)
   SetMenuAttribute( 'encodefrom', MIA_DISABLED, on)
   SetMenuAttribute( 'encodeto',   MIA_DISABLED, on)

   on = QueryConfigKey( KeyPath'\EditCmdBefore')
   SetMenuAttribute( 'editencodingcmd', MIA_CHECKED, not on)

const
  DEFAULT_SPELLCHECK_OPTIONS = '0 1'

defc menuinit_spellcheck
   SpellCheckOptions = GetAVar( 'spellcheckoptions')
   -- Init
   if words( SpellCheckOptions) <> 2 then
      SpellCheckOptions = DEFAULT_SPELLCHECK_OPTIONS
   endif
   parse value SpellCheckOptions with fMark fCursor .

   SetAVar( 'spellcheckoptions', fMark fCursor)

   if not FileIsMarked() then
      fMark = 0
      fMarkEnabled = 0
   else
      fMarkEnabled = 1
   endif
   if fMark then
      fCursor = 0
      fCursorEnabled = 0
   else
      fCursorEnabled = 1
   endif
   SetMenuAttribute( 'spellcheckmark',   MIA_DISABLED, fMarkEnabled)
   SetMenuAttribute( 'spellcheckcursor', MIA_DISABLED, fCursorEnabled)

   SetMenuAttribute( 'spellcheckmark',   MIA_CHECKED, not fMark)
   SetMenuAttribute( 'spellcheckcursor', MIA_CHECKED, not fCursor)

   None = '-none-'
   new = GetDictBaseName()
   if new = '' then
      new = None
   endif
   SetMenuVarText( 'switchdict', new)

defc ToggleSpellCheckOption
   SpellCheckOptions = GetAVar( 'spellcheckoptions')
   -- Init
   if words( SpellCheckOptions) <> 2 then
      SpellCheckOptions = DEFAULT_SPELLCHECK_OPTIONS
   endif

   parse value SpellCheckOptions with fMark fCursor.
   ThisOption = lowcase( strip( arg( 1)))

   if ThisOption = 'mark' then
      -- Toggle
      fMark = not fMark
   elseif ThisOption = 'cursor' then
      -- Toggle
      fCursor = not fCursor
   endif

   SetAVar( 'spellcheckoptions', fMark fCursor)
   'menuinit_spellcheck'

defc ProofMenuCmd
   SpellCheckOptions = GetAVar( 'spellcheckoptions')
   -- Init
   if words( SpellCheckOptions) <> 2 then
      SpellCheckOptions = DEFAULT_SPELLCHECK_OPTIONS
   endif
   parse value SpellCheckOptions with fMark fCursor .

   if fMark & FileIsMarked() then
      fProcessMark = 1
   else
      fProcessMark = 0
      pSave_Mark( SavedMark)  -- unmarks
   endif
   if not fCursor then
      if fProcessMark then
         --'BeginMark'  -- Not required, already done by proof
      else
         .line = 1
         .col  = 1
      endif
   endif

   'Proof'

   if not fProcessMark then
      pRestore_Mark( SavedMark)
   endif

; ---------------------------------------------------------------------------
defc menuinit_linewraps
   KeyPath     = '\NEPMD\User\Format\Wrap'
   fSoftWrap   = QueryConfigKey( KeyPath'\SoftWrap')
   fToWindow   = QueryConfigKey( KeyPath'\ToWindow')
   fMark       = QueryConfigKey( KeyPath'\Mark')
   fPar        = QueryConfigKey( KeyPath'\Par')
   fKeepIndent = QueryConfigKey( KeyPath'\KeepIndent')

   KeyPath     = '\NEPMD\User\Format\HeaderWidth'
   SplitCol    = QueryConfigKey( KeyPath)

   if fMark & fPar then
      fPar = 0
   endif

   SetMenuAttribute( 'wraplines',           MIA_CHECKED,  not (fSoftWrap & GetWrapped()))
   SetMenuCondText( 'wraplines', not fSoftWrap, fSoftWrap & not GetWrapped(), fSoftWrap & GetWrapped())

   SetMenuAttribute( 'softwrap2',           MIA_CHECKED,  not fSoftWrap)
   SetMenuAttribute( 'softwrap2',           MIA_DISABLED, not GetWrapped())

   SetMenuAttribute( 'softwraptowindow2',   MIA_CHECKED,  not (fToWindow & fSoftWrap))
   SetMenuAttribute( 'softwraptowindow2',   MIA_DISABLED, not (GetWrapped() | not fSoftWrap))

   SetMenuVarText( 'headerwidth2', SplitCol)
   SetMenuAttribute( 'headerwidth2',        MIA_DISABLED, not (fSoftWrap & (fToWindow | GetWrapped())))

   SetMenuAttribute( 'linewrapsmark',       MIA_CHECKED,  not (fMark & not fSoftWrap & FileIsMarked()))
   SetMenuAttribute( 'linewrapsmark',       MIA_DISABLED, not (fSoftWrap | not FileIsMarked()))

   SetMenuAttribute( 'linewrapspar',        MIA_CHECKED,  not (fPar & not fSoftWrap))
   SetMenuAttribute( 'linewrapspar',        MIA_DISABLED, not fSoftWrap)

   SetMenuAttribute( 'linewrapskeepindent', MIA_CHECKED,  not (fKeepIndent & not fSoftWrap))
   SetMenuAttribute( 'linewrapskeepindent', MIA_DISABLED, not fSoftWrap)

; ---------------------------------------------------------------------------
defc ToggleLineWrapOption
   Option = lowcase( strip( arg( 1)))
   KeyPath = '\NEPMD\User\Format\Wrap'
   if Option = 'softwrap' then
      on = QueryConfigKey( KeyPath'\SoftWrap')
      on = not on
      WriteConfigKey( KeyPath'\SoftWrap', on)
   elseif Option = 'towindow' then
      on = QueryConfigKey( KeyPath'\ToWindow')
      on = not on
      WriteConfigKey( KeyPath'\ToWindow', on)
   elseif Option = 'mark' then
      on = QueryConfigKey( KeyPath'\Mark')
      on = not on
      WriteConfigKey( KeyPath'\Mark', on)
      if on then
         WriteConfigKey( KeyPath'\Par', not on)
      endif
   elseif Option = 'par' then
      on = QueryConfigKey( KeyPath'\Par')
      on = not on
      WriteConfigKey( KeyPath'\Par', on)
      if on then
         WriteConfigKey( KeyPath'\Mark', not on)
      endif
   elseif Option = 'keepindent' then
      on = QueryConfigKey( KeyPath'\KeepIndent')
      on = not on
      WriteConfigKey( KeyPath'\KeepIndent', on)
   endif
   'menuinit_linewraps'

; ---------------------------------------------------------------------------
; This executes Reflow<selected mode> with keywords.
defc ReflowMenuCmd
   KeyPath = '\NEPMD\User\Format\Reflow'
   fMark = QueryConfigKey( KeyPath'\Mark')
   fPar  = QueryConfigKey( KeyPath'\Par')
   SelectedMode = ReflowGetSelectedMode()
   ReflowMargins = ReflowGetReflowMargins()

   if SelectedMode = 'TEXT' then
      -- TEXT options
      -- twospaces is handled by the universal twospaces internally
      fBlock    = QueryConfigKey( KeyPath'\Block')
      fWordProc = QueryConfigKey( KeyPath'\WordProc')
      fMail     = QueryConfigKey( KeyPath'\Mail')
      if fBlock & marktype() = 'BLOCK' & FileIsMarked() then
         fNormal   = 0
         fWordProc = 0
         fMail     = 0
      elseif fWordProc then
         fNormal   = 0
         fBlock    = 0
         fMail     = 0
      elseif fMail then
         fNormal   = 0
         fBlock    = 0
         fWordProc = 0
      else
         fNormal   = 1
         fBlock    = 0
         fWordProc = 0
         fMail     = 0
      endif

      if fBlock then
         'ReflowBlock'
      elseif fWordProc then
         'WordProc'
      elseif fMail then
         -- Options for indented lines and for lists are queried by ReflowMail
         'ReflowMail'
      elseif fNormal then
         if fMark then
            'ReflowMark 'ReflowMargins
         elseif fPar then
            'ReflowPar 'ReflowMargins
         else
            'ReflowAll 'ReflowMargins
         endif
      endif
/*
   -- Add special defs for modes here
   elseif SelectedMode = '' then
      -- ...
   elseif SelectedMode = '' then
      -- ...
*/
   else
      'Reflow'SelectedMode
   endif

; ---------------------------------------------------------------------------
defc menuinit_reflow
   KeyPath = '\NEPMD\User\Format\Reflow'

   SelectedMode = ReflowGetSelectedMode()
   SetMenuVarText( 'reflowmode',        SelectedMode)

   Select = QueryConfigKey( KeyPath'\Mode')
   if not Select then
      Select = '-auto-'
   endif
   SetMenuVarText( 'reflowselectmode',  Select)

   -- Common options
   fMark = QueryConfigKey( KeyPath'\Mark')
   fPar  = QueryConfigKey( KeyPath'\Par')
   fCommonOptionsEnabled = 1

   if SelectedMode = 'TEXT' then
      -- TEXT options
      fBlock    = QueryConfigKey( KeyPath'\Block')
      fWordProc = QueryConfigKey( KeyPath'\WordProc')
      fMail     = QueryConfigKey( KeyPath'\Mail')
      if fBlock & marktype() = 'BLOCK' & FileIsMarked() then
         fNormal   = 0
         fWordProc = 0
         fMail     = 0
      elseif fWordProc then
         fNormal   = 0
         fBlock    = 0
         fMail     = 0
      elseif fMail then
         fNormal   = 0
         fBlock    = 0
         fWordProc = 0
      else
         fNormal   = 1
         fBlock    = 0
         fWordProc = 0
         fMail     = 0
      endif

      SetMenuAttribute( 'reflownormal',   MIA_CHECKED,  not fNormal)
      SetMenuAttribute( 'reflowblock',    MIA_CHECKED,  not fBlock)
      SetMenuAttribute( 'reflowblock',    MIA_DISABLED, marktype() = 'BLOCK' & FileIsMarked())
      SetMenuAttribute( 'reflowwordproc', MIA_CHECKED,  not fWordProc)
      SetMenuAttribute( 'reflowmail',     MIA_CHECKED,  not fMail)

      fTwoSpaces     = QueryConfigKey( KeyPath'\TwoSpaces')
      fKeepIndented  = QueryConfigKey( KeyPath'\KeepIndented')
      fWrapIndented  = QueryConfigKey( KeyPath'\WrapIndented')
      fReflowLists   = QueryConfigKey( KeyPath'\ReflowLists')
      fReindentLists = QueryConfigKey( KeyPath'\ReindentLists')
      fFixReplyLevel = QueryConfigKey( KeyPath'\MailFixFalseReplyLevels')

      SetMenuAttribute( 'reflowtwospaces',     MIA_CHECKED, not fTwoSpaces)
      SetMenuVarText( 'reflowmargins', ReflowGetReflowMargins())
      SetMenuAttribute( 'reflowkeepindented',  MIA_CHECKED, not (fKeepIndented & fMail))
      --SetMenuAttribute( 'reflowwrapindented',  MIA_CHECKED, not (fWrapIndented & fMail))
      SetMenuAttribute( 'reflowreflowlists',   MIA_CHECKED, not (fReflowLists & fMail))
      --SetMenuAttribute( 'reflowreindentlists', MIA_CHECKED, not (fReindentLists & fMail))
      SetMenuAttribute( 'reflowmailfixreplylevels', MIA_CHECKED, not (fFixReplyLevel & fMail))

      --new = QueryConfigKey( KeyPath'\ListIndent')
      --SetMenuVarText( 'reflowlistindent', new)

      SetMenuAttribute( 'reflowmargins',       MIA_DISABLED, not fMail)
      SetMenuAttribute( 'reflowkeepindented',  MIA_DISABLED, fMail)
      --SetMenuAttribute( 'reflowwrapindented',  MIA_DISABLED, fKeepIndented & fMail)
      SetMenuAttribute( 'reflowreflowlists',   MIA_DISABLED, fMail)
      --SetMenuAttribute( 'reflowreindentlists', MIA_DISABLED, fMail)
      --SetMenuAttribute( 'reflowlistindent',    MIA_DISABLED, fReindentLists & fMail)
      SetMenuAttribute( 'reflowmailfixreplylevels', MIA_DISABLED, fMail)

      SetMenuVarText( 'reflowmodeoptions', SelectedMode)
      SetMenuVarText( 'reflowtypes',       SelectedMode)

      fCommonOptionsEnabled = fNormal
/*
   elseif SelectedMode = 'CONFIGSYS' then
      SetMenuVarText( 'reflowmodeoptions', SelectedMode)
      fCommonOptionsEnabled = 0
*/
   endif

   SetMenuAttribute( 'reflowmark', MIA_DISABLED, fCommonOptionsEnabled & FileIsMarked())
   SetMenuAttribute( 'reflowpar',  MIA_DISABLED, fCommonOptionsEnabled)
   SetMenuAttribute( 'reflowmark', MIA_CHECKED,  not (fMark & fCommonOptionsEnabled & FileIsMarked()))
   SetMenuAttribute( 'reflowpar',  MIA_CHECKED,  not (fPar  & fCommonOptionsEnabled))

; ---------------------------------------------------------------------------
defc menuinit_reflowmargins

   KeyPath = '\NEPMD\User\Format\Reflow'

   new = '1 'GetDefaultHeaderWidth()' 1'
   SetMenuVarText( 'reflowmargins1', new)

   new = QueryConfigKey( KeyPath'\Margins\2')
   SetMenuVarText( 'reflowmargins2', new)

   new = .margins
   SetMenuVarText( 'reflowmargins3', new)

   i = QueryConfigKey( KeyPath'\Margins\Selected')
   SetMenuAttribute( 'reflowmargins1', MIA_CHECKED, not (i = 1))
   SetMenuAttribute( 'reflowmargins2', MIA_CHECKED, not (i = 2))
   SetMenuAttribute( 'reflowmargins3', MIA_CHECKED, not (i = 3))

   if i = 1 then
      text = '~Configure header width...'
   elseif i = 2 then
      text = '~Configure selected...'
   elseif i = 3 then
      text = '~Configure margins...'
   endif
   SetMenuText( 'mid_reflowmarginsconfig', text)

; ---------------------------------------------------------------------------
defc ToggleReflowMargins
   KeyPath = '\NEPMD\User\Format\Reflow'
   i = arg( 1)
   WriteConfigKey( KeyPath'\Margins\Selected', i)
   'menuinit_reflowmargins'
   'menuinit_reflow'

; ---------------------------------------------------------------------------
defc ReflowMarginsMenuCmd
   args = arg( 1)
   parse value args with lma rma parma
   if lma <> '' then
      if rma = '' then
         args = 1 args  -- default value for lma is 1
      endif
      if parma = '' then
         args = args 1  -- default value for parma is 1
      endif
   endif

   KeyPath = '\NEPMD\User\Format\Reflow'
   i   = QueryConfigKey( KeyPath'\Margins\Selected')
   if i = 1 then
      'DefaultHeaderWidth'
      return
   elseif i = 3 then
      'ma'  -- Open command line with current .margins value
      return
   else
      new = args
      if new = '' then
         old = QueryConfigKey( KeyPath'\Margins\'i)
         'commandline set_ReflowMargins' old  -- Open command line with current value
         return
      endif
      WriteConfigKey( KeyPath'\Margins\'i, new)
   endif
   ReflowMargins = ReflowGetReflowMargins()
   SetMenuVarText( 'reflowmargins'i, ReflowMargins)
   SetMenuVarText( 'reflowmargins', ReflowMargins)

; ---------------------------------------------------------------------------
defc ReflowmarginsSelect
   i = arg( 1)
   KeyPath = '\NEPMD\User\Format\Reflow\Margins\Selected'
   WriteConfigKey( KeyPath, i)

   SetMenuAttribute( 'reflowmargins1', MIA_CHECKED, not (i = 1))
   SetMenuAttribute( 'reflowmargins2', MIA_CHECKED, not (i = 2))
   SetMenuAttribute( 'reflowmargins3', MIA_CHECKED, not (i = 3))
   SetMenuVarText( 'reflowmargins', ReflowGetReflowMargins())
   if i = 1 then
      text = '~Configure header width...'
   elseif i = 2 then
      text = '~Configure selected...'
   elseif i = 3 then
      text = '~Configure margins...'
   endif
   SetMenuText( 'mid_reflowmarginsconfig', text)

; ---------------------------------------------------------------------------
defc ToggleReflowOption
   universal twospaces

   Option = lowcase( strip( arg( 1)))
   KeyPath = '\NEPMD\User\Format\Reflow'
   if Option = 'mark' then
      on = QueryConfigKey( KeyPath'\Mark')
      on = not on
      WriteConfigKey( KeyPath'\Mark', on)
   elseif Option = 'par' then
      on = QueryConfigKey( KeyPath'\Par')
      on = not on
      WriteConfigKey( KeyPath'\Par', on)
   elseif Option = 'twospaces' then
      on = QueryConfigKey( KeyPath'\TwoSpaces')
      on = not on
      WriteConfigKey( KeyPath'\TwoSpaces', on)
      twospaces = on
   elseif Option = 'normal' then
      WriteConfigKey( KeyPath'\Block',    0)
      WriteConfigKey( KeyPath'\WordProc', 0)
      WriteConfigKey( KeyPath'\Mail',     0)
   elseif Option = 'block' then
      WriteConfigKey( KeyPath'\Block',    1)
      WriteConfigKey( KeyPath'\WordProc', 0)
      WriteConfigKey( KeyPath'\Mail',     0)
   elseif Option = 'wordproc' then
      WriteConfigKey( KeyPath'\Block',    0)
      WriteConfigKey( KeyPath'\WordProc', 1)
      WriteConfigKey( KeyPath'\Mail',     0)
   elseif Option = 'mail' then
      WriteConfigKey( KeyPath'\Block',    0)
      WriteConfigKey( KeyPath'\WordProc', 0)
      WriteConfigKey( KeyPath'\Mail',     1)
   elseif Option = 'keepindented' then
      on = QueryConfigKey( KeyPath'\KeepIndented')
      on = not on
      WriteConfigKey( KeyPath'\KeepIndented', on)
   elseif Option = 'wrapindented' then
      on = QueryConfigKey( KeyPath'\WrapIndented')
      on = not on
      WriteConfigKey( KeyPath'\WrapIndented', on)
   elseif Option = 'reflowlists' then
      on = QueryConfigKey( KeyPath'\ReflowLists')
      on = not on
      WriteConfigKey( KeyPath'\ReflowLists', on)
   elseif Option = 'reindentlists' then
      on = QueryConfigKey( KeyPath'\ReindentLists')
      on = not on
      WriteConfigKey( KeyPath'\ReindentLists', on)
   elseif Option = 'listitems' then
      on = QueryConfigKey( KeyPath'\ListItems')
      on = not on
      WriteConfigKey( KeyPath'\ListItems', on)
   elseif Option = 'mailfixreplylevels' then
      on = QueryConfigKey( KeyPath'\MailFixFalseReplyLevels')
      on = not on
      WriteConfigKey( KeyPath'\MailFixFalseReplyLevels', on)
   endif
   'menuinit_reflow'

; ---------------------------------------------------------------------------
defc menuinit_codingstyle
   KeyPath = '\NEPMD\User\SyntaxExpansion'
   on = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'selectcodingstyle2',   MIA_DISABLED, on)
   None = '-none-'
   new = GetCodingStyle()
   if new = '' then
      new = None
   endif
   SetMenuVarText( 'selectcodingstyle2', new, GetMode())

defc menuinit_centerboxdraw
   mt = MarkTypeOfFile()
   on = (mt <> '')
   SetMenuAttribute( 'centermark', MIA_DISABLED, on)
   SetMenuAttribute( 'box',        MIA_DISABLED, on)
   None = '-none-'
   KeyPath = '\NEPMD\User\Format\BoxStyle'
   new = QueryConfigKey( KeyPath)
   if new = '' then
      new = None
   endif
   SetMenuVarText( 'selectboxstyle', new)
   new = GetDefaultHeaderWidth()
   SetMenuVarText( 'headerwidth', new)
   KeyPath = '\NEPMD\User\Format\DrawStyle'
   new = QueryConfigKey( KeyPath)
   if new = '' then
      new = None
   endif
   SetMenuVarText( 'selectdrawstyle', new)

; ------------------------------------ Search -------------------------------
defc menuinit_search
   LastChangeArgs = GetLastChangeArgs()
   delim = \1
   parse value LastChangeArgs with (delim)SearchString(delim)ReplaceString(delim)SearchOptions
   LastSearchArgs = GetLastSearchArgs()

   SetMenuAttribute( 'findnext',         MIA_DISABLED, LastSearchArgs <> '')

   SetMenuAttribute( 'changenext',       MIA_DISABLED, LastChangeArgs <> '')

   SetMenuAttribute( 'globalfindnext',   MIA_DISABLED, LastSearchArgs <> '')

   SetMenuAttribute( 'globalchangenext', MIA_DISABLED, LastChangeArgs <> '')

   if GetAddKeyDefs() = '' then  -- stdkeys
      -- Menu for CUA keys doesn't have a Backward item
      on = (GetSearchDirection() = '-')
      SetMenuAttribute( 'searchbackward', MIA_CHECKED, not on)
   else  -- cuakeys
      -- Menu for std keys doesn't have the following items
      SetMenuAttribute( 'findprev',       MIA_DISABLED, LastSearchArgs <> '')
      SetMenuAttribute( 'changeprev',     MIA_DISABLED, LastChangeArgs <> '')
      SetMenuAttribute( 'globalfindprev', MIA_DISABLED, LastSearchArgs <> '')
   endif

   on = FileIsMarked()
   SetMenuAttribute( 'findmark', MIA_DISABLED, on)

defc menuinit_goto
   on = (marktype() <> '')  -- on when any file is marked
   SetMenuAttribute( 'gotobeginmark', MIA_DISABLED, on)
   SetMenuAttribute( 'gotoendmark',   MIA_DISABLED, on)
   SetMenuAttribute( 'gotoaftermark', MIA_DISABLED, on)

defc menuinit_markstack
   universal mark_stack

   on = FileIsMarked()
   SetMenuAttribute( 'savemark',    MIA_DISABLED, on)

   SetMenuAttribute( 'restoremark', MIA_DISABLED, mark_stack <> '')

   SetMenuAttribute( 'swapmark',    MIA_DISABLED, on & mark_stack <> '')

defc menuinit_cursorstack
   universal position_stack

   SetMenuAttribute( 'restorecursor', MIA_DISABLED, position_stack <> '')
   SetMenuAttribute( 'swapcursor',    MIA_DISABLED, position_stack <> '')

defc menuinit_bookmarks
   getfileid bmfid
   bmcount = GetAVar( 'bmname.'bmfid'.0')
   SetMenuAttribute( 'bookmarks_set',      MIA_DISABLED, not (browse() | .readonly))  -- Set
   SetMenuAttribute( 'bookmarks_next',     MIA_DISABLED, bmcount > 0)   -- Next
   SetMenuAttribute( 'bookmarks_previous', MIA_DISABLED, bmcount > 0)   -- Previous

defc menuinit_finddef
   universal finddefsearchtype
   universal finddefexact
   universal finddefprototypes
   universal finddefcomlit
   universal finddefallmatches
   universal finddefrecursion

   CurMode = GetMode()
   SetMenuVarText( 'finddefmode', CurMode)
/*
   SetMenuVarText( 'finddefaddsearchpath', CurMode)
   SetMenuVarText( 'finddefsearchrelative', CurMode)
*/
   --dprintf( 'MenuInit: finddefsearchtype = 'finddefsearchtype', finddefexact = 'finddefexact', finddefprototypes = 'finddefprototypes', finddefcomlit = 'finddefcomlit', finddefallmatches = 'finddefallmatches)
   SetMenuAttribute( 'finddefincludes',   MIA_CHECKED, not (finddefsearchtype = 'I'))
   SetMenuAttribute( 'finddeffilemasks',  MIA_CHECKED, not (finddefsearchtype = 'F'))
   SetMenuAttribute( 'finddefgrep',       MIA_CHECKED, not (finddefsearchtype = 'G'))
   SetMenuAttribute( 'finddefwords',      MIA_CHECKED, (wordpos( finddefsearchtype, 'I F G') > 0))

   SetMenuAttribute( 'finddefexact',      MIA_CHECKED, not finddefexact)
   SetMenuAttribute( 'finddefprototypes', MIA_CHECKED, not finddefprototypes)
   SetMenuAttribute( 'finddefcomlit',     MIA_CHECKED, not finddefcomlit)
   SetMenuAttribute( 'finddefallmatches', MIA_CHECKED, not finddefallmatches)
   SetMenuAttribute( 'finddefrecursion',  MIA_CHECKED, not finddefrecursion)

   SetMenuAttribute( 'finddefmode',       MIA_DISABLED, GetFindDefOptionDefined( 'finddefmode'))

   SetMenuAttribute( 'finddeffilemasks',  MIA_DISABLED, GetFindDefOptionDefined( 'finddeffilemasks'))
   SetMenuAttribute( 'finddefincludes',   MIA_DISABLED, GetFindDefOptionDefined( 'finddefincludes'))
   SetMenuAttribute( 'finddefgrep',       MIA_DISABLED, GetFindDefOptionDefined( 'finddefgrep'))
   SetMenuAttribute( 'finddefwords',      MIA_DISABLED, GetFindDefOptionDefined( 'finddefwords'))

   SetMenuAttribute( 'finddefexact',      MIA_DISABLED, GetFindDefOptionDefined( 'finddefexact'))
   SetMenuAttribute( 'finddefprototypes', MIA_DISABLED, GetFindDefOptionDefined( 'finddefprototypes'))
   SetMenuAttribute( 'finddefcomlit',     MIA_DISABLED, GetFindDefOptionDefined( 'finddefcomlit'))
   SetMenuAttribute( 'finddefallmatches', MIA_DISABLED, GetFindDefOptionDefined( 'finddefallmatches'))
   SetMenuAttribute( 'finddefrecursion',  MIA_DISABLED, GetFindDefOptionDefined( 'finddefrecursion'))
/*
   SetMenuAttribute( 'finddefaddsearchpath',  MIA_DISABLED, not 1)
   SetMenuAttribute( 'finddefsearchrelative', MIA_DISABLED, not 1)
*/

defc menuinit_tags
   -- Tags_Supported is defined in TAGS.E.
   fLinkedBefore = 0
   if linked( 'tags') then
      fLinkedBefore = 1
   else
      'Link tags'
   endif
   if isadefproc( 'Tags_Supported') then
      on = Tags_Supported( GetMode())
   else
      on = 1
   endif
   if not fLinkedBefore then
      'UnLink tags'
   endif
   SetMenuAttribute( 'tagfindcurrent', MIA_DISABLED, on)
   SetMenuAttribute( 'tagfindall',     MIA_DISABLED, on)
   SetMenuAttribute( 'tagscan',        MIA_DISABLED, on)

; ------------------------------------ View ---------------------------------
defc menuinit_view
   SetMenuAttribute( 'softwrap', MIA_CHECKED, GetWrapped() = 0)

   KeyPath = '\NEPMD\User\Format\Wrap\ToWindow'
   on = (QueryConfigKey( KeyPath) = 1)
   SetMenuAttribute( 'softwraptowindow', MIA_CHECKED, not on)

   SetMenuAttribute( 'nextview', MIA_DISABLED, .currentview_of_file <> .nextview_of_file)

   on = (filesinring() > 1)
   SetMenuAttribute( 'listring', MIA_DISABLED, on)
   SetMenuAttribute( 'nextfile', MIA_DISABLED, on)
   SetMenuAttribute( 'prevfile', MIA_DISABLED, on)

   on = not IsOnlyEpmWindow()
   SetMenuAttribute( 'switchnextwindow', MIA_DISABLED, on)

defc ToggleSoftWrapToWindow
   KeyPath = '\NEPMD\User\Format\Wrap\ToWindow'
   on = (QueryConfigKey( KeyPath) = 1)
   on = not on
   call WriteConfigKey( KeyPath, on)
   SetMenuAttribute( 'softwraptowindow',  MIA_CHECKED, not on)
   --SetMenuAttribute( 'softwraptowindow2', MIA_CHECKED, not on)

   if GetWrapped() then
      --'UnWrap'  -- Done by SoftWrap
      if on then
         -- PostMe doesn't reduce crashes.
         'SoftWrap win'
      else
         'SoftWrap 0'
      endif
   endif

; ------------------------------------ Run ----------------------------------
defc menuinit_run
   is_shell = IsAShell()
   SetMenuAttribute( 'writetoshell',     MIA_DISABLED, is_shell)
   SetMenuAttribute( 'sendbreaktoshell', MIA_DISABLED, is_shell)

defc menuinit_configureshell
   KeyPath = '\NEPMD\User\Shell\FilenameCompletion'
   on = (QueryConfigKey( KeyPath) <> 0)
   SetMenuAttribute( 'filenamecompletion', MIA_CHECKED, not on)

   KeyPath = '\NEPMD\User\Shell\Alias'
   on = (QueryConfigKey( KeyPath) <> 0)
   SetMenuAttribute( 'alias', MIA_CHECKED, not on)

   file = ExpandEnvVars( '%NEPMD_USERDIR%\bin\alias.cfg')
   file_exist = Exist( file)
   if file_exist then
      SetMenuText( 'editalias', 'Edit ALIAS.~CFG')
   else
      SetMenuText( 'editalias', 'Create ALIAS.~CFG')
   endif

defc menuinit_treecommands
   is_tree = upcase( leftstr( .filename, 5)) = '.TREE'
   SetMenuAttribute( 'treesort', MIA_DISABLED, is_tree)
   SetMenuAttribute( 'treeit'  , MIA_DISABLED, is_tree)

defc menuinit_recordkeys
   universal recordingstate
   universal recfid

   -- SetMenuText appends the accel key defs automatically
   if recordingstate = 'R' then
      SetMenuText( 'startrecording', 'End ~recording')
      SetMenuText( 'playback',       'End recording & ~playback')
   else
      SetMenuText( 'startrecording', 'Start ~recording')
      SetMenuText( 'playback',       '~Playback')
   endif
   enabled = ((recordingstate = 'R' | recordingstate = 'P') & validatefileid( recfid) = 1)
   SetMenuAttribute( 'playback', MIA_DISABLED, enabled)
   new = GetFileSpec( RecKeysGetFile(), 'N')
   SetMenuVarText( 'selectrecordkeysmacro', new)

defc menuinit_emacros
   universal rexx_profile

   file = ExpandEnvVars( '%NEPMD_USERDIR%\ex\macroadd.lst')
   file_exist = Exist( file)
   if file_exist then
      SetMenuText( 'editaddlst', 'Edit MACROA~DD.LST')
   else
      SetMenuText( 'editaddlst', 'Create MACROA~DD.LST')
   endif

   SetMenuAttribute( 'activateprofile',  MIA_CHECKED, not rexx_profile)

   file = ExpandEnvVars( '%NEPMD_USERDIR%\bin\profile.erx')
   file_exist = Exist( file)
   SetMenuAttribute( 'activateprofile', MIA_DISABLED, file_exist)
   if file_exist then
      SetMenuText( 'editprofile', 'Edit ~PROFILE.ERX')
   else
      SetMenuText( 'editprofile', 'Create ~PROFILE.ERX')
   endif

   file = ExpandEnvVars( '%NEPMD_USERDIR%\macros\mystuff.e')
   file_exist = Exist( file)
   if file_exist then
      SetMenuText( 'editmystuff', 'Edit MY~STUFF.E')
   else
      SetMenuText( 'editmystuff', 'Create MY~STUFF.E')
   endif

; ------------------------------------ Keyword help -------------------------
defc menuinit_keywordhelp
   KeyPath = '\NEPMD\User\KeywordHelp\NewView\UseIfFound'
   on = (QueryConfigKey( KeyPath) <> 0)
   SetMenuAttribute( 'usenewview',        MIA_CHECKED, not on)
   SetMenuAttribute( 'usenewviewxsearch', MIA_DISABLED, on)

   KeyPath = '\NEPMD\User\KeywordHelp\NewView\ExtendedSearch'
   on = (QueryConfigKey( KeyPath) <> 0)
   SetMenuAttribute( 'usenewviewxsearch', MIA_CHECKED, not on)

; ------------------------------------ Preferences --------------------------
; This is automatically executed on opening the prefspopup menu.
defc menuinit_prefspopup
   MenuText_editoptions()
   MenuText_saveoptions()
   MenuText_searchoptions()

defproc MenuText_editoptions
   universal default_edit_options

   new = default_edit_options
   SetMenuVarText( 'editoptions', new)

defproc MenuText_saveoptions
   universal default_save_options

   new = default_save_options
   SetMenuVarText( 'saveoptions', new)

defproc MenuText_searchoptions
   universal default_search_options

   new = default_search_options
   SetMenuVarText( 'searchoptions', new)

defc menuinit_basicconfig
   universal cua_marking_switch
   universal default_stream_mode
   universal cua_menu_accel

   BitStr = GetPresetBitStr()
   SetMenuAttribute( 'presetusercua', MIA_CHECKED, not (BitStr = PRESET_BITSTR_USER_CUA))
   SetMenuAttribute( 'presetprogcua', MIA_CHECKED, not (BitStr = PRESET_BITSTR_PROG_CUA))
   SetMenuAttribute( 'presetprogstd', MIA_CHECKED, not (BitStr = PRESET_BITSTR_PROG_STD))

   SetMenuAttribute( 'advancedmarking2', MIA_CHECKED, cua_marking_switch)

   on = (lowcase( GetAddKeyDefs()) = 'cua')
   SetMenuAttribute( 'cuakeyset', MIA_CHECKED, not on)

   SetMenuAttribute( 'defaultstreammode2', MIA_CHECKED, not default_stream_mode)

   KeyPath = '\NEPMD\User\CursorPos\CursorEverywhere'
   on = (QueryConfigKey( KeyPath) <> 0)
   SetMenuAttribute( 'cursoreverywhere2', MIA_CHECKED, not on)

   KeyPath1 = '\NEPMD\User\SpecialKeys\HomeToggles'
   on1 = QueryConfigKey( KeyPath1)
   KeyPath2 = '\NEPMD\User\SpecialKeys\EndToggles'
   on2 = QueryConfigKey( KeyPath2)
   on = (on1 = 1 & on2 = 1)
   SetMenuAttribute( 'homeendtoggle', MIA_CHECKED, not on)

   on = not cua_menu_accel
   SetMenuAttribute( 'blockaltletterkeys2', MIA_CHECKED, not on)

   Hint = 'Let 'word( GetExpandKeys(), 1)' and 'word( GetExpandKeys(), 2)' do syntax expansion'
   mid = GetAVar( 'mid_defaultsyntaxexpansion2')
   call SetAVar( 'midhint_'mid, Hint)
   KeyPath = '\NEPMD\User\SyntaxExpansion'
   on = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'defaultsyntaxexpansion2', MIA_CHECKED, not on)

   KeyPath1 = '\NEPMD\User\SpecialKeys\MatchFindOnMove'
   on1 = QueryConfigKey( KeyPath1)
   KeyPath2 = '\NEPMD\User\SpecialKeys\MatchFindOpening'
   on2 = QueryConfigKey( KeyPath2)
   KeyPath3 = '\NEPMD\User\SpecialKeys\MatchInsertPair'
   on3 = QueryConfigKey( KeyPath3)
   on = (on1 = 1 & on2 = 1 & on3 = 1)
   SetMenuAttribute( 'matchbrackets', MIA_CHECKED, not on)

   on1 = QueryModeSpecific( 'TABS')
   on2 = QueryModeSpecific( 'MARGINS')
   on = (on1 = 1 & on2 = 1)
   SetMenuAttribute( 'modespecificsettings', MIA_CHECKED, not on)

defc toggle_AddKeyDefs
   on = (lowcase( GetAddKeyDefs()) = 'cua')
   on = not on
   if on then
      'SetAddKeyDefs cua'
   else
      'SetAddKeyDefs'
   endif
   'RefreshInfoLine KEYSET'
   SetMenuAttribute( 'cuakeyset', MIA_CHECKED, not on)

defc toggle_HomeEndToggle
   KeyPath1 = '\NEPMD\User\SpecialKeys\HomeToggles'
   on1 = QueryConfigKey( KeyPath1)
   KeyPath2 = '\NEPMD\User\SpecialKeys\EndToggles'
   on2 = QueryConfigKey( KeyPath2)
   on = (on1 = 1 & on2 = 1)
   on = not on
   WriteConfigKey( KeyPath1, on)
   WriteConfigKey( KeyPath2, on)
   SetMenuAttribute( 'homeendtoggle', MIA_CHECKED, not on)

defc toggle_MatchBrackets
   KeyPath1 = '\NEPMD\User\SpecialKeys\MatchFindOnMove'
   on1 = QueryConfigKey( KeyPath1)
   KeyPath2 = '\NEPMD\User\SpecialKeys\MatchFindOpening'
   on2 = QueryConfigKey( KeyPath2)
   KeyPath3 = '\NEPMD\User\SpecialKeys\MatchInsertPair'
   on3 = QueryConfigKey( KeyPath3)
   on = (on1 = 1 & on2 = 1 & on3 = 1)
   on = not on
   WriteConfigKey( KeyPath1, on)
   WriteConfigKey( KeyPath2, on)
   WriteConfigKey( KeyPath3, on)
   SetMenuAttribute( 'matchbrackets', MIA_CHECKED, not on)

defc toggle_ModeSpecificSettings
   on1 = QueryModeSpecific( 'TABS')
   on2 = QueryModeSpecific( 'MARGINS')
   on = (on1 = 1 & on2 = 1)
   on = not on
   -- The changed value is written by 'ModeSpecific'.
   'ModeSpecific 'on' Tabs Margins'
   SetMenuAttribute( 'modespecificsettings', MIA_CHECKED, not on)

defc menuinit_editoptions
   'EditOptionsMenuCmd MENUINIT'

defc menuinit_saveoptions
   'SaveOptionsMenuCmd MENUINIT'

defc menuinit_searchoptions
   'SearchOptionsMenuCmd MENUINIT'

; This for the Preferences popup menu only.
defc menuinit_markingsettings
   universal cua_marking_switch

   SetMenuAttribute( 'advancedmarking', MIA_CHECKED, cua_marking_switch)

   KeyPath = '\NEPMD\User\Mark\ShiftMarkExtends'
   on = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'shiftmarkextends', MIA_CHECKED, not (on & not cua_marking_switch))
   SetMenuAttribute( 'shiftmarkextends', MIA_DISABLED, not cua_marking_switch)

   KeyPath = '\NEPMD\User\Mark\UnmarkAfterMove'
   on = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'unmarkaftermove', MIA_CHECKED, not (on & not cua_marking_switch))
   SetMenuAttribute( 'unmarkaftermove', MIA_DISABLED, not cua_marking_switch)

   KeyPath = '\NEPMD\User\Mark\LineInsert'
   LineInsert = QueryConfigKey( KeyPath)
   if LineInsert = 'A' then
      new = 'above'
   else
      new = 'below'
   endif
   SetMenuVarText( 'lineinsert', new)
   SetMenuAttribute( 'lineinsert', MIA_DISABLED, not cua_marking_switch)

   KeyPath = '\NEPMD\User\Mark\DefaultMouseMark'
   DefaultMouseMark = QueryConfigKey( KeyPath)
   if (DefaultMouseMark <> 'B' | cua_marking_switch) then
      DefaultMouseMark = 'C'
      new = 'char'
   else
      new = 'block'
   endif
   SetMenuVarText( 'defaultmousemark', new)
   SetMenuAttribute( 'defaultmousemark', MIA_DISABLED, not cua_marking_switch)

   KeyPath = '\NEPMD\User\Mark\DefaultPaste'
   next = substr( upcase( QueryConfigKey( KeyPath)), 1, 1)
   if next = 'L' then
      new = 'line'
   elseif next = 'B' then
      new = 'block'
   else
      new = 'char'
   endif
   SetMenuVarText( 'defaultpaste', new)

   KeyPath = '\NEPMD\User\Mark\DragAlwaysMarks'
   on = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'dragalwaysmarks', MIA_CHECKED, not (on | cua_marking_switch))
   SetMenuAttribute( 'dragalwaysmarks', MIA_DISABLED, not cua_marking_switch)

   KeyPath = '\NEPMD\User\SearchDialog\SearchMarkIfMarked'
   on = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'searchinmark', MIA_CHECKED, not on)

   KeyPath = '\NEPMD\User\SearchDialog\SearchMarkedWords'
   on = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'searchmarkedwords', MIA_CHECKED, not on)

defc menuinit_keysets
    MenuText_keydefs()

defproc MenuText_keydefs
   None = '-none-'
   new = GetAddKeyDefs()
   if new = '' then
      new = None
   else
      parse value new with new'keys'
      new = new'keys'
   endif
   SetMenuVarText( 'keydefs', new)

defc menuinit_streamnewline
   universal default_stream_mode

   SetMenuAttribute( 'defaultstreammode', MIA_CHECKED, not default_stream_mode)
   MenuText_newlineenter()

   MenuText_newline( 'newline')
   MenuText_newlinesplit( 'newline')
   MenuText_newlinecol( 'newline')
   MenuText_newline( 's_newline')
   MenuText_newlinesplit( 's_newline')
   MenuText_newlinecol( 's_newline')
   MenuText_newline( 'a_newline')
   MenuText_newlinesplit( 'a_newline')
   MenuText_newlinecol( 'a_newline')
   MenuText_newline( 'c_newline')
   MenuText_newlinesplit( 'c_newline')
   MenuText_newlinecol( 'c_newline')

defc toggle_newlineenter
   universal newline_enter
   if newline_enter = 'enter' then
      newline_enter = 'newline'
   else
      newline_enter = 'enter'
   endif
   MenuText_newlineenter()

   MenuText_newline( 'newline')
   MenuText_newlinesplit( 'newline')
   MenuText_newlinecol( 'newline')
   MenuText_newline( 's_newline')
   MenuText_newlinesplit( 's_newline')
   MenuText_newlinecol( 's_newline')
   MenuText_newline( 'a_newline')
   MenuText_newlinesplit( 'a_newline')
   MenuText_newlinecol( 'a_newline')
   MenuText_newline( 'c_newline')
   MenuText_newlinesplit( 'c_newline')
   MenuText_newlinecol( 'c_newline')

defproc MenuText_newlineenter
   universal newline_enter
   if newline_enter = 'enter' then
      new = PADENTER_KEY__MSG
   else
      new = NEWLINE_KEY__MSG
   endif
   SetMenuVarText( 'newlineenter', new)

defc toggle_newlinesplit
   universal newline_enter
   universal default_stream_mode

   miname = arg( 1)
   if miname = '' then
      miname = 'newline'
   endif
   key = lowcase( miname)
   if newline_enter = 'enter' then
      key = ChangeStr( 'newline', key, 'enter')
   endif

   KeyPath = '\NEPMD\User\SpecialKeys'
   if default_stream_mode then
      KeyPath = KeyPath'\Stream'
   else
      KeyPath = KeyPath'\Line'
   endif
   KeyPath = KeyPath'\'key
   next = QueryConfigKey( KeyPath)
   if next = '' then
      next = 'Split,KeepIndent'
   endif
   parse value next with SplitCfg','ColCfg','fCmd','Cmd
   if fCmd <> 1 then
      fCmd = 0
   endif
   UpSplitCfg = upcase( SplitCfg)

   if CtrlIsDown() then
      'ConfigureNewlineCmd' key
      return
   elseif default_stream_mode then
      if fCmd then
         new = 'Split'
         SplitCfg = new
         fCmd = 0
      elseif UpSplitCfg = 'SPLIT' then
         if Cmd = '' then
            new = '-no cmd-'
         else
            new = Cmd
         endif
         fCmd = 1
      else
         new = 'Split'
         SplitCfg = new
         fCmd = 0
      endif
   else
      if fCmd then
         new = 'Split'
         SplitCfg = new
         fCmd = 0
      elseif UpSplitCfg = 'SPLIT' then
         new = 'Add'
         SplitCfg = new
         fCmd = 0
      elseif UpSplitCfg = 'ADD' then
         new = 'NoAdd'
         SplitCfg = new
         fCmd = 0
      elseif UpSplitCfg = 'NOADD' then
         new = 'SplitIns'
         SplitCfg = new
         fCmd = 0
      elseif UpSplitCfg = 'SPLITINS' then
         new = 'AddIns'
         SplitCfg = new
         fCmd = 0
      elseif UpSplitCfg = 'ADDINS' then
         if Cmd = '' then
            new = '-no cmd-'
         else
            new = Cmd
         endif
         fCmd = 1
      else
         new = 'Split'
         SplitCfg = new
         fCmd = 0
      endif
   endif

   SetMenuVarText( miname'split', new)
   Val = SplitCfg','ColCfg','fCmd','Cmd
   WriteConfigKey( KeyPath, Val)
   -- Disable col menu item if cmd is selected
   on = not fCmd
   SetMenuAttribute( GetAVar( 'mid_'miname'col'), MIA_DISABLED, on)

defc toggle_newlinecol
   universal newline_enter
   universal default_stream_mode

   miname = arg( 1)
   if miname = '' then
      miname = 'newline'
   endif
   key = lowcase( miname)
   if newline_enter = 'enter' then
      key = ChangeStr( 'newline', key, 'enter')
   endif

   KeyPath = '\NEPMD\User\SpecialKeys'
   if default_stream_mode then
      KeyPath = KeyPath'\Stream'
   else
      KeyPath = KeyPath'\Line'
   endif
   KeyPath = KeyPath'\'key
   next = QueryConfigKey( KeyPath)
   if next = '' then
      next = 'Split,KeepIndent'
   endif
   parse value next with SplitCfg','ColCfg','fCmd','Cmd
   if fCmd <> 1 then
      fCmd = 0
   endif
   UpColCfg   = upcase( ColCfg)

   if default_stream_mode then
      if UpColCfg = 'KEEPINDENT' then
         new = 'NoIndent'
      elseif UpColCfg = 'NOINDENT' then
         new = 'Margins'
      elseif UpColCfg = 'MARGINS' then
         new = 'ParMargin'
      else
         new = 'KeepIndent'
      endif
   else
      if UpColCfg = 'KEEPINDENT' then
         new = 'KeepCol'
      elseif UpColCfg = 'KEEPCOL' then
         new = 'NoIndent'
      elseif UpColCfg = 'NOINDENT' then
         new = 'Margins'
      elseif UpColCfg = 'MARGINS' then
         new = 'ParMargin'
      else
         new = 'KeepIndent'
      endif
   endif
   ColCfg = new

   SetMenuVarText( miname'col', new)
   Val = SplitCfg','ColCfg','fCmd','Cmd
   WriteConfigKey( KeyPath, Val)

defproc MenuText_newlinesplit
   universal newline_enter
   universal default_stream_mode

   miname = arg( 1)
   if miname = '' then
      miname = 'newline'
   endif
   key = lowcase( miname)
   if newline_enter = 'enter' then
      key = ChangeStr( 'newline', key, 'enter')
   endif

   KeyPath = '\NEPMD\User\SpecialKeys'
   if default_stream_mode then
      KeyPath = KeyPath'\Stream'
   else
      KeyPath = KeyPath'\Line'
   endif
   KeyPath = KeyPath'\'key
   next = QueryConfigKey( KeyPath)
   if next = '' then
      next = 'Split,KeepIndent'
   endif
   parse value next with SplitCfg','ColCfg','fCmd','Cmd
   if fCmd <> 1 then
      fCmd = 0
      new = SplitCfg
   else
      if Cmd = '' then
         new = '-no cmd-'
      else
         new = Cmd
      endif
   endif
   -- Disable col menu item if cmd is selected
   on = not fCmd
   SetMenuAttribute( GetAVar( 'mid_'miname'col'), MIA_DISABLED, on)

   SetMenuVarText( miname'split', new)

defproc MenuText_newlinecol
   universal newline_enter
   universal default_stream_mode

   miname = arg( 1)
   if miname = '' then
      miname = 'newline'
   endif
   key = lowcase( miname)
   if newline_enter = 'enter' then
      key = ChangeStr( 'newline', key, 'enter')
   endif

   KeyPath = '\NEPMD\User\SpecialKeys'
   if default_stream_mode then
      KeyPath = KeyPath'\Stream'
   else
      KeyPath = KeyPath'\Line'
   endif
   KeyPath = KeyPath'\'key
   next = QueryConfigKey( KeyPath)
   if next = '' then
      next = 'Split,KeepIndent'
   endif
   parse value next with SplitCfg','ColCfg','fCmd','Cmd

   SetMenuVarText( miname'col', ColCfg)

defproc MenuText_newline
   universal newline_enter
   universal default_stream_mode

   miname = arg( 1)
   if miname = '' then
      miname = 'newline'
   endif
   key = lowcase( miname)
   if newline_enter = 'enter' then
      key = ChangeStr( 'newline', key, 'enter')
   endif

   Rest = key
   KeyString = ''
   do while Rest <> ''
      if pos( 'c_', Rest) = 1 then
         KeyString = KeyString''CTRL_KEY__MSG'+'
         Rest = substr( Rest, 3)
      endif
      if pos( 'a_', Rest) = 1 then
         KeyString = KeyString''ALT_KEY__MSG'+'
         Rest = substr( Rest, 3)
      endif
      if pos( 's_', Rest) = 1 then
         KeyString = KeyString''SHIFT_KEY__MSG'+'
         Rest = substr( Rest, 3)
      endif
      if Rest <> '' then
         VKString = GetVKMenuName( Rest)
         if VKString <> '' then
            KeyString = KeyString''VKString
         else
            KeyString = KeyString''upcase( Rest)
         endif
         Rest = ''
      endif
   enddo

   if default_stream_mode then
      stream_line = 'stream'
   else
      stream_line = 'line'
   endif

   SetMenuText( miname, KeyString' in 'stream_line' mode')

defc ResetNewlineKeyDefs
   universal numcfgkeys

   numcfgkeys = 0
   DeleteConfigTree( '\NEPMD\User\SpecialKeys\Stream')
   DeleteConfigTree( '\NEPMD\User\SpecialKeys\Line')
   'sayerror' numcfgkeys 'key(s) deleted.'

   MenuText_newlinesplit( 'newline')
   MenuText_newlinecol( 'newline')
   MenuText_newlinesplit( 's_newline')
   MenuText_newlinecol( 's_newline')
   MenuText_newlinesplit( 'a_newline')
   MenuText_newlinecol( 'a_newline')
   MenuText_newlinesplit( 'c_newline')
   MenuText_newlinecol( 'c_newline')

defc menuinit_tabsandmargins
   universal cua_menu_accel
   universal join_after_wrap

   MenuText_tab()
   MenuText_c_tab()
   MenuText_c_a_tab()
   MenuText_s_backtab()
   MenuText_c_s_backtab()

   KeyPath = '\NEPMD\User\SpecialKeys\Tab\MatchWordsAbove'
   on = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'defaultmatchtab', MIA_CHECKED, not on)

   on = tabglyph()
   SetMenuAttribute( 'showtabs', MIA_CHECKED, not on)

   MenuText_defaulttabs()

   on = QueryModeSpecific( 'TABS')
   SetMenuAttribute( 'modespecifictabs', MIA_CHECKED, not on)

   MenuText_defaultmargins()

   on = QueryModeSpecific( 'MARGINS')
   SetMenuAttribute( 'modespecificmargins', MIA_CHECKED, not on)

   SetMenuAttribute( 'joinafterwrap', MIA_CHECKED, not join_after_wrap)

defc menuinit_specialkeys
   universal cua_menu_accel
   universal cua_marking_switch

   KeyPath = '\NEPMD\User\CursorPos\CursorEverywhere'
   on = (QueryConfigKey( KeyPath) <> 0)
   SetMenuAttribute( 'cursoreverywhere', MIA_CHECKED, not on)

   KeyPath = '\NEPMD\User\CursorPos\KeepCursorOnScreen'
   on = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'keepcursoronscreen', MIA_CHECKED, not on)

   KeyPath = '\NEPMD\User\CursorPos\VPosAfterLocate'
   val = QueryConfigKey( KeyPath)
   call MenuText_VPosAfterLocate()
   SetMenuAttribute( 'scrollafterlocate', MIA_DISABLED, val = 'S')

   MenuText_scrollafterlocate()

   KeyPath = '\NEPMD\User\CursorPos\HighlightCursor'
   on = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'highlightcursor', MIA_CHECKED, not on)

   KeyPath = '\NEPMD\User\Mark\DefaultMouseMark'
   DefaultMouseMark = QueryConfigKey( KeyPath)
   if (DefaultMouseMark <> 'B' | cua_marking_switch) then
      DefaultMouseMark = 'C'
      new = 'char'
   else
      new = 'block'
   endif
   SetMenuVarText( 'defaultmousemark2', new)
   SetMenuAttribute( 'defaultmousemark2', MIA_DISABLED, not cua_marking_switch)

   KeyPath = '\NEPMD\User\EditWindow\ClickOnlyGivesFocus'
   ClickOnlyGivesFocus = QueryConfigKey( KeyPath)
   fIgnore = 0
   if cua_marking_switch then
      fIgnore = (ClickOnlyGivesFocus = 1 | ClickOnlyGivesFocus = 'CUA')
   else
      fIgnore = (ClickOnlyGivesFocus = 1 | ClickOnlyGivesFocus = 'ADVANCED')
   endif
   on = (fIgnore = 1)
   SetMenuAttribute( 'clickonlygivesfocus', MIA_CHECKED, not on)

   KeyPath = '\NEPMD\User\SpecialKeys\HomeToggles'
   on = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'hometoggles', MIA_CHECKED, not on)

   KeyPath = '\NEPMD\User\SpecialKeys\EndToggles'
   on = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'endtoggles', MIA_CHECKED, not on)

   MenuText_syntaxexpansionkeys()
   KeyPath = '\NEPMD\User\SyntaxExpansion'
   on = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'syntaxexpansionkeys', MIA_DISABLED, on)

   SetMenuAttribute( 'blockaltletterkeys', MIA_CHECKED, cua_menu_accel)

   KeyPath = '\NEPMD\User\SpecialKeys\Alt\BlockLeftAlt'
   on = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'blockleftaltkey', MIA_CHECKED, not on)

   KeyPath = '\NEPMD\User\SpecialKeys\Alt\BlockRightAlt'
   on = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'blockrightaltkey', MIA_CHECKED, not on)

defc ChangeTab
   KeyString = arg( 1)
   KeyPath = '\NEPMD\User\SpecialKeys\'KeyString
   Options = QueryConfigKey( KeyPath)
   NewOptions = Options
   if wordpos( KeyString, 'tab c_tab c_a_tab') then
      List = 'Text,Spaces Text,Tab TextIns,Spaces TextIns,Tab Cursor'
      wp = wordpos( Options, List)
      Nextwp = wp + 1
      if Nextwp > words( List) then
         Nextwp = 1
      endif
      NewOptions = word( List, Nextwp)
   elseif wordpos( KeyString, 's_backtab c_s_backtab') then
      List = 'Text Text,Whitespace TextIns TextIns,Whitespace Cursor'
      wp = wordpos( Options, List)
      Nextwp = wp + 1
      if Nextwp > words( List) then
         Nextwp = 1
      endif
      NewOptions = word( List, Nextwp)
   endif
   if NewOptions <> Options & NewOptions <> '' then
      WriteConfigKey( KeyPath, NewOptions)
   endif
   SetMenuVarText( KeyString, TabOptionsToText( NewOptions))

defproc TabOptionsToText
   Options = arg( 1)
   Text = ''
   if Options = 'Text,Spaces' then
      Text = 'moves text with spaces'
   elseif Options = 'Text,Tab' then
      Text = 'moves text with tab'
   elseif Options = 'TextIns,Spaces' then
      Text = 'moves text in insert mode with spaces'
   elseif Options = 'TextIns,Tab' then
      Text = 'moves text in insert mode with tab'
   elseif Options = 'Cursor' then
      Text = 'moves cursor'
   elseif Options = 'Text' then
      Text = 'moves text'
   elseif Options = 'Text,Whitespace' then
      Text = 'moves text over whitespace'
   elseif Options = 'TextIns' then
      Text = 'moves text in insert mode'
   elseif Options = 'TextIns,Whitespace' then
      Text = 'moves text in insert mode over whitespace'
   endif
   return Text

defproc MenuText_tab
   KeyString = 'tab'
   KeyPath = '\NEPMD\User\SpecialKeys\'KeyString
   Options = QueryConfigKey( KeyPath)
   SetMenuVarText( KeyString, TabOptionsToText( Options))

defproc MenuText_c_tab
   KeyString = 'c_tab'
   KeyPath = '\NEPMD\User\SpecialKeys\'KeyString
   Options = QueryConfigKey( KeyPath)
   SetMenuVarText( KeyString, TabOptionsToText( Options))

defproc MenuText_c_a_tab
   KeyString = 'c_a_tab'
   KeyPath = '\NEPMD\User\SpecialKeys\'KeyString
   Options = QueryConfigKey( KeyPath)
   SetMenuVarText( KeyString, TabOptionsToText( Options))

defproc MenuText_s_backtab
   KeyString = 's_backtab'
   KeyPath = '\NEPMD\User\SpecialKeys\'KeyString
   Options = QueryConfigKey( KeyPath)
   SetMenuVarText( KeyString, TabOptionsToText( Options))

defproc MenuText_c_s_backtab
   KeyString = 'c_s_backtab'
   KeyPath = '\NEPMD\User\SpecialKeys\'KeyString
   Options = QueryConfigKey( KeyPath)
   SetMenuVarText( KeyString, TabOptionsToText( Options))

defproc MenuText_defaulttabs
   KeyPath    = '\NEPMD\User\Tabs'
   DefaultVal = QueryConfigKey( KeyPath)

   getfileid fid
   SettingName = 'tabs'
   SettingSrc  = GetAVar( fid'.'SettingName'.src')
   if SettingSrc = 'EA' then
      AppendText = ' (overridden by file EAs)'
   elseif SettingSrc = 'file' then
      AppendText = ' (overridden by file settings)'
   elseif SettingSrc = 'mode' then
      AppendText = ' (overridden by mode settings)'
   else
      AppendText = ''
   endif

   new = DefaultVal
   --dprintf( 'MenuText_defaulttabs: EaVal = 'EaVal', FileVal = 'FileVal', ModeVal = 'ModeVal', DefaultVal = 'DefaultVal', on = 'on', AppendText = 'AppendText)
   SetMenuVarText( 'defaulttabs', new, AppendText)

defproc MenuText_defaultmargins
   KeyPath    = '\NEPMD\User\Margins'
   DefaultVal = QueryConfigKey( KeyPath)

   getfileid fid
   SettingName = 'margins'
   SettingSrc  = GetAVar( fid'.'SettingName'.src')
   if SettingSrc = 'EA' then
      AppendText = ' (overridden by file EAs)'
   elseif SettingSrc = 'file' then
      AppendText = ' (overridden by file settings)'
   elseif SettingSrc = 'mode' then
      AppendText = ' (overridden by mode settings)'
   else
      AppendText = ''
   endif

   new = DefaultVal
   --dprintf( 'MenuText_defaultmargins: EaVal = 'EaVal', FileVal = 'FileVal', ModeVal = 'ModeVal', DefaultVal = 'DefaultVal', on = 'on', AppendText = 'AppendText)
   SetMenuVarText( 'defaultmargins', new, AppendText)

defproc MenuText_vposafterlocate
   KeyPath = '\NEPMD\User\CursorPos\VPosAfterLocate'
   val = QueryConfigKey( KeyPath)
   if val = 'C' then
      new = 'center'
   elseif val = 'K' then
      new = 'keep'
   else
      new = 'scroll'
   endif
   SetMenuVarText( 'vposafterlocate', new)

defc SwitchVPosAfterLocate
   KeyPath = '\NEPMD\User\CursorPos\VPosAfterLocate'
   val = QueryConfigKey( KeyPath)
   if val = 'C' then
      val = 'K'
   elseif val = 'K' then
      val = 'S'
   else
      val = 'C'
   endif
   call WriteConfigKey( KeyPath, val)
   call MenuText_VPosAfterLocate()
   SetMenuAttribute( 'scrollafterlocate', MIA_DISABLED, val = 'S')

defproc MenuText_scrollafterlocate
   KeyPath = '\NEPMD\User\CursorPos\ScrollAfterLocate'
   new = QueryConfigKey( KeyPath)
   SetMenuVarText( 'scrollafterlocate', new)

defproc MenuText_syntaxexpansionkeys
   new = GetExpandKeys()
   SetMenuVarText( 'syntaxexpansionkeys', new)

defc menuinit_windowsdialogs
   universal cua_marking_switch

   KeyPath = '\NEPMD\User\EditWindow\ClickOnlyGivesFocus'
   ClickOnlyGivesFocus = QueryConfigKey( KeyPath)
   fIgnore = 0
   if cua_marking_switch then
      fIgnore = (ClickOnlyGivesFocus = 1 | ClickOnlyGivesFocus = 'CUA')
   else
      fIgnore = (ClickOnlyGivesFocus = 1 | ClickOnlyGivesFocus = 'ADVANCED')
   endif
   on = (fIgnore = 1)
   SetMenuAttribute( 'clickonlygivesfocus2', MIA_CHECKED, not on)

   KeyPath = '\NEPMD\User\SearchDialog\FocusToEditWindow'
   on = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'focustoeditwindow', MIA_CHECKED, not on)

   KeyPath = '\NEPMD\User\SearchDialog\SearchMarkIfMarked'
   on = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'searchinmark2', MIA_CHECKED, not on)

   KeyPath = '\NEPMD\User\SearchDialog\SearchMarkedWords'
   on = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'searchmarkedwords2', MIA_CHECKED, not on)

   KeyPath = '\NEPMD\User\EditWindow\MaxCols'
   MaxCols = QueryConfigKey( KeyPath)
   Text = GetMenuCondText( 'startwithmaxcols', MaxCols = 0, MaxCols <> 0)
   Text = ExpandVarText( Text, MaxCols)
   SetMenuText( 'startwithmaxcols', Text)

-- Max. word length of a command name: 28 chars
defc menuinit_modeexpansion
   KeyPath = '\NEPMD\User\KeywordHighlighting'
   on = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'defaultkeywordhighlighting', MIA_CHECKED, not on)

   KeyPath = '\NEPMD\User\SyntaxExpansion'
   on = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'defaultsyntaxexpansion', MIA_CHECKED, not on)
   SetMenuAttribute( 'selectcodingstyle', MIA_DISABLED, on)
   MenuHint_defaultsyntaxexpansion()

   KeyPath = '\NEPMD\User\SpecialKeys\MatchFindOnMove'
   on = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'matchfindonmove', MIA_CHECKED, not on)

   KeyPath = '\NEPMD\User\SpecialKeys\MatchFindOpening'
   on = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'matchfindopening', MIA_CHECKED, not on)

   KeyPath = '\NEPMD\User\SpecialKeys\MatchInsertPair'
   on = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'matchinsertpair', MIA_CHECKED, not on)

   CurMode = GetMode()
   None = '-none-'
   new = GetCodingStyle()
   if new = '' then
      new = None
   endif
   SetMenuVarText( 'selectcodingstyle', new, CurMode)
   SetMenuVarText( 'openhilitedirs', CurMode)

   new = GetHiliteCacheDir()
   SetMenuVarText( 'hilitecachedirdialog', new)

defproc MenuHint_defaultsyntaxexpansion
   Hint = 'Let 'word( GetExpandKeys(), 1)' and 'word( GetExpandKeys(), 2)' do syntax expansion'
   mid = GetAVar( 'mid_defaultsyntaxexpansion')
   call SetAVar( 'midhint_'mid, Hint)

defc menuinit_readonlyandlock
   KeyPath = '\NEPMD\User\Readonly'
   on = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'respectreadonly', MIA_CHECKED, not on)

   KeyPath = '\NEPMD\User\Lock\OnModify'
   on = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'lockonmodify', MIA_CHECKED, not on)

defc menuinit_autorestore
   KeyPath = '\NEPMD\User\AutoRestore\CursorPos'
   Enabled = QueryConfigKey( KeyPath)
   Unchecked = (Enabled <> 1)
   SetMenuAttribute( 'restorecursorpos', MIA_CHECKED, Unchecked)

   KeyPath = '\NEPMD\User\History'
   Enabled = QueryConfigKey( KeyPath)
   Unchecked = (Enabled <> 1)
   SetMenuAttribute( 'trackhistorylists', MIA_CHECKED, Unchecked)

   KeyPath = '\NEPMD\User\AutoRestore\Ring\SaveLast'
   Enabled = QueryConfigKey( KeyPath)
   Unchecked = (Enabled <> 1)
   SetMenuAttribute( 'autosavelastring', MIA_CHECKED, Unchecked)

   KeyPath = '\NEPMD\User\AutoRestore\Ring\LoadLast'
   Enabled = QueryConfigKey( KeyPath)
   Unchecked = (Enabled <> 1)
   SetMenuAttribute( 'autoloadlastring', MIA_CHECKED, Unchecked)

defc menuinit_backup
   KeyPath = '\NEPMD\User\AutoSave'
   on = (QueryConfigKey( KeyPath) <> 0)
   WriteConfigKey( KeyPath, on)
   SetMenuAttribute( 'autosaveenabled', MIA_CHECKED, not on)

   new = GetAutoSaveNum()
   SetMenuVarText( 'autosavenumdialog', new)
   SetMenuAttribute( 'autosavenumdialog', MIA_DISABLED, on)

   KeyPath = '\NEPMD\User\Backup'
   on = (QueryConfigKey( KeyPath) = 1)
   WriteConfigKey( KeyPath, on)
   SetMenuAttribute( 'backupenabled', MIA_CHECKED, not on)

   new = GetBackupNum()
   SetMenuVarText( 'backupnumdialog', new)
   SetMenuAttribute( 'backupnumdialog', MIA_DISABLED, on)

   new = GetBackupDir()
   SetMenuVarText( 'backupdirdialog', new)

defc menuinit_workdir
   KeyPath = '\NEPMD\User\StartDir\WorkDir\Type'
   opt = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'workdirprogram', MIA_CHECKED, (opt = 1 | opt = 2))
   SetMenuAttribute( 'workdirprev',    MIA_CHECKED, not (opt = 1))
   SetMenuAttribute( 'workdirfile',    MIA_CHECKED, not (opt = 2))

defc menuinit_opendlgdir
   KeyPath = '\NEPMD\User\StartDir\OpenDlg\Type'
   opt = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'opendlgdirprev', MIA_CHECKED, (opt = 1 | opt = 2))
   SetMenuAttribute( 'opendlgdirwork', MIA_CHECKED, not (opt = 1))
   SetMenuAttribute( 'opendlgdirfile', MIA_CHECKED, not (opt = 2))

defc menuinit_saveasdlgdir
   KeyPath = '\NEPMD\User\StartDir\SaveasDlg\Type'
   opt = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'saveasdlgdirprev', MIA_CHECKED, (opt = 1))
   SetMenuAttribute( 'saveasdlgdirwork', MIA_CHECKED, not (opt = 1))

defc menuinit_prg
   KeyPath = '\NEPMD\User\ProgramObjects\OpenSameWindow'
   opt = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'prgnewwindow',   MIA_CHECKED, not (opt = 1))

   KeyPath = '\NEPMD\User\ProgramObjects\OmitOpenBox'
   opt = QueryConfigKey( KeyPath)
   SetMenuAttribute( 'prgomitopenbox', MIA_CHECKED, not (opt = 1))

defc menuinit_menu
   universal nodismiss

   SetMenuAttribute( 'nodismiss', MIA_CHECKED, not (nodismiss = 32))

defc menuinit_infobars
   universal show_longnames
   universal menu_prompt

   SetMenuAttribute( 'showlongname',  MIA_CHECKED, not show_longnames)
   SetMenuAttribute( 'messageline',   MIA_CHECKED, not QueryFrameControl( 2))
   SetMenuAttribute( 'statusbar',     MIA_CHECKED, not QueryFrameControl( 1))
   SetMenuAttribute( 'infoattop',     MIA_CHECKED, not QueryFrameControl( 32))
   SetMenuAttribute( 'showmenuhints', MIA_CHECKED, not menu_prompt)

defc menuinit_toolbar
   SetMenuAttribute( 'toolbarenabled', MIA_CHECKED, not QueryFrameControl( EFRAMEF_TOOLBAR))

defc menuinit_toolbarstyle
   parse value GetToolbarSetup() with \1 Style \1 Cx \1 Cy \1 .
   fText     = not (Style bitand 16)
   fAutosize = not (Style bitand 4)
   fFlat     = not (Style bitand 8)
   fScaleDel = (not (Style bitand 32)) and (not (Style bitand 64))
   fScaleOr  = (Style bitand 32) and (Style bitand 64)
   fScaleAnd = (Style bitand 32) and (not (Style bitand 64))

   SetMenuAttribute( 'toolbartext',     MIA_CHECKED, not fText)
   SetMenuAttribute( 'toolbarautosize', MIA_CHECKED, not fAutosize)
   SetMenuAttribute( 'toolbarsize',     MIA_DISABLED, not fAutosize)
   SetMenuAttribute( 'toolbarscaling',  MIA_DISABLED, not fAutosize)

   new = Cx'x'Cy
   SetMenuVarText( 'toolbarsize', new)

   if fScaleDel then
      new = 'delete'
   elseif fScaleOr then
      new = 'or'
   else
      new = 'and'
   endif
   SetMenuVarText( 'toolbarscaling', new)

defproc MenuText_toolbarsize
   parse value GetToolbarSetup() with \1 Style \1 Cx \1 Cy \1 .
   new = Cx'x'Cy
   SetMenuVarText( 'toolbarsize', new)

defc menuinit_backgroundbitmap
   universal bitmap_present

   SetMenuAttribute( 'backgroundbitmapenabled', MIA_CHECKED, not bitmap_present)

defc menuinit_cursorstyle
   universal menuloaded
   universal cursordimensions

   if menuloaded then
      KeyPath = '\NEPMD\User\Appearance\CursorStyle'
      Style = QueryConfigKey( KeyPath)
      SetMenuAttribute( 'cursorbar',          MIA_CHECKED, not (Style = 1))
      SetMenuAttribute( 'cursorunderline',    MIA_CHECKED, not (Style = 2))
      SetMenuAttribute( 'cursorspecial',      MIA_CHECKED, not (Style = 3))

      SetMenuAttribute( 'cursorbarwidth',     MIA_DISABLED, (Style = 1))
      SetMenuAttribute( 'cursorspecialdimen', MIA_DISABLED, (Style = 3))

      KeyPath     = '\NEPMD\User\Appearance\CursorWidth'
      CursorWidth = QueryConfigKey( KeyPath)
      new = CursorWidth
      SetMenuVarText( 'cursorbarwidth', new)

      new = cursordimensions
      SetMenuVarText( 'cursorspecialdimen', new)
   endif

defc menuinit_pointerstyle
   universal menuloaded

   if menuloaded then
      KeyPath = '\NEPMD\User\Appearance\PointerStyle'
      Style = QueryConfigKey( KeyPath)
      SetMenuAttribute( 'pointerarrow', MIA_CHECKED, not (Style = 1))
      SetMenuAttribute( 'pointeribeam', MIA_CHECKED, not (Style = 2))
   endif

; ---------------------------------------------------------------------------
; Moved from STDCTRL.E
/*
Ŀ
 what's it called: ToggleControl                                            
                                                                            
 what does it do : The command either toggles a EPM control window on or off
                   or forces a EPM control window on or off.                
                   arg1   = EPM control window handle ID.  Control window   
                            ids given above.  The following windows handles 
                            are currently supported.                        
                            EDITSTATUS, EDITVSCROLL, EDITHSCROLL, and       
                            EDITMSGLINE.                                    
                   arg2   [optional] = force option.                        
                            a value of 0, forces control window off         
                            a value of 1, forces control window on          
                           IF this argument is not specified the window     
                           in question is toggled.                          
                                                                            
                   This command is possible because of the EPM_EDIT_CONTROL 
                   EPM_EDIT_CONTROLSTATUS message.                          
                   (All EPM_EDIT_xxx messages are defined in the ETOOLKT    
                    PACKAGE available on PCTOOLS.)                          
                                                                            
 who and when    : Jerry C.   2/27/89                                       

*/
/*
; Some control ids are replaced by EFRAMEF_ constants. See ToggleFrame.
;   EPMINFO_EDITSTATUSAREA    =  7  -- EFRAMEF_STATUSWND = 1
;   EPMINFO_EDITORMSGAREA     =  8  -- EFRAMEF_MESSAGEWND = 2
;   EPMINFO_EDITORVSCROLL     =  9  -- EFRAMEF_VSCROLLBAR = 8
;   EPMINFO_EDITORHSCROLL     = 10  -- EFRAMEF_HSCROLLBAR = 16
   EPMINFO_EDITORINTERPRETER = 11
   EPMINFO_EDITVIOPS         = 12
   EPMINFO_EDITTITLEBAR      = 13
   EPMINFO_EDITCURSOR        = 14  -- No effect in current EPM version
;   EPMINFO_PARTIALTEXT       = 15  -- No longer used
   EPMINFO_EDITEXSEARCH      = 16
   EPMINFO_EDITMENUHWND      = 17
   EPMINFO_HDC               = 18
   EPMINFO_HINI              = 19
;   EPMINFO_RINGICONS         = 20  -- EFRAMEF_RINGBUTTONS = 4
;   EPMINFO_FILEICON          = 22  -- EFRAMEF_FILEWND = 64
;   EPMINFO_EXTRAWINDOWPOS    = 23  -- EFRAMEF_INFOONTOP = 32
*/
defc ToggleControl
   universal menuloaded

   forceon = 0
   parse arg controlid fon
   if fon <> '' then
      forceon = (fon + 1) * 65536
   else
      fon = not QueryFrameControl( controlid)  -- Query now, since toggling is asynch.
   endif

   call windowmessage( 0, getpminfo( EPMINFO_EDITFRAME),
                       5388,               -- EPM_EDIT_CONTROLTOGGLE
                       controlid + forceon,
                       0)

   if menuloaded then
      -- Set MIA_CHECKED attributes for the case MIA_NODISMISS attribute is on
      ControlIdList = ''     ||
        24 'streammode'      || ' ' ||  -- stream_mode
        25 'advancedmarking' || ' ' ||  -- not cua_marking_switch
        26 'internalkeys'    || ' ' ||  -- should be off
        ''
      p = wordpos( controlid, ControlIdList)
      if p then
         midtext = word( ControlIdList, p + 1)
         mid = GetAVar( 'mid_'midtext)
         -- Check if mid exists, because 'initconfig' sets some controls before the menu
         if mid > '' then
            SetMenuAttribute( mid, MIA_CHECKED, not fon)
         endif
      endif
   endif

; ---------------------------------------------------------------------------
/*                                -- old:
   EFRAMEF_STATUSWND       = 1    -- EPMINFO_EDITSTATUSAREA = 7
   EFRAMEF_MESSAGEWND      = 2    -- EPMINFO_EDITORMSGAREA = 8
   EFRAMEF_RINGBUTTONS     = 4    -- EPMINFO_RINGICONS = 20
   EFRAMEF_VSCROLLBAR      = 8    -- EPMINFO_EDITORVSCROLL = 9
   EFRAMEF_HSCROLLBAR      = 16   -- EPMINFO_EDITORHSCROLL = 10
   EFRAMEF_INFOONTOP       = 32   -- EPMINFO_EXTRAWINDOWPOS = 23
   EFRAMEF_FILEWND         = 64   -- = 8  EPMINFO_FILEICON = 22
   EFRAMEF_DMTBWND         = 128  -- = 16
   EFRAMEF_TASKLISTENTRY   = 256  -- doesn't work?
   EFRAMEF_TOOLBAR         = 2048
   drop style                8192
*/
defc ToggleFrame
   universal menu_prompt
   universal menuloaded

   forceon = 0
   parse arg controlid fon
   if fon <> '' then
      forceon = (fon + 1) * 65536
   else
      fon = not QueryFrameControl( controlid)  -- Query now, since toggling is asynch.
   endif

   call windowmessage( 0, getpminfo( EPMINFO_EDITFRAME),
                       5907,               -- EFRAMEM_ToggleControl
                       controlid + forceon,
                       0)

   -- If 'Info at top' gets activated, deactivate 'Prompting'
   if controlid = 32 then
      if fon then  -- 1=top; 0=bottom.  If now top, turn off.
         menu_prompt = 0
         if menuloaded then
            -- Set MIA_CHECKED attributes for the case MIA_NODISMISS attribute is on
            mid = GetAVar( 'mid_showmenuhints')
            -- Check if mid exists, because 'initconfig' sets some controls before the menu
            if mid > '' then
               SetMenuAttribute( mid, MIA_CHECKED, 1)
            endif
         endif
      endif
   endif

   if menuloaded then
      -- Set MIA_CHECKED attributes for the case MIA_NODISMISS attribute is on
      ControlIdList = ''    ||
         1 'statusbar'      || ' ' ||
         2 'messageline'    || ' ' ||
         4 'rotatebuttons'  || ' ' ||
        16 'scrollbars'     || ' ' ||  -- acts on hscrollbar change only, menuitem for h and vscrollbars
        32 'infoattop'      || ' ' ||
      2048 'toolbarenabled'

      p = wordpos( controlid, ControlIdList)
      if p then
         midtext = word( ControlIdList, p + 1)
         mid = GetAVar( 'mid_'midtext)
         -- Check if mid exists, because 'initconfig' sets some controls before the menu
         if mid > '' then
            SetMenuAttribute( mid, MIA_CHECKED, not fon)
         endif
      endif

      -- ToggleFrame toggles permanently in NewMenu
-- Toolbar enabled is not restored on startup
      ControlIdList = ''                   ||
         1 '\Appearance\InfoLine\Status'   || ' ' ||
         2 '\Appearance\InfoLine\Message'  || ' ' ||
        32 '\Appearance\InfoLine\AtTop'    || ' ' ||
      2048 '\Toolbar'

      p = wordpos( controlid, ControlIdList)
      if p then
         SubKey = word( ControlIdList, p + 1)
         KeyPath = '\NEPMD\User'SubKey
         call WriteConfigKey( KeyPath, fon)
      endif
   endif

; ---------------------------------------------------------------------------
defproc QueryFrameControl( controlid)
   return windowmessage( 1, getpminfo( EPMINFO_EDITFRAME),   -- Send message to edit client
                         5907,               -- EFRAMEM_ToggleControl
                         controlid,
                         1)

; ---------------------------------------------------------------------------
defc toggle_profile
   universal rexx_profile
   universal menuloaded

   rexx_profile = not rexx_profile
   if menuloaded then
      -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
      SetMenuAttribute( 'activateprofile', MIA_CHECKED, not rexx_profile)
      KeyPath = '\NEPMD\User\RexxProfile'
      call WriteConfigKey( KeyPath, rexx_profile)
   endif

; ---------------------------------------------------------------------------
defc toggleprompt, toggle_prompt
   universal menu_prompt
   universal menuloaded

   menu_prompt = not menu_prompt
   if menu_prompt then
      'ToggleFrame 32 0'      -- Force Extra window to bottom.
   endif
   if menuloaded then
      -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
      SetMenuAttribute( 'showmenuhints', MIA_CHECKED, not menu_prompt)
      KeyPath = '\NEPMD\User\Appearance\InfoLine\Message\MenuHints'
      call WriteConfigKey( KeyPath, menu_prompt)
   endif

; ---------------------------------------------------------------------------
; Todo: merge this with defc LoadToolbar.
defc toggle_toolbar
   universal toolbar_loaded
   universal menuloaded
   universal toolbar_present

   'ToggleFrame' EFRAMEF_TOOLBAR
   if not toolbar_loaded then
      'default_toolbar'
   endif
   on = QueryFrameControl( EFRAMEF_TOOLBAR)
   toolbar_present = on
   if menuloaded then
      KeyPath = '\NEPMD\User\Toolbar'
      call WriteConfigKey( KeyPath, on)
   endif

; ---------------------------------------------------------------------------
defc toggle_toolbar_text
   universal menuloaded

   parse value GetToolbarSetup() with \1 Style \1 SetupRest
   fText     = not (Style bitand 16)
   fAutosize = not (Style bitand 4)
   fFlat     = not (Style bitand 8)
   fScaleDel = (not (Style bitand 32)) and (not (Style bitand 64))
   fScaleOr  = (Style bitand 32) and (Style bitand 64)
   fScaleAnd = (Style bitand 32) and (not (Style bitand 64))

   fText = not fText

   Style = 16 * (not fText) + 4 * (not fAutosize) + 8 * (not fFlat) +
           32 * (fScaleAnd) + 96 * (fScaleOr)
   call SetToolbarSetup( \1''Style\1''SetupRest)

   if menuloaded then
      SetMenuAttribute( 'toolbartext', MIA_CHECKED, not fText)
   endif

; ---------------------------------------------------------------------------
defc toggle_toolbar_autosize
   universal menuloaded

   parse value GetToolbarSetup() with \1 Style \1 SetupRest
   fText     = not (Style bitand 16)
   fAutosize = not (Style bitand 4)
   fFlat     = not (Style bitand 8)
   fScaleDel = (not (Style bitand 32)) and (not (Style bitand 64))
   fScaleOr  = (Style bitand 32) and (Style bitand 64)
   fScaleAnd = (Style bitand 32) and (not (Style bitand 64))

   fAutosize = not fAutosize

   Style = 16 * (not fText) + 4 * (not fAutosize) + 8 * (not fFlat) +
           32 * (fScaleAnd) + 96 * (fScaleOr)
   call SetToolbarSetup( \1''Style\1''SetupRest)

   if menuloaded then
      SetMenuAttribute( 'toolbarautosize', MIA_CHECKED, not fAutosize)
      SetMenuAttribute( 'toolbarsize',     MIA_DISABLED, not fAutosize)
      SetMenuAttribute( 'toolbarscaling',  MIA_DISABLED, not fAutosize)
   endif

; ---------------------------------------------------------------------------
defc toggle_toolbar_scaling
   universal menuloaded

   parse value GetToolbarSetup() with \1 Style \1 SetupRest
   fText     = not (Style bitand 16)
   fAutosize = not (Style bitand 4)
   fFlat     = not (Style bitand 8)
   fScaleDel = (not (Style bitand 32)) and (not (Style bitand 64))
   fScaleOr  = (Style bitand 32) and (Style bitand 64)
   fScaleAnd = (Style bitand 32) and (not (Style bitand 64))

   if fScaleAnd then
      fScaleDel = 1
      fScaleOr  = 0
      fScaleAnd = 0
   elseif fScaleDel then
      fScaleDel = 0
      fScaleOr  = 1
      fScaleAnd = 0
   else
      fScaleDel = 0
      fScaleOr  = 0
      fScaleAnd = 1
   endif

   Style = 16 * (not fText) + 4 * (not fAutosize) + 8 * (not fFlat) +
           32 * (fScaleAnd) + 96 * (fScaleOr)
   call SetToolbarSetup( \1''Style\1''SetupRest)

   if menuloaded then
      if fScaleDel then
         new = 'delete'
      elseif fScaleOr then
         new = 'or'
      else
         new = 'and'
      endif
      SetMenuVarText( 'toolbarscaling', new)
   endif

; ---------------------------------------------------------------------------
defc setscrolls
   on = arg( 1)
   'ToggleFrame 8' on
   'ToggleFrame 16' on

; ---------------------------------------------------------------------------
defc toggle_bitmap
   universal bitmap_present
   universal menuloaded

   'SetBackgroundBitmap TOGGLE'
   on = bitmap_present
   if menuloaded then
      -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
      SetMenuAttribute( 'backgroundbitmapenabled', MIA_CHECKED, not on)
   endif

; ---------------------------------------------------------------------------
defc toggle_ClickOnlyGivesFocus
   universal menuloaded

   KeyPath = '\NEPMD\User\EditWindow\ClickOnlyGivesFocus'
   ClickOnlyGivesFocus = QueryConfigKey( KeyPath)
   on = (ClickOnlyGivesFocus <> 0)
   on = not on
   WriteConfigKey( KeyPath, on)
   if menuloaded then
      -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
      SetMenuAttribute( 'clickonlygivesfocus', MIA_CHECKED, not on)
      SetMenuAttribute( 'clickonlygivesfocus2', MIA_CHECKED, not on)
   endif

; ---------------------------------------------------------------------------
defc toggle_SearchMarkIfMarked
   universal menuloaded

   KeyPath = '\NEPMD\User\SearchDialog\SearchMarkIfMarked'
   on = QueryConfigKey( KeyPath)
   on = not on
   WriteConfigKey( KeyPath, on)
   if menuloaded then
      -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
      SetMenuAttribute( 'searchinmark', MIA_CHECKED, not on)
      SetMenuAttribute( 'searchinmark2', MIA_CHECKED, not on)
   endif

; ---------------------------------------------------------------------------
defc toggle_SearchMarkedWords
   universal menuloaded

   KeyPath = '\NEPMD\User\SearchDialog\SearchMarkedWords'
   on = QueryConfigKey( KeyPath)
   on = not on
   WriteConfigKey( KeyPath, on)
   if menuloaded then
      -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
      SetMenuAttribute( 'searchmarkedwords', MIA_CHECKED, not on)
      SetMenuAttribute( 'searchmarkedwords2', MIA_CHECKED, not on)
   endif

; ---------------------------------------------------------------------------
defc toggle_FocusToEditWindow
   universal menuloaded

   KeyPath = '\NEPMD\User\SearchDialog\FocusToEditWindow'
   on = QueryConfigKey( KeyPath)
   on = not on
   WriteConfigKey( KeyPath, on)
   if menuloaded then
      -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
      SetMenuAttribute( 'focustoeditwindow', MIA_CHECKED, not on)
   endif

; ---------------------------------------------------------------------------
defc toggle_readonly
   on = not .readonly
   .readonly = on  -- update the edit/browse mode
   'SetStatusLine'
   -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
   SetMenuAttribute( 'readonly', MIA_CHECKED, not on)

; ---------------------------------------------------------------------------
defc toggle_readonly_attrib
   on = GetReadonly()
   on = not on
   'ReadonlyAttrib' on  -- update the file attribute
   KeyPath = '\NEPMD\User\Readonly'
   RespectReadonly = QueryConfigKey( KeyPath)
   if RespectReadonly then
      .readonly = on  -- update the edit/browse mode
      'SetStatusLine'
   endif
   -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
   SetMenuAttribute( 'readonlyattrib', MIA_CHECKED, not on)
   if RespectReadonly then
      SetMenuAttribute( 'readonly', MIA_CHECKED, not on)
   endif

; ---------------------------------------------------------------------------
defc toggle_locked
   if .lockhandle then
      'unlock'
   else
      'lock'
   endif
   on = (.lockhandle > 0)
   -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
   SetMenuAttribute( 'locked', MIA_CHECKED, not on)

; ---------------------------------------------------------------------------
defc toggle_restore_pos
   KeyPath = '\NEPMD\User\AutoRestore\CursorPos'
   on = QueryConfigKey( KeyPath)
   on = not on
   WriteConfigKey( KeyPath, on)
   SetMenuAttribute( 'restorecursorpos', MIA_CHECKED, not on)

; ---------------------------------------------------------------------------
defc toggle_history
   KeyPath = '\NEPMD\User\History'
   on = QueryConfigKey( KeyPath)
   on = not on
   WriteConfigKey( KeyPath, on)
   SetMenuAttribute( 'trackhistorylists', MIA_CHECKED, not on)

; ---------------------------------------------------------------------------
defc toggle_save_ring
   KeyPath = '\NEPMD\User\AutoRestore\Ring\SaveLast'
   on = QueryConfigKey( KeyPath)
   on = not on
   WriteConfigKey( KeyPath, on)
   -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
   SetMenuAttribute( 'autosavelastring', MIA_CHECKED, not on)

; ---------------------------------------------------------------------------
defc toggle_restore_ring
   KeyPath = '\NEPMD\User\AutoRestore\Ring\LoadLast'
   on = QueryConfigKey( KeyPath)
   on = not on
   WriteConfigKey( KeyPath, on)
   -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
   SetMenuAttribute( 'autoloadlastring', MIA_CHECKED, not on)

; ---------------------------------------------------------------------------
defc toggle_search_backward
   'ToggleSearchDirection'
   on = (GetSearchDirection() = '-')
   SetMenuAttribute( 'searchbackward', MIA_CHECKED, not on)

; ---------------------------------------------------------------------------
defc toggle_default_highlight
   KeyPath = '\NEPMD\User\KeywordHighlighting'
   on = QueryConfigKey( KeyPath)
   on = not on
   WriteConfigKey( KeyPath, on)
   -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
   SetMenuAttribute( 'defaultkeywordhighlighting', MIA_CHECKED, not on)
   opt = 'N'  -- 'N' = don't check for changed .hil and .ini files to make it more stable for a huge ring
   -- Change highlight for every file with default setting
   'RingRefreshSetting DEFAULT SetHighlight 'on opt
   -- Todo: Define a new command, that processes the ring.
   --       Note: When an entire ring should be refreshed, the epmkwds
   --             file needs to be reloaded only once per mode (used by
   --             defc toggle_default_highlight in NEWMENU.E).

; ---------------------------------------------------------------------------
defc toggle_MatchFindOnMove
   KeyPath = '\NEPMD\User\SpecialKeys\MatchFindOnMove'
   on = QueryConfigKey( KeyPath)
   on = not on
   WriteConfigKey( KeyPath, on)
   -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
   SetMenuAttribute( 'matchfindonmove', MIA_CHECKED, not on)

; ---------------------------------------------------------------------------
defc toggle_MatchFindOpening
   KeyPath = '\NEPMD\User\SpecialKeys\MatchFindOpening'
   on = QueryConfigKey( KeyPath)
   on = not on
   WriteConfigKey( KeyPath, on)
   -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
   SetMenuAttribute( 'matchfindopening', MIA_CHECKED, not on)

; ---------------------------------------------------------------------------
defc toggle_MatchInsertPair
   KeyPath = '\NEPMD\User\SpecialKeys\MatchInsertPair'
   on = QueryConfigKey( KeyPath)
   on = not on
   WriteConfigKey( KeyPath, on)
   -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
   SetMenuAttribute( 'matchinsertpair', MIA_CHECKED, not on)

; ---------------------------------------------------------------------------
; Unused
defc toggle_expand
   universal menuloaded
   universal expand_on

   -- Change expand for current file
   expand_on = not expand_on
   'SetExpand' expand_on  -- refresh file setting
   if menuloaded then
      -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
      SetMenuAttribute( 'defaultsyntaxexpansion', MIA_CHECKED, not expand_on)
   endif

; ---------------------------------------------------------------------------
defc toggle_default_expand
   universal menuloaded
   universal expand_on

   KeyPath = '\NEPMD\User\SyntaxExpansion'
   on = QueryConfigKey( KeyPath)
   on = not on
   WriteConfigKey( KeyPath, on)
   -- Query file setting and eventually change universal for current file
   getfileid fid
   next = GetAVar( fid'.expand')  -- query file setting
   if on = 1 & not wordpos( upcase( next), '0 OFF') then
      expand_on = 1
   else
      expand_on = 0
   endif
   'RefreshInfoLine EXPAND'
   if menuloaded then  -- check not required
      -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
      SetMenuAttribute( 'defaultsyntaxexpansion',  MIA_CHECKED, not on)
      SetMenuAttribute( 'defaultsyntaxexpansion2', MIA_CHECKED, not on)
      SetMenuAttribute( 'selectcodingstyle',       MIA_DISABLED, on)
   endif

; ---------------------------------------------------------------------------
defc ToggleModeSpecific
   universal menuloaded

   parse arg SettingName
   SettingName = strip( SettingName)
   if leftstr( upcase( SettingName), 3) = 'SET' then
      SettingName = substr( SettingName, 4)  -- strip 'Set'
   endif
   SettingName = lowcase( SettingName)
   ValidSettings = 'tabs margins'
   if wordpos( SettingName, ValidSettings) then
      'ModeSpecific TOGGLE' SettingName
      if menuloaded then
         -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
         -- Eventually change appended 'overridden' text
         on = QueryModeSpecific( SettingName)
         if SettingName = 'tabs' then
            SetMenuAttribute( 'modespecifictabs',    MIA_CHECKED, not on)
            MenuText_defaulttabs()
         elseif SettingName = 'margins' then
            SetMenuAttribute( 'modespecificmargins', MIA_CHECKED, not on)
            MenuText_defaultmargins()
         endif
      endif
   endif

; ---------------------------------------------------------------------------
; This switches between Advanced and CUA marking
defc toggle_cua_mark, cua_mark_toggle
   universal cua_marking_switch
   universal menuloaded
   universal defaultmenu
   universal cua_menu_accel
   universal cursoreverywhere

   cua_marking_switch = not cua_marking_switch
   KeyPath = '\NEPMD\User\Mark\AdvancedMarking'
   call WriteConfigKey( KeyPath, not cua_marking_switch)
   'ToggleControl 25' cua_marking_switch
   call MH_set_mouse()
   'RefreshInfoLine MARKINGMODE'
   if menuloaded then
      -- Set menu attributes and text for the case MIA_NODISMISS attribute is on
      'menuinit_mark'
   endif
   -- The menu from where the action stems can't be queried with activemenu.
   -- After opening the 'prefspopup' menu, the universal is switched back to
   -- 'default'.
   if GetAVar( 'mid_advancedmarking2') then
      SetMenuAttribute( 'advancedmarking2', MIA_CHECKED, cua_marking_switch)
      'menuinit_markingsettings'
   endif

   -- Toggle also blockaltletterkeys with AdvancedMarking
   cua_menu_accel = cua_marking_switch

   -- Set also cursoreverywhere. It can be changed after that.
   KeyPath = '\NEPMD\User\CursorPos\CursorEverywhere'
   if cua_marking_switch then
      on = 0
   else
      on = 1
   endif
   WriteConfigKey( KeyPath, on)
   cursoreverywhere = on

   -- The menu from where the action stems can't be queried with activemenu.
   -- After opening the 'prefspopup' menu, the universal is switched back to
   -- 'default'.
   if GetAVar( 'mid_advancedmarking2') then
      SetMenuAttribute( 'cursoreverywhere2',   MIA_CHECKED, not on)
      SetMenuAttribute( 'blockaltletterkeys2', MIA_CHECKED, not on)
   endif

   'ReloadKeyset'  -- -> SetKeyset2 -> SwitchKeyset -> RefreshMenu - required for hint text

; ---------------------------------------------------------------------------
defc toggle_lineinsert
   KeyPath = '\NEPMD\User\Mark\LineInsert'
   LineInsert = QueryConfigKey( KeyPath)
   if LineInsert = 'A' then  -- toggle
      LineInsert = 'B'
      new = 'below'
   else
      LineInsert = 'A'
      new = 'above'
   endif
   WriteConfigKey( KeyPath, LineInsert)
   SetMenuVarText( 'lineinsert', new)

; ---------------------------------------------------------------------------
defc toggle_defaultmousemark
   KeyPath = '\NEPMD\User\Mark\DefaultMouseMark'
   DefaultMouseMark = QueryConfigKey( KeyPath)
   if DefaultMouseMark = 'B' then  -- toggle
      DefaultMouseMark = 'C'
      new = 'char'
   else
      DefaultMouseMark = 'B'
      new = 'block'
   endif
   WriteConfigKey( KeyPath, DefaultMouseMark)
   SetMenuVarText( 'defaultmousemark', new)
   SetMenuVarText( 'defaultmousemark2', new)
  'mouse_init'  -- refresh the register_mousehandler defs

; ---------------------------------------------------------------------------
defc toggle_default_paste
   KeyPath = '\NEPMD\User\Mark\DefaultPaste'
   next = substr( upcase( QueryConfigKey( KeyPath)), 1, 1)
   if next = 'L' then      -- toggle L -> C
      style = 'C'
      new = 'char'
   elseif next = 'B' then  -- toggle B -> L
      style = 'L'
      new = 'line'
   else                    -- toggle B -> L
      style = 'B'
      new = 'block'
   endif
   WriteConfigKey( KeyPath, style)
   SetMenuVarText( 'defaultpaste', new)
   'mouse_init'                    -- refresh the register_mousehandler defs

; ---------------------------------------------------------------------------
defc toggle_shift_mark_extends
   KeyPath = '\NEPMD\User\Mark\ShiftMarkExtends'
   on = QueryConfigKey( KeyPath)
   on = not on
   WriteConfigKey( KeyPath, on)
   SetMenuAttribute( 'shiftmarkextends', MIA_CHECKED, not on)

; ---------------------------------------------------------------------------
defc toggle_unmark_after_move
   KeyPath = '\NEPMD\User\Mark\UnmarkAfterMove'
   on = QueryConfigKey( KeyPath)
   on = not on
   WriteConfigKey( KeyPath, on)
   SetMenuAttribute( 'unmarkaftermove', MIA_CHECKED, not on)

; ---------------------------------------------------------------------------
defc toggle_drag_always_marks
   KeyPath = '\NEPMD\User\Mark\DragAlwaysMarks'
   on = QueryConfigKey( KeyPath)
   on = not on
   WriteConfigKey( KeyPath, on)
   SetMenuAttribute( 'dragalwaysmarks', MIA_CHECKED, not on)
  'mouse_init'  -- refresh the register_mousehandler defs

; ---------------------------------------------------------------------------
defc toggle_cursor_everywhere
   universal cursoreverywhere

   KeyPath = '\NEPMD\User\CursorPos\CursorEverywhere'
   on = QueryConfigKey( KeyPath)
   on = not on
   WriteConfigKey( KeyPath, on)
   -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
   SetMenuAttribute( 'cursoreverywhere',  MIA_CHECKED, not on)
   SetMenuAttribute( 'cursoreverywhere2', MIA_CHECKED, not on)
   cursoreverywhere = on

; ---------------------------------------------------------------------------
-- Max. word length of a command name: 28 chars
defc toggle_keep_cursor_on_screen
   KeyPath = '\NEPMD\User\CursorPos\KeepCursorOnScreen'
   on = QueryConfigKey( KeyPath)
   on = not on
   WriteConfigKey( KeyPath, on)
   -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
   SetMenuAttribute( 'keepcursoronscreen', MIA_CHECKED, not on)

; ---------------------------------------------------------------------------
defc toggle_highlight_cursor
   KeyPath = '\NEPMD\User\CursorPos\HighlightCursor'
   on = QueryConfigKey( KeyPath)
   on = not on
   WriteConfigKey( KeyPath, on)
   -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
   SetMenuAttribute( 'highlightcursor', MIA_CHECKED, not on)

; ---------------------------------------------------------------------------
defc toggle_home_toggles
   KeyPath = '\NEPMD\User\SpecialKeys\HomeToggles'
   on = QueryConfigKey( KeyPath)
   on = not on
   WriteConfigKey( KeyPath, on)
   -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
   SetMenuAttribute( 'hometoggles', MIA_CHECKED, not on)

; ---------------------------------------------------------------------------
defc toggle_end_toggles
   KeyPath = '\NEPMD\User\SpecialKeys\EndToggles'
   on = QueryConfigKey( KeyPath)
   on = not on
   WriteConfigKey( KeyPath, on)
   -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
   SetMenuAttribute( 'endtoggles', MIA_CHECKED, not on)

; ---------------------------------------------------------------------------
defc toggle_block_left_alt_key
   KeyPath = '\NEPMD\User\SpecialKeys\Alt\BlockLeftAlt'
   on = QueryConfigKey( KeyPath)
   on = not on
   WriteConfigKey( KeyPath, on)
   -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
   SetMenuAttribute( 'blockleftaltkey', MIA_CHECKED, not on)
   'RefreshBlockAlt'

; ---------------------------------------------------------------------------
defc toggle_block_right_alt_key
   KeyPath = '\NEPMD\User\SpecialKeys\Alt\BlockRightAlt'
   on = QueryConfigKey( KeyPath)
   on = not on
   WriteConfigKey( KeyPath, on)
   -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
   SetMenuAttribute( 'blockrightaltkey', MIA_CHECKED, not on)
   'RefreshBlockAlt'

; ---------------------------------------------------------------------------
defc toggle_matchtab
   universal menuloaded
   universal matchtab_on

   -- Change matchtab for current file
   matchtab_on = not matchtab_on
   'SetMatchTab' matchtab_on  -- refresh file settings
   if menuloaded then
      -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
      SetMenuAttribute( 'matchtab', MIA_CHECKED, not matchtab_on)
   endif

; ---------------------------------------------------------------------------
defc toggle_default_matchtab
   universal matchtab_on
   universal menuloaded

   KeyPath = '\NEPMD\User\SpecialKeys\Tab\MatchWordsAbove'
   on = QueryConfigKey( KeyPath)
   on = not on
   WriteConfigKey( KeyPath, on)
   -- Change match_tab for current file if it's default
   getfileid fid
   next = GetAVar( fid'.matchtab')  -- query file setting
   if next = 'DEFAULT' | next = '' then
      matchtab_on = on
      'RefreshInfoLine MATCHTAB'
   endif
   if menuloaded then  -- check not required
      -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
      SetMenuAttribute( 'defaultmatchtab', MIA_CHECKED, not on)
   endif

; ---------------------------------------------------------------------------
defc toggle_tabglyph
   on = tabglyph()
   on = not on
   call tabglyph( on)
   KeyPath = '\NEPMD\User\SpecialKeys\Tab\ShowTabs'
   WriteConfigKey( KeyPath, on)
   -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
   SetMenuAttribute( 'showtabs', MIA_CHECKED, not on)

; ---------------------------------------------------------------------------
defc toggle_dynaspell
   universal activeaccel

   on = (activeaccel = 'spell')
   on = not on
   'SetDynaspell' on
   -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
   SetMenuAttribute( 'autospellcheck', MIA_CHECKED, not on)

; ---------------------------------------------------------------------------
defc toggle_join_after_wrap
   universal join_after_wrap

   join_after_wrap = not join_after_wrap
   KeyPath = '\NEPMD\User\Format\Reflow\JoinAfterWrap'
   WriteConfigKey( KeyPath, join_after_wrap)
   -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
   SetMenuAttribute( 'joinafterwrap', MIA_CHECKED, not join_after_wrap)

; ---------------------------------------------------------------------------
defc toggle_respect_readonly
   KeyPath = '\NEPMD\User\Readonly'
   on = QueryConfigKey( KeyPath)
   on = not on
   WriteConfigKey( KeyPath, on)
   -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
   SetMenuAttribute( 'respectreadonly', MIA_CHECKED, not on)
   'ring enable_readonly 'on

; ---------------------------------------------------------------------------
defc toggle_lock_on_modify
   KeyPath = '\NEPMD\User\Lock\OnModify'
   on = QueryConfigKey( KeyPath)
   on = not on
   WriteConfigKey( KeyPath, on)
   -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
   SetMenuAttribute( 'lockonmodify', MIA_CHECKED, not on)
   'ring lock_on_modify 'on

; ---------------------------------------------------------------------------
defc toggle_nodismiss
   universal nodismiss
   universal defaultmenu

   KeyPath = '\NEPMD\User\Menu\NoDismiss'
   on = not (nodismiss = 32)
   WriteConfigKey( KeyPath, on)
   nodismiss = on * 32
   -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
   SetMenuAttribute( 'nodismiss', MIA_CHECKED, not on)
   deletemenu defaultmenu
   'loaddefaultmenu'
   call showmenu_activemenu()

/*
see also: CONFIG.E

OPTFLAGS:
   Bit              Setting
        for value = 1      for value = 0
   ---  ----------------   -------------------
    1   status bar on      status bar off
    2   msgline on         msgline off
    3   vscrollbar on      vscrollbar off
    4   hscrollbar on      hscrollbar off
    5   fileicon on        fileicon off        unused in EPM 6 (icon beside system menu icon)
    6   rotbuttons on      rotbuttons off
    7   info at top        info at bottom      pos of status + msg lines
    8   CUA marking        advanced marking
    9   menuprompt on      menuprompt off      menu hints on msg line
   10   stream mode        line mode
   11   longnames on       longnames off       show .LONGNAME EA instead of file name in titletext
   12   REXX profile on    REXX profile off
   13   escapekey on       escapekey off       ESC opens cmdbox
   14   tabkey on          tabkey off          1 = TAB inserts tab char
   15   bgbitmap on        bgbitmap off
   16   toolbar on         toolbar off
   17   dropstyle import   dropstyle edit      action for dropped file icon
   18   ?extra stuff on    ?extra stuff off    ?

OPT2FLAGS:
   Bit              Setting
        for value = 1      for value = 0
   ---  ----------------   -------------------
    1   I-beam pointer     arrow pointer       1 = (vepm_pointer = 2)
    2   underline cursor   bar cursor          1 = (cursordimensions = '-128.3 -128.-64')
*/

; ---------------------------------------------------------------------------
defc toggle_default_stream
   universal default_stream_mode
   universal stream_mode
   universal menuloaded

   default_stream_mode = not default_stream_mode
   KeyPath = '\NEPMD\User\StreamMode'
   call WriteConfigKey( KeyPath, default_stream_mode)
   -- Change stream_mode for current file if it's default
   getfileid fid
   next = GetAVar( fid'.streammode')  -- query file setting
   if next = 'DEFAULT' | next = '' then  -- unset if streammode was not changed by any modeexecute
      stream_mode = default_stream_mode
      'ToggleControl 24' stream_mode
      'RefreshInfoLine STREAMMODE'
   endif
   if menuloaded then
      -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
      SetMenuAttribute( 'defaultstreammode',  MIA_CHECKED, not default_stream_mode)
      SetMenuAttribute( 'defaultstreammode2', MIA_CHECKED, not default_stream_mode)
      MenuText_newline( 'newline')
      MenuText_newlinesplit( 'newline')
      MenuText_newlinecol( 'newline')
      MenuText_newline( 's_newline')
      MenuText_newlinesplit( 's_newline')
      MenuText_newlinecol( 's_newline')
      MenuText_newline( 'a_newline')
      MenuText_newlinesplit( 'a_newline')
      MenuText_newlinecol( 'a_newline')
      MenuText_newline( 'c_newline')
      MenuText_newlinesplit( 'c_newline')
      MenuText_newlinecol( 'c_newline')
   endif

; ---------------------------------------------------------------------------
; used, but empty definition
defc stack_toggle
/*
   universal stack_cmds
   universal activemenu
   universal defaultmenu
   stack_cmds = not stack_cmds
   deletemenu defaultmenu, GetAVar( 'mid_view'), 0, 1           -- Delete the view menu
   call add_view_menu( defaultmenu)
   call maybe_show_menu()
*/

; ---------------------------------------------------------------------------
; This changes the behavior of Alt+<letter> keys. When Advanced marking is
; activated, it's useful to diable them to activate the menu items, because
; their definitions are used for Advanced marking operations.
; Note: Even when Alt+<letter> is configured to block jumping to the menu,
; Sh+Alt+<letter> and Ctrl+Alt+<letter> keys still work as accelerator
; keys for the menu items.
defc toggle_accel, accel_toggle
   universal cua_menu_accel
   universal menuloaded

   cua_menu_accel = not cua_menu_accel
   KeyPath = '\NEPMD\User\SpecialKeys\Alt\BlockAltLetter'
   call WriteConfigKey( KeyPath, not cua_menu_accel)
   'ReloadKeyset'
   if menuloaded then
      -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
      SetMenuAttribute( 'blockaltletterkeys',  MIA_CHECKED, cua_menu_accel)
      SetMenuAttribute( 'blockaltletterkeys2', MIA_CHECKED, cua_menu_accel)
   endif

; ---------------------------------------------------------------------------
defc toggle_longname
   universal show_longnames
   universal menuloaded
   universal app_hini
   universal appname

   show_longnames = not show_longnames
   KeyPath = '\NEPMD\User\Appearance\InfoLine\Longname'
   call WriteConfigKey( KeyPath, show_longnames)
   if menuloaded then  -- check not required
      -- Set MIA_CHECKED attribute for the case MIA_NODISMISS attribute is on
      SetMenuAttribute( 'showlongname', MIA_CHECKED, not show_longnames)
      'RefreshInfoLine FILE'
   endif

; ---------------------------------------------------------------------------
defc toggle_use_newview
   universal menuloaded

   KeyPath = '\NEPMD\User\KeywordHelp\NewView\UseIfFound'
   on = (QueryConfigKey( KeyPath) <> 0)
   on = not on
   WriteConfigKey( KeyPath, on)
   if menuloaded then
      SetMenuAttribute( 'usenewview',        MIA_CHECKED, not on)
      SetMenuAttribute( 'usenewviewxsearch', MIA_DISABLED, on)
   endif

; ---------------------------------------------------------------------------
defc toggle_newview_xsearch
   universal menuloaded

   KeyPath = '\NEPMD\User\KeywordHelp\NewView\ExtendedSearch'
   on = (QueryConfigKey( KeyPath) <> 0)
   on = not on
   WriteConfigKey( KeyPath, on)
   if menuloaded then
      SetMenuAttribute( 'usenewviewxsearch', MIA_CHECKED, not on)
   endif

; ---------------------------------------------------------------------------
defc toggle_new_same_window
   KeyPath = '\NEPMD\User\ProgramObjects\OpenSameWindow'
   on = (QueryConfigKey( KeyPath) <> 0)
   on = not on
   WriteConfigKey( KeyPath, on)
   'rx newsamewindow.erx' on
   SetMenuAttribute( 'prgnewwindow', MIA_CHECKED, not on)

; ---------------------------------------------------------------------------
defc toggle_omit_openbox
   KeyPath = '\NEPMD\User\ProgramObjects\OmitOpenBox'
   on = (QueryConfigKey( KeyPath) <> 0)
   on = not on
   WriteConfigKey( KeyPath, on)
   'rx omitopenbox.erx' on
   SetMenuAttribute( 'prgomitopenbox', MIA_CHECKED, not on)

; ---------------------------------------------------------------------------
defc toggle_autosave
   universal menuloaded

   KeyPath = '\NEPMD\User\AutoSave'
   on = (QueryConfigKey( KeyPath) <> 0)
   on = not on
   WriteConfigKey( KeyPath, on)
   if menuloaded then
      SetMenuAttribute( 'autosaveenabled',   MIA_CHECKED, not on)
      SetMenuAttribute( 'autosavenumdialog', MIA_DISABLED, on)
   endif

; ---------------------------------------------------------------------------
defc toggle_backup
   universal menuloaded

   KeyPath = '\NEPMD\User\Backup'
   on = (QueryConfigKey( KeyPath) <> 0)
   on = not on
   WriteConfigKey( KeyPath, on)
   if menuloaded then
      SetMenuAttribute( 'backupenabled',   MIA_CHECKED, not on)
      SetMenuAttribute( 'backupnumdialog', MIA_DISABLED, on)
      SetMenuAttribute( 'backupdirdialog', MIA_DISABLED, on)
      SetMenuAttribute( 'backuplistdir',   MIA_DISABLED, on)
   endif

; ---------------------------------------------------------------------------
defc toggle_filename_completion
   universal menuloaded

   KeyPath = '\NEPMD\User\Shell\FilenameCompletion'
   on = (QueryConfigKey( KeyPath) <> 0)
   on = not on
   WriteConfigKey( KeyPath, on)
   if menuloaded then
      SetMenuAttribute( 'filenamecompletion', MIA_CHECKED, not on)
   endif

; ---------------------------------------------------------------------------
defc toggle_alias
   universal menuloaded

   KeyPath = '\NEPMD\User\Shell\Alias'
   on = (QueryConfigKey( KeyPath) <> 0)
   on = not on
   WriteConfigKey( KeyPath, on)
   if menuloaded then
      SetMenuAttribute( 'alias', MIA_CHECKED, not on)
   endif

; ---------------------------------------------------------------------------
defc SwitchEncodingTool
   universal codepage

   Tool = lowcase( strip( arg( 1)))
   'SetEncodingTool' Tool

   fUconvFound  = (GetAVar( 'tool.uconv') <> '')
   fRecodeFound = (GetAVar( 'tool.recode') <> '')

   on = (lowcase( GetEncodingTool()) <> 'recode')
   SetMenuAttribute( 'useuconv',  MIA_CHECKED, not on)
   on = not on
   SetMenuAttribute( 'userecode', MIA_CHECKED, not on)

   None = '-none-'
   new = GetEncodingCode()
   if new = '' then
      new = None
   endif
   old = GetStandardEncodingCode()
   if old = '' then
      old = None
   endif

   on = (new <> None) & (old <> None) & (fUconvFound | fRecodeFound)
   SetMenuAttribute( 'encodefrom', MIA_DISABLED, on)
   SetMenuAttribute( 'encodeto',   MIA_DISABLED, on)

; ---------------------------------------------------------------------------
defc SwitchEncoding
   universal codepage

   'EncodingNum SWITCH'

   fUconvFound  = (GetAVar( 'tool.uconv') <> '')
   fRecodeFound = (GetAVar( 'tool.recode') <> '')

   SetMenuVarText( 'switchencoding', GetEncodingName())
   None = '-none-'
   new = GetEncodingCode()
   if new = '' then
      new = None
   endif
   old = GetStandardEncodingCode()
   if old = '' then
      old = None
   endif

   on = (new <> None) & (old <> None) & (fUconvFound | fRecodeFound)
   SetMenuAttribute( 'encodefrom', MIA_DISABLED, on)
   SetMenuAttribute( 'encodeto',   MIA_DISABLED, on)

; ---------------------------------------------------------------------------
defc SwitchStandardEncoding
   universal codepage

   'StandardEncodingNum SWITCH'

   fUconvFound  = (GetAVar( 'tool.uconv') <> '')
   fRecodeFound = (GetAVar( 'tool.recode') <> '')

   SetMenuVarText( 'switchstandardencoding', GetStandardEncodingName())
   None = '-none-'
   new = GetEncodingCode()
   if new = '' then
      new = None
   endif
   old = GetStandardEncodingCode()
   if old = '' then
      old = None
   endif

   on = (new <> None) & (old <> None) & (fUconvFound | fRecodeFound)
   SetMenuAttribute( 'encodefrom', MIA_DISABLED, on)
   SetMenuAttribute( 'encodeto',   MIA_DISABLED, on)

; ---------------------------------------------------------------------------
defc ToggleEditEncodingCmd
   universal menuloaded

   KeyPath = '\NEPMD\User\Encoding\EditCmdBefore'
   on = (QueryConfigKey( KeyPath) <> 0)
   on = not on
   WriteConfigKey( KeyPath, on)
   if menuloaded then
      SetMenuAttribute( 'editencodingcmd', MIA_CHECKED, not on)
   endif

; ---------------------------------------------------------------------------
defc SwitchDict
   None = '-none-'
   'DictNum SWITCH'
   new = GetDictBaseName()
   if new = '' then
      new = None
   endif
   SetMenuVarText( 'switchdict', new)

; ---------------------------------------------------------------------------
defc ChangeWorkDirMenuCmd
   opt = arg( 1)
   if wordpos( opt, '0 1 2') = 0 then
      return
   endif
   KeyPath = '\NEPMD\User\StartDir\WorkDir\Type'
   WriteConfigKey( KeyPath, opt)
   -- Query type and change to dir of type 0, 1 or 2
   'ChangeWorkDirProgram'
   'ChangeWorkDirLast'
   'ChangeWorkDirSelected'
   SetMenuAttribute( 'workdirprogram', MIA_CHECKED, (opt = 1 | opt = 2))
   SetMenuAttribute( 'workdirprev',    MIA_CHECKED, not (opt = 1))
   SetMenuAttribute( 'workdirfile',    MIA_CHECKED, not (opt = 2))

; ---------------------------------------------------------------------------
defc OpenDlgDirMenuCmd
   opt = arg( 1)
   if wordpos( opt, '0 1 2') = 0 then
      return
   endif
   KeyPath = '\NEPMD\User\StartDir\OpenDlg\Type'
   WriteConfigKey( KeyPath, opt)
   SetMenuAttribute( 'opendlgdirprev', MIA_CHECKED, (opt = 1 | opt = 2))
   SetMenuAttribute( 'opendlgdirwork', MIA_CHECKED, not (opt = 1))
   SetMenuAttribute( 'opendlgdirfile', MIA_CHECKED, not (opt = 2))

; ---------------------------------------------------------------------------
defc SaveasDlgDirMenuCmd
   opt = arg( 1)
   if wordpos( opt, '0 1') = 0 then
      return
   endif
   KeyPath = '\NEPMD\User\StartDir\SaveasDlg\Type'
   WriteConfigKey( KeyPath, opt)
   SetMenuAttribute( 'saveasdlgdirprev', MIA_CHECKED, (opt = 1))
   SetMenuAttribute( 'saveasdlgdirwork', MIA_CHECKED, not (opt = 1))

; ---------------------------------------------------------------------------
defc FindDefSearchTypeMenuCmd
   universal finddefexact
   universal finddefprototypes
   universal finddefcomlit
   universal finddefallmatches
   universal finddefrecursion

   Type = upcase( arg( 1))
   'SetFindDefSearchType 'Type

   SetMenuAttribute( 'finddefincludes',   MIA_CHECKED, not (Type = 'I'))
   SetMenuAttribute( 'finddeffilemasks',  MIA_CHECKED, not (Type = 'F'))
   SetMenuAttribute( 'finddefgrep',       MIA_CHECKED, not (Type = 'G'))
   SetMenuAttribute( 'finddefwords',      MIA_CHECKED, (wordpos( Type, 'I F G') > 0))

   SetMenuAttribute( 'finddefexact',      MIA_CHECKED, not finddefexact)
   SetMenuAttribute( 'finddefprototypes', MIA_CHECKED, not finddefprototypes)
   SetMenuAttribute( 'finddefcomlit',     MIA_CHECKED, not finddefcomlit)
   SetMenuAttribute( 'finddefallmatches', MIA_CHECKED, not finddefallmatches)
   SetMenuAttribute( 'finddefrecursion',  MIA_CHECKED, not finddefrecursion)

   SetMenuAttribute( 'finddefexact',      MIA_DISABLED, GetFindDefOptionDefined( 'finddefexact'))
   SetMenuAttribute( 'finddefprototypes', MIA_DISABLED, GetFindDefOptionDefined( 'finddefprototypes'))
   SetMenuAttribute( 'finddefcomlit',     MIA_DISABLED, GetFindDefOptionDefined( 'finddefcomlit'))
   SetMenuAttribute( 'finddefallmatches', MIA_DISABLED, GetFindDefOptionDefined( 'finddefallmatches'))
   SetMenuAttribute( 'finddefrecursion',  MIA_DISABLED, GetFindDefOptionDefined( 'finddefrecursion'))

; ---------------------------------------------------------------------------
defc ToggleFindDefExact
   universal finddefexact

   'SetFindDefExact 'not finddefexact
   SetMenuAttribute( 'finddefexact',      MIA_CHECKED, not finddefexact)

; ---------------------------------------------------------------------------
defc ToggleFindDefPrototypes
   universal finddefprototypes

   'SetFindDefPrototypes 'not finddefprototypes
   SetMenuAttribute( 'finddefprototypes', MIA_CHECKED, not finddefprototypes)

; ---------------------------------------------------------------------------
defc ToggleFindDefComLit
   universal finddefcomlit

   'SetFindDefComLit 'not finddefcomlit
   SetMenuAttribute( 'finddefcomlit',     MIA_CHECKED, not finddefcomlit)

; ---------------------------------------------------------------------------
defc ToggleFindDefAllMatches
   universal finddefallmatches

   'SetFindDefAllMatches 'not finddefallmatches
   SetMenuAttribute( 'finddefallmatches', MIA_CHECKED, not finddefallmatches)

; ---------------------------------------------------------------------------
defc ToggleFindDefRecursion
   universal finddefrecursion

   finddefrecursion = not finddefrecursion
   SetMenuAttribute( 'finddefrecursion', MIA_CHECKED, not finddefrecursion)

; ---------------------------------------------------------------------------
defproc GetPresetBitStr
   universal cua_marking_switch
   universal default_stream_mode
   universal cua_menu_accel

   fAvancedMarking   = not cua_marking_switch

   fCuaKeyset        = (lowcase( GetAddKeyDefs()) = 'cua')

   fStreamMode       = default_stream_mode

   KeyPath = '\NEPMD\User\CursorPos\CursorEverywhere'
   fCursorEverywhere = (QueryConfigKey( KeyPath) <> 0)

   KeyPath1 = '\NEPMD\User\SpecialKeys\HomeToggles'
   KeyPath2 = '\NEPMD\User\SpecialKeys\EndToggles'
   fHomeEndToggle    = ((QueryConfigKey( KeyPath1) = 1) &
      (QueryConfigKey( KeyPath2) = 1))

   fBlockAltLetter   = not cua_menu_accel

   KeyPath = '\NEPMD\User\SyntaxExpansion'
   fSyntaxExpansion  = QueryConfigKey( KeyPath)

   KeyPath1 = '\NEPMD\User\SpecialKeys\MatchFindOnMove'
   KeyPath2 = '\NEPMD\User\SpecialKeys\MatchFindOpening'
   KeyPath3 = '\NEPMD\User\SpecialKeys\MatchInsertPair'
   fMatchBrackets    = (QueryConfigKey( KeyPath1) &
      QueryConfigKey( KeyPath2) & QueryConfigKey( KeyPath3))

   fModeSpecific     = (QueryModeSpecific( 'TABS') &
      QueryModeSpecific( 'MARGINS'))

   BitStr = ''          ||
      fAvancedMarking   ||
      fCuaKeyset        ||
      fStreamMode       ||
      fCursorEverywhere ||
      fHomeEndToggle    ||
      fBlockAltLetter   ||
      fSyntaxExpansion  ||
      fMatchBrackets    ||
      fModeSpecific

   return BitStr

; ---------------------------------------------------------------------------
defc ApplyPresetBitStr
   universal cua_marking_switch
   universal default_stream_mode
   universal cua_menu_accel

   do once = 1 to 1
      -- Query BitStr
      BitStr = strip( arg( 1))
      -- Check BitStr
      if length( BitStr) <> 9 then
         leave
      elseif (verify( BitStr, '01') <> 0) then
         leave
      endif
      -- Parse BitStr
      fAvancedMarking   = substr( BitStr, 1, 1)
      fCuaKeyset        = substr( BitStr, 2, 1)
      fStreamMode       = substr( BitStr, 3, 1)
      fCursorEverywhere = substr( BitStr, 4, 1)
      fHomeEndToggle    = substr( BitStr, 5, 1)
      fBlockAltLetter   = substr( BitStr, 6, 1)
      fSyntaxExpansion  = substr( BitStr, 7, 1)
      fMatchBrackets    = substr( BitStr, 8, 1)
      fModeSpecific     = substr( BitStr, 9, 1)

      -- Apply bits
      if fAvancedMarking <> (not cua_marking_switch) then
         'toggle_cua_mark'
      endif

      if fCuaKeyset <> (lowcase( GetAddKeyDefs()) = 'cua') then
         'toggle_AddKeyDefs cuakeys'
      endif

      if fStreamMode <> default_stream_mode then
         'toggle_default_stream'
      endif

      KeyPath = '\NEPMD\User\CursorPos\CursorEverywhere'
      if fCursorEverywhere <> (QueryConfigKey( KeyPath) <> 0) then
         'toggle_cursor_everywhere'
      endif

      KeyPath1 = '\NEPMD\User\SpecialKeys\HomeToggles'
      KeyPath2 = '\NEPMD\User\SpecialKeys\EndToggles'
      if fCursorEverywhere <> ((QueryConfigKey( KeyPath1) = 1) &
         (QueryConfigKey( KeyPath2) = 1)) then
         'toggle_HomeEndToggle'
      endif

      if fBlockAltLetter <> (not cua_menu_accel) then
         'toggle_Accel'
      endif

      KeyPath = '\NEPMD\User\SyntaxExpansion'
      if fSyntaxExpansion <> QueryConfigKey( KeyPath) then
         'toggle_default_expand'
      endif

      KeyPath1 = '\NEPMD\User\SpecialKeys\MatchFindOnMove'
      KeyPath2 = '\NEPMD\User\SpecialKeys\MatchFindOpening'
      KeyPath3 = '\NEPMD\User\SpecialKeys\MatchInsertPair'
      if fMatchBrackets <> (QueryConfigKey( KeyPath1) &
         QueryConfigKey( KeyPath2) & QueryConfigKey( KeyPath3)) then
         'toggle_MatchBrackets'
      endif

      if fModeSpecific <> (QueryModeSpecific( 'TABS') &
         QueryModeSpecific( 'MARGINS')) then
         'toggle_ModeSpecificSettings'
      endif

      SetMenuAttribute( 'presetusercua', MIA_CHECKED, not (BitStr = PRESET_BITSTR_USER_CUA))
      SetMenuAttribute( 'presetprogcua', MIA_CHECKED, not (BitStr = PRESET_BITSTR_PROG_CUA))
      SetMenuAttribute( 'presetprogstd', MIA_CHECKED, not (BitStr = PRESET_BITSTR_PROG_STD))

   enddo

; ---------------------------------------------------------------------------
defc TogglePresetBit
   do once = 1 to 1
      Bit = strip( arg( 1))
      if not IsUInt( Bit) then
         leave
      endif

      if Bit = 1 then
         'toggle_cua_mark'
      elseif Bit = 2 then
         'toggle_AddKeyDefs cuakeys'
      elseif Bit = 3 then
         'toggle_default_stream'
      elseif Bit = 4 then
         'toggle_cursor_everywhere'
      elseif Bit = 5 then
         'toggle_HomeEndToggle'
      elseif Bit = 6 then
         'toggle_Accel'
      elseif Bit = 7 then
         'toggle_default_expand'
      elseif Bit = 8 then
         'toggle_MatchBrackets'
      elseif Bit = 9 then
         'toggle_ModeSpecificSettings'
      endif

      BitStr = GetPresetBitStr()
      SetMenuAttribute( 'presetusercua', MIA_CHECKED, not (BitStr = PRESET_BITSTR_USER_CUA))
      SetMenuAttribute( 'presetprogcua', MIA_CHECKED, not (BitStr = PRESET_BITSTR_PROG_CUA))
      SetMenuAttribute( 'presetprogstd', MIA_CHECKED, not (BitStr = PRESET_BITSTR_PROG_STD))
   enddo

; ---------------------------------------------------------------------------
; Change edit options and set menu attributes.
; Some options exclude each other, see ExcludeList. The last option wins.
; Flags: universal, nepmd.ini
defc EditOptionsMenuCmd
   universal default_edit_options

   opt = arg( 1)
   KeyPath = '\NEPMD\User\Edit\DefaultOptions'
   if opt = 'SAVE' then
      ConfigValue = default_edit_options
      WriteConfigKey( KeyPath, ConfigValue)
      if not rc then
         'SayError default_edit_options = 'default_edit_options' saved.'
      endif
      return
   elseif opt = 'RESET' then
      DeleteConfigKey( KeyPath)
      ConfigValue = QueryConfigKey( KeyPath)
      default_edit_options = ConfigValue
      if not rc then
         'SayError default_edit_options = 'default_edit_options' saved.'
      endif
      -- Call again without opt to refresh the check marks and the variable text
      'EditOptionsMenuCmd'
      return
   elseif opt = 'MENUINIT' then
;      opt = '/b /t /u 'default_edit_options  -- internal default + default_edit_options
      opt = '/b /t 'default_edit_options  -- internal default + default_edit_options
   endif
   opt = lowcase( opt)
   -- Remove multiple and excluding options and spaces
;   ExcludeList = '/b/c/d /t/nt /l/u'    -- for every word in this list: every expression excludes each other
   ExcludeList = '/b/c/d /t/nt'    -- for every word in this list: every expression excludes each other
   rest = lowcase( default_edit_options' 'opt)
   rest = translate( rest, ' ', '/')  -- replace all slashes with spaces to use the word* procs
   newopt = ''
   do while rest <> ''
      parse value rest with next rest
      next = strip( next)
      --call NepmdPmPrintf( '*** next = |'next'|')
      if wordpos( next, rest) = 0 then   -- if not found in rest
         -- Find excluding options
         ExcludeWrd = ''
         do w = 1 to words( ExcludeList)
            wrd = word( ExcludeList, w)
            wrd = translate( wrd, ' ', '/')  -- replace all slashes with spaces to use the word* procs
            if wordpos( next, wrd) then
               ExcludeWrd = wrd          -- ExcludeWrd = word of ExcludeList where next belongs to, slashes are replaced
               leave
            endif
         enddo
         --call NepmdPmPrintf( '*** ExcludeWrd = |'ExcludeWrd'|')
         -- Check if rest contains options of ExcludeWrd
         Found = 0
         do x = 1 to words( ExcludeWrd)
            xopt = word( ExcludeWrd, x)
            if wordpos( xopt, rest) then
               Found = 1
               leave
            endif
         enddo
         if not Found then  -- if rest doesn't contain options of ExcludeWrd
            newopt = newopt' /'next      -- append '/'next
            -- Set MIA_CHECKED attribute for all options listed in ExcludeWrd
            do x = 1 to words( ExcludeWrd)
               xopt = word( ExcludeWrd, x)  -- single option of ExcludeWrd without the slash
               name = lowcase( xopt)
               --call NepmdPmPrintf( 'opt = 'arg( 1)', name = |'name'|, mid = 'GetAVar( 'mid_editoptions_'name))
               SetMenuAttribute( GetAVar( 'mid_editoptions_'name), MIA_CHECKED, next <> xopt)
            enddo
         endif
      endif
   enddo
   newopt = strip( newopt)
   default_edit_options = newopt
   if isadefproc( 'MenuText_editoptions') then
      MenuText_editoptions()
   endif
   --call NepmdPmPrintf( 'opt = 'arg( 1)', default_edit_options = 'default_edit_options)
   --'SayError default_edit_options = 'default_edit_options

; ---------------------------------------------------------------------------
; Change save options and set menu attributes.
; Some options exclude each other, see ExcludeList. The last option wins.
defc SaveOptionsMenuCmd
   universal default_save_options

   opt = arg( 1)
   KeyPath = '\NEPMD\User\Save\DefaultOptions'
   if opt = 'SAVE' then
      ConfigValue = default_save_options
      WriteConfigKey( KeyPath, ConfigValue)
      if not rc then
         'SayError default_save_options = 'default_save_options' saved.'
      endif
      return
   elseif opt = 'RESET' then
      DeleteConfigKey( KeyPath)
      ConfigValue = QueryConfigKey( KeyPath)
      default_save_options = ConfigValue
      if not rc then
         'SayError default_save_options = 'default_save_options' saved.'
      endif
      -- Call again without opt to refresh the check marks and the variable text
      'SaveOptionsMenuCmd'
      return
   elseif opt = 'MENUINIT' then
      opt = '/a /ns /ne /nt 'default_save_options  -- internal default + default_save_options
   endif
   opt = lowcase( opt)
   -- Replace option /u with /l /ne
   wp = ''
   do while wp <> 0
      wp = wordpos( '/u', opt)
      if wp > 0 then
         opt = subword( opt, 1, wp - 1)' /l /ne 'subword( opt, wp + 1)
      endif
   enddo
   -- Remove multiple and excluding options and spaces
   ExcludeList = '/s/ns /e/ne /t/nt /o/l/a'    -- for every word in this list: every expression excludes each other
   rest = lowcase( default_save_options' 'opt)
   rest = translate( rest, ' ', '/')  -- replace all slashes with spaces to use the word* procs
   newopt = ''
   do while rest <> ''
      parse value rest with next rest
      next = strip( next)
      --call NepmdPmPrintf( '*** next = |'next'|')
      if wordpos( next, rest) = 0 then   -- if not found in rest
         -- Find excluding options
         ExcludeWrd = ''
         do w = 1 to words( ExcludeList)
            wrd = word( ExcludeList, w)
            wrd = translate( wrd, ' ', '/')  -- replace all slashes with spaces to use the word* procs
            if wordpos( next, wrd) then
               ExcludeWrd = wrd          -- ExcludeWrd = word of ExcludeList where next belongs to, slashes are replaced
               leave
            endif
         enddo
         --call NepmdPmPrintf( '*** ExcludeWrd = |'ExcludeWrd'|')
         -- Check if rest contains options of ExcludeWrd
         Found = 0
         do x = 1 to words( ExcludeWrd)
            xopt = word( ExcludeWrd, x)
            if wordpos( xopt, rest) then
               Found = 1
               leave
            endif
         enddo
         if not Found then  -- if rest doesn't contain options of ExcludeWrd
            newopt = newopt' /'next      -- append '/'next
            -- Set MIA_CHECKED attribute for all options listed in ExcludeWrd
            do x = 1 to words( ExcludeWrd)
               xopt = word( ExcludeWrd, x)  -- single option of ExcludeWrd without the slash
               name = lowcase( xopt)
               --call NepmdPmPrintf( 'opt = 'arg( 1)', name = |'name'|, mid = 'GetAVar( 'mid_saveoptions_'name))
               SetMenuAttribute( GetAVar( 'mid_saveoptions_'name), MIA_CHECKED, next <> xopt)
            enddo
         endif
      endif
   enddo
   newopt = strip( newopt)
   -- Remove non-standard options
   --call NepmdPmPrintf( 'opt = 'arg( 1)', newopt = 'newopt)
   rest = newopt
   newopt = ''
   do w = 1 to words( rest)
      wrd = word( rest, w)
      if wrd <> '/a' then
         newopt = newopt' 'wrd
      endif
   enddo
   newopt = strip( newopt)
   default_save_options = newopt
   if isadefproc( 'MenuText_saveoptions') then
      MenuText_saveoptions()
   endif
   --call NepmdPmPrintf( 'opt = 'arg( 1)', default_save_options = 'default_save_options)
   --'SayError default_save_options = 'default_save_options

; ---------------------------------------------------------------------------
; Change search options and set menu attributes.
; Some options exclude each other, see ExcludeList. The last option wins.
defc SearchOptionsMenuCmd
   universal default_search_options

   opt = arg( 1)
   KeyPath = '\NEPMD\User\Search\DefaultOptions'
   if opt = 'SAVE' then
      ConfigValue = default_search_options
      WriteConfigKey( KeyPath, ConfigValue)
      if not rc then
          'SayError default_search_options = 'default_search_options' saved.'
      endif
      return
   elseif opt = 'RESET' then
      DeleteConfigKey( KeyPath)
      ConfigValue = QueryConfigKey( KeyPath)
      default_search_options = ConfigValue
      if not rc then
          'SayError default_search_options = 'default_search_options' saved.'
      endif
      -- Call again without opt to refresh the check marks and the variable text
      'SearchOptionsMenuCmd'
      return
   elseif opt = 'MENUINIT' then
      opt = '+faeuihp1q'default_search_options  -- internal default + default_search_options
   endif
   opt = lowcase( opt)
   -- Remove multiple and excluding options and spaces
   ExcludeList = '+- fr ubt am eco i^ hgxw pk 1* nq'    -- for every word in this list: every char excludes each other
   rest = lowcase( default_search_options''opt)
   newopt = ''
   do while rest <> ''
      parse value rest with next 2 rest  -- parse 1 char of rest
      -- Remove all spaces
      if next = ' ' then
         iterate
      elseif pos( next, rest) = 0 then   -- if not found in rest
         -- Find excluding options
         ExcludeWrd = ''
         do w = 1 to words( ExcludeList)
            wrd = word( ExcludeList, w)
            if pos( next, wrd) then
               ExcludeWrd = wrd          -- ExcludeWrd = word of ExcludeList where next belongs to
               leave
            endif
         enddo
         if not verify( rest, ExcludeWrd, 'M') then  -- if rest doesn't contain chars of ExcludeWrd
            newopt = newopt''next      -- append next
            -- Set MIA_CHECKED attribute for all options listed in ExcludeWrd
            do x = 1 to length( ExcludeWrd)
               xopt = substr( ExcludeWrd, x, 1)  -- single option of ExcludeWrd
               name = lowcase( xopt)
               --call NepmdPmPrintf( 'opt = 'arg( 1)', name = 'name)
               SetMenuAttribute( GetAVar( 'mid_searchoptions_'name), MIA_CHECKED, next <> xopt)
            enddo
         endif
      endif
   enddo
   -- Remove non-standard options
   rest = newopt
   newopt = ''
   do while rest <> ''
      parse value rest with next 2 rest  -- parse 1 char of rest
      if pos( next, 'uihp1q') = 0 then
         newopt = newopt''next
      endif
   enddo
   default_search_options = newopt
   if isadefproc( 'MenuText_searchoptions') then
      MenuText_searchoptions()
   endif

   -- Change search dialog check boxes
   SearchDlgOptions( opt)

   --call NepmdPmPrintf( 'opt = 'arg( 1)', default_search_options = 'default_search_options)
   --'SayError default_search_options = 'default_search_options

; ---------------------------------------------------------------------------
; Nothing defined here.
defproc build_menu_accelerators( activeaccel)
   -- Get the last used id from an array var
   i = GetAVar( 'lastkeyaccelid')

   --i = i + 1
   --buildacceltable activeaccel, 'dokey ...',  AF_... + AF_..., VK_..., i

   -- Save the last used id in an array var
   call SetAVar( 'lastkeyaccelid', i)
   return

